/*
<DEPENDENCIES>
    <DEPENDS>Globals</DEPENDS>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function Snapshot( parent, snapId, htmlElement, left, top, width, height, right, bottom, parentLeft, parentTop )
{
  this.parent = parent;
  this.snapId = snapId;
  this.element = htmlElement;
  if( arguments.length == 3 )
  {
    var pos = new Point(0, 0);
    Browser.getAbsPos(htmlElement, pos);
    this.left = pos.x;
    this.top = pos.y;
    this.width = htmlElement.offsetWidth;
    this.height = htmlElement.offsetHeight;
    this.parentPos = new Point(0, 0);
    if( htmlElement.offsetParent )
    {
      Browser.getAbsPos(htmlElement.offsetParent, this.parentPos);
    }
    this.right = Browser.getPixelComputedStyle(htmlElement, 'right');
    if( this.right == 0 )
    {
      this.right = this.left + this.width;
    }
    this.bottom = Browser.getPixelComputedStyle(htmlElement, 'bottom');
    if( this.bottom == 0 )
    {
      this.bottom = this.top + this.height;
    }
    //    ;
  }
  else if( arguments.length == 7 )
  {
    this.left = left;
    this.top = top;
    this.width = width;
    this.height = height;
    this.right = left + width;
    this.bottom = top + height;
  }
  else if( arguments.length == 9 )
  {
    this.left = left;
    this.top = top;
    this.width = width;
    this.height = height;
    this.right = right;
    this.bottom = bottom;
  }
  else if( arguments.length == 11 )
  {
    this.left = left;
    this.top = top;
    this.width = width;
    this.height = height;
    this.right = right;
    this.bottom = bottom;
    this.parentPos = new Point(parentLeft, parentTop);
  }
}
Snapshot.prototype.clone = function()
{
  return new Snapshot(this.parent, this.snapId, this.element,
          this.left, this.top, this.width, this.height, this.right, this.bottom, this.parentPos.x, this.parentPos.y);
}
Snapshot.prototype.copyNeutral = function()
{
  return new Snapshot(this.parent, this.snapId, this.element, 0, 0, 0, 0, 0, 0, 0, 0);
}
Snapshot.prototype.copyDimensionNeutral = function()
{
  return new Snapshot(this.parent, this.snapId, this.element, this.left, this.top, 0, 0, this.right, this.bottom,
          this.parentPos.x, this.parentPos.y);
}
Snapshot.prototype.delta = function( left, top, width, height )
{
  return new Snapshot(this.parent, this.snapId, this.element,
          left - this.left, top - this.top, width - this.width, height - this.height,
          left - this.left + width - this.width, top - this.top + height - this.height,
          this.parentPos.x, this.parentPos.y);
}
Snapshot.prototype.add = function( deltaSnapshot )
{
  if( deltaSnapshot )
  {
    this.left += deltaSnapshot.left;
    this.top += deltaSnapshot.top;
    this.width += deltaSnapshot.width;
    this.height += deltaSnapshot.height;
    this.right += deltaSnapshot.right;
    this.bottom += deltaSnapshot.bottom;
  }
  return this;
}
Snapshot.prototype.addPosition = function( deltaSnapshot )
{
  if( deltaSnapshot )
  {
    this.left += deltaSnapshot.left;
    this.top += deltaSnapshot.top;
    this.right += deltaSnapshot.right;
    this.bottom += deltaSnapshot.bottom;
  }
  return this;
}
Snapshot.prototype.addDimension = function( deltaSnapshot, originNameX, originNameY )
{
  if( deltaSnapshot )
  {
    this.width += deltaSnapshot.width;
    this.height += deltaSnapshot.height;
    if( originNameX == 'right' )
      this.left = this.right - this.width;
    else
      this.right = this.left + this.width;
    if( originNameY == 'bottom' )
      this.top = this.bottom - this.height;
    else
      this.bottom = this.top + this.height;
  }
  return this;
}
Snapshot.prototype.getCssLeft = function()
{
  return this.left - this.parentPos.x;
}
Snapshot.prototype.getCssTop = function()
{
  return this.top - this.parentPos.y;
}
Snapshot.prototype.toString = function()
{
  return 'Snapshot{ parent=' + this.parent.getId()
          + ', snapId=' + this.snapId
          + ', element={ name=' + this.element.tagName
          + ', id=' + this.element.id
          + ', class=' + this.element.className + '}'
          + ', left=' + this.left + ', top=' + this.top
          + ', width=' + this.width + ', height=' + this.height + ' }';
}
Snapshot.elementToString = function( element )
{
  if( element )
    return 'element={ name=' + element.tagName + ', id=' + element.id + ', class=' + element.className + '}';
  else
    return 'element=NULL';
}
Snapshot.prototype.debug = function( msg )
{
  //  ;
}
function Delta( x, y )
{
  if( x )
    this.x = x;
  else
    this.x = 0;
  if( y )
    this.y = y;
  else
    this.y = 0;
}
Delta.prototype.clone = function()
{
  return new Delta(this.x, this.y);
}
Delta.prototype.toString = function()
{
  return 'Delta{ x=' + this.x + ', y=' + this.y + ' }';
}
function Alteration( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
Alteration.prototype.init = function( sId, lowerBound, upperBound )
{
  this.sId = sId;
  this.lowerBound = lowerBound;
  this.upperBound = upperBound;
  this.eLowerBoundSetHandles = new Array();
  this.eUpperBoundSetHandles = new Array();
}
Alteration.prototype.setLowerBound = function( lowerBound )
{
  this.lowerBound = lowerBound;
}
Alteration.prototype.getSetLowerBoundAction = function()
{
  return new ObjectMethodAction(this, "setLowerBound");
}
Alteration.prototype.addLowerBoundSetHandle = function( action )
{
  if( action )
    this.eLowerBoundSetHandles.push(action);
}
Alteration.prototype.getLowerBound = function()
{
  return this.lowerBound;
}
Alteration.prototype.getGetLowerBoundAction = function()
{
  return new ObjectMethodAction(this, "getLowerBound");
}
Alteration.prototype.setUpperBound = function( upperBound )
{
  this.upperBound = upperBound;
}
Alteration.prototype.getSetUpperBoundAction = function()
{
  return new ObjectMethodAction(this, "setUpperBound");
}
Alteration.prototype.addUpperBoundSetHandle = function( action )
{
  if( action )
    this.eUpperBoundSetHandles.push(action);
}
Alteration.prototype.getUpperBound = function()
{
  return this.upperBound;
}
Alteration.prototype.getGetUpperBoundAction = function()
{
  return new ObjectMethodAction(this, "getUpperBound");
}
Alteration.prototype.getId = function()
{
  return this.sId;
}
Alteration.prototype.isFeasibleValue = function( value )
{
  var isLowerBoundOK = ( this.lowerBound ? this.lowerBound <= value : true );
  var isUpperBoundOK = ( this.upperBound ? value <= this.upperBound : true );
  return isLowerBoundOK && isUpperBoundOK;
}
Alteration.prototype.maxFeasibleOffset = function( value, offset )
{
  if( !this.isFeasibleValue(value) )
  {
    return 0;
  }
  else if( this.isFeasibleValue(value + offset) )
  {
    return offset;
  }
  else
  {
    if( offset > 0 )
    {
      return this.upperBound - value;
    }
    else if( offset < 0 )
    {
      return this.lowerBound - value;
    }
    else
    {
      return 0;
    }
  }
}
Alteration.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(0, 0);
}
Alteration.prototype.reset = function( snapshot )
{
}
Alteration.prototype.alter = function( snapshot, delta )
{
  // ;
  if( !this.isFeasible(snapshot, delta) )
    this.reset(snapshot);
}
Alteration.prototype.alterAdequate = function( snapshot, delta )
{
  return this.alter(snapshot, this.getFeasibleDelta(snapshot, delta));
}
AlterationLeft.prototype = new Alteration();
AlterationLeft.prototype.constructor = AlterationLeft;
AlterationLeft.superclass = Alteration.prototype;
function AlterationLeft( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationLeft.prototype.init = function( sId, lowerBound, upperBound )
{
  ;
  AlterationLeft.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationLeft.prototype.isFeasible = function( snapshot, delta )
{
  // ;
  return this.isFeasibleValue(snapshot.left + delta.x);
}
AlterationLeft.prototype.getFeasibleDelta = function( snapshot, delta )
{
  // ;
  return new Delta(this.maxFeasibleOffset(snapshot.left, delta.x), delta.y);
}
AlterationLeft.prototype.reset = function( snapshot )
{
  // ;
  Browser.changePixelStyle(snapshot.element, 'left', snapshot.getCssLeft());
}
AlterationLeft.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    // ;
    Browser.changePixelStyle(snapshot.element, 'left', snapshot.getCssLeft() + delta.x);
    return snapshot.delta(snapshot.left + delta.x, snapshot.top, snapshot.width, snapshot.height);
  }
}
AlterationTop.prototype = new Alteration();
AlterationTop.prototype.constructor = AlterationTop;
AlterationTop.superclass = Alteration.prototype;
function AlterationTop( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationTop.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationTop.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationTop.prototype.isFeasible = function( snapshot, delta )
{
  var result = this.isFeasibleValue(snapshot.top + delta.y)
  return result;
}
AlterationTop.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(delta.x, this.maxFeasibleOffset(snapshot.top, delta.y));
}
AlterationTop.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'top', snapshot.getCssTop());
}
AlterationTop.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    //    ;
    Browser.changePixelStyle(snapshot.element, 'top', snapshot.getCssTop() + delta.y);
    return snapshot.delta(snapshot.left, snapshot.top + delta.y, snapshot.width, snapshot.height);
  }
  //    ;
}
AlterationRight.prototype = new Alteration();
AlterationRight.prototype.constructor = AlterationRight;
AlterationRight.superclass = Alteration.prototype;
function AlterationRight( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationRight.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationRight.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationRight.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.right + delta.x);
}
AlterationRight.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(this.maxFeasibleOffset(snapshot.right, delta.x), delta.y);
}
AlterationRight.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'right', snapshot.right);
}
AlterationRight.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    Browser.changePixelStyle(snapshot.element, 'right', snapshot.right + delta.x);
    return snapshot.delta(snapshot.left + delta.x, snapshot.top, snapshot.width, snapshot.height);
  }
}
AlterationBottom.prototype = new Alteration();
AlterationBottom.prototype.constructor = AlterationBottom;
AlterationBottom.superclass = Alteration.prototype;
function AlterationBottom( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationBottom.prototype.init = function( sId, lowerBound, upperBound )
{
  ;
  AlterationBottom.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationBottom.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.top + delta.y);
}
AlterationBottom.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(delta.x, this.maxFeasibleOffset(snapshot.top, delta.y));
}
AlterationBottom.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'bottom', snapshot.bottom);
}
AlterationBottom.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    Browser.changePixelStyle(snapshot.element, 'bottom', snapshot.bottom + delta.y);
    return snapshot.delta(snapshot.left, snapshot.top + delta.y, snapshot.width, snapshot.height);
  }
}
AlterationWidth.prototype = new Alteration();
AlterationWidth.prototype.constructor = AlterationWidth;
AlterationWidth.superclass = Alteration.prototype;
function AlterationWidth( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationWidth.prototype.init = function( sId, lowerBound, upperBound )
{
  ;
  AlterationWidth.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationWidth.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.width + delta.x);
}
AlterationWidth.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(this.maxFeasibleOffset(snapshot.width, delta.x), delta.y);
}
AlterationWidth.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'width', snapshot.width);
}
AlterationWidth.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    Browser.changePixelStyle(snapshot.element, 'width', snapshot.width + delta.x);
    return snapshot.delta(snapshot.left, snapshot.top, snapshot.width + delta.x, snapshot.height);
  }
}
AlterationHeight.prototype = new Alteration();
AlterationHeight.prototype.constructor = AlterationHeight;
AlterationHeight.superclass = Alteration.prototype;
function AlterationHeight( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationHeight.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationHeight.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationHeight.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.height + delta.y);
}
AlterationHeight.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(delta.x, this.maxFeasibleOffset(snapshot.height, delta.y));
}
AlterationHeight.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'height', snapshot.height);
}
AlterationHeight.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    Browser.changePixelStyle(snapshot.element, 'height', snapshot.height + delta.y);
    return snapshot.delta(snapshot.left, snapshot.top, snapshot.width, snapshot.height + delta.y);
  }
}
AlterationLeftInverse.prototype = new Alteration();
AlterationLeftInverse.prototype.constructor = AlterationLeftInverse;
AlterationLeftInverse.superclass = Alteration.prototype;
function AlterationLeftInverse( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationLeftInverse.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationLeftInverse.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationLeftInverse.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.left - delta.x);
}
AlterationLeftInverse.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(-this.maxFeasibleOffset(snapshot.left, -delta.x), delta.y);
}
AlterationLeftInverse.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'left', snapshot.getCssLeft());
}
AlterationLeftInverse.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    Browser.changePixelStyle(snapshot.element, 'left', snapshot.getCssLeft() - delta.x);
    return snapshot.delta(snapshot.left - delta.x, snapshot.top, snapshot.width, snapshot.height);
  }
}
AlterationTopInverse.prototype = new Alteration();
AlterationTopInverse.prototype.constructor = AlterationTopInverse;
AlterationTopInverse.superclass = Alteration.prototype;
function AlterationTopInverse( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationTopInverse.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationTopInverse.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationTopInverse.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.top - delta.y);
}
AlterationTopInverse.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(delta.x, -this.maxFeasibleOffset(snapshot.top, -delta.y));
}
AlterationTopInverse.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'top', snapshot.getCssTop());
}
AlterationTopInverse.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    //    ;
    Browser.changePixelStyle(snapshot.element, 'top', snapshot.getCssTop() - delta.y);
    return snapshot.delta(snapshot.left, snapshot.top - delta.y, snapshot.width, snapshot.height);
  }
  //    ;
}
AlterationRightInverse.prototype = new Alteration();
AlterationRightInverse.prototype.constructor = AlterationRightInverse;
AlterationRightInverse.superclass = Alteration.prototype;
function AlterationRightInverse( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationRightInverse.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationRightInverse.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationRightInverse.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.right - delta.x);
}
AlterationRightInverse.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(-this.maxFeasibleOffset(snapshot.right, -delta.x), delta.y);
}
AlterationRightInverse.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'right', snapshot.right);
}
AlterationRightInverse.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    Browser.changePixelStyle(snapshot.element, 'right', snapshot.right - delta.x);
    return snapshot.delta(snapshot.left - delta.x, snapshot.top, snapshot.width, snapshot.height);
  }
}
AlterationBottomInverse.prototype = new Alteration();
AlterationBottomInverse.prototype.constructor = AlterationBottomInverse;
AlterationBottomInverse.superclass = Alteration.prototype;
function AlterationBottomInverse( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationBottomInverse.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationBottomInverse.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationBottomInverse.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.top - delta.y);
}
AlterationBottomInverse.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(delta.x, -this.maxFeasibleOffset(snapshot.top, -delta.y));
}
AlterationBottomInverse.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'bottom', snapshot.bottom);
}
AlterationBottomInverse.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    Browser.changePixelStyle(snapshot.element, 'bottom', snapshot.bottom - delta.y);
    return snapshot.delta(snapshot.left, snapshot.top - delta.y, snapshot.width, snapshot.height);
  }
}
AlterationWidthInverse.prototype = new Alteration();
AlterationWidthInverse.prototype.constructor = AlterationWidthInverse;
AlterationWidthInverse.superclass = Alteration.prototype;
function AlterationWidthInverse( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationWidthInverse.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationWidthInverse.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationWidthInverse.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.width - delta.x);
}
AlterationWidthInverse.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(-this.maxFeasibleOffset(snapshot.width, -delta.x), delta.y);
}
AlterationWidthInverse.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'width', snapshot.width);
}
AlterationWidthInverse.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    Browser.changePixelStyle(snapshot.element, 'width', snapshot.width - delta.x);
    return snapshot.delta(snapshot.left, snapshot.top, snapshot.width - delta.x, snapshot.height);
  }
}
AlterationHeightInverse.prototype = new Alteration();
AlterationHeightInverse.prototype.constructor = AlterationHeightInverse;
AlterationHeightInverse.superclass = Alteration.prototype;
function AlterationHeightInverse( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationHeightInverse.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationHeightInverse.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationHeightInverse.prototype.isFeasible = function( snapshot, delta )
{
  return this.isFeasibleValue(snapshot.height - delta.y);
}
AlterationHeightInverse.prototype.getFeasibleDelta = function( snapshot, delta )
{
  return new Delta(delta.x, -this.maxFeasibleOffset(snapshot.height, -delta.y));
}
AlterationHeightInverse.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'height', snapshot.height);
}
AlterationHeightInverse.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    //    ;
    Browser.changePixelStyle(snapshot.element, 'height', snapshot.height - delta.y);
    return snapshot.delta(snapshot.left, snapshot.top, snapshot.width, snapshot.height - delta.y);
  }
}
AlterationLeftRelative.prototype = new Alteration();
AlterationLeftRelative.prototype.constructor = AlterationLeftRelative;
AlterationLeftRelative.superclass = Alteration.prototype;
function AlterationLeftRelative( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationLeftRelative.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationLeftRelative.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationLeftRelative.prototype.isFeasible = function( snapshot, delta )
{
  if( this.lowerBound && this.upperBound )
    return ( 0.0 <= delta.x && delta.x <= 1.0 );
  else
    return false;
}
AlterationLeftRelative.prototype.getFeasibleDelta = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
    return delta;
  else
    return new Delta(0.0, delta.y);
}
AlterationLeftRelative.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'left', snapshot.getCssLeft());
}
AlterationLeftRelative.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    var left = ((this.upperBound - this.lowerBound) * delta.x + this.lowerBound) - snapshot.parentPos.x;
    Browser.changePixelStyle(snapshot.element, 'left', left);
    return snapshot.delta(left, snapshot.top, snapshot.width, snapshot.height);
  }
}
AlterationLeftRelative.prototype.alterAdequate = function( snapshot, delta )
{
  return this.alter(snapshot, delta);
}
AlterationTopRelative.prototype = new Alteration();
AlterationTopRelative.prototype.constructor = AlterationTopRelative;
AlterationTopRelative.superclass = Alteration.prototype;
function AlterationTopRelative( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationTopRelative.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationTopRelative.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationTopRelative.prototype.isFeasible = function( snapshot, delta )
{
  if( this.lowerBound && this.upperBound )
    return ( 0.0 <= delta.y && delta.y <= 1.0 );
  else
    return false;
}
AlterationTopRelative.prototype.getFeasibleDelta = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
    return delta;
  else
    return new Delta(delta.x, 0.0);
}
AlterationTopRelative.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'top', snapshot.getCssTop());
}
AlterationTopRelative.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    var top = ((this.upperBound - this.lowerBound) * delta.y + this.lowerBound) - snapshot.parentPos.y;
//    ;
//    ;
//    ;
    Browser.changePixelStyle(snapshot.element, 'top', top);
    return snapshot.delta(snapshot.left, top, snapshot.width, snapshot.height);
  }
}
AlterationTopRelative.prototype.alterAdequate = function( snapshot, delta )
{
  return this.alter(snapshot, delta);
}
AlterationWidthRelative.prototype = new Alteration();
AlterationWidthRelative.prototype.constructor = AlterationWidthRelative;
AlterationWidthRelative.superclass = Alteration.prototype;
function AlterationWidthRelative( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationWidthRelative.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationWidthRelative.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationWidthRelative.prototype.isFeasible = function( snapshot, delta )
{
  if( this.lowerBound && this.upperBound )
    return ( 0.0 <= delta.x && delta.x <= 1.0 );
  else
    return false;
}
AlterationWidthRelative.prototype.getFeasibleDelta = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
    return delta;
  else
    return new Delta(0.0, delta.y);
}
AlterationWidthRelative.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'width', snapshot.width);
}
AlterationWidthRelative.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    var width = (this.upperBound - this.lowerBound) * delta.x;
    Browser.changePixelStyle(snapshot.element, 'width', width);
    return snapshot.delta(snapshot.left, snapshot.top, width, snapshot.height);
  }
}
AlterationWidthRelative.prototype.alterAdequate = function( snapshot, delta )
{
  return this.alter(snapshot, delta);
}
AlterationHeightRelative.prototype = new Alteration();
AlterationHeightRelative.prototype.constructor = AlterationHeightRelative;
AlterationHeightRelative.superclass = Alteration.prototype;
function AlterationHeightRelative( sId, lowerBound, upperBound )
{
  if( sId )
  {
    this.init(sId, lowerBound, upperBound);
  }
}
AlterationHeightRelative.prototype.init = function( sId, lowerBound, upperBound )
{
  AlterationHeightRelative.superclass.init.call(this, sId, lowerBound, upperBound);
}
AlterationHeightRelative.prototype.isFeasible = function( snapshot, delta )
{
  if( this.lowerBound && this.upperBound )
    return ( 0.0 <= delta.y && delta.y <= 1.0 );
  else
    return false;
}
AlterationHeightRelative.prototype.getFeasibleDelta = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
    return delta;
  else
    return new Delta(delta.x, 0.0);
}
AlterationHeightRelative.prototype.reset = function( snapshot )
{
  Browser.changePixelStyle(snapshot.element, 'height', snapshot.height);
}
AlterationHeightRelative.prototype.alter = function( snapshot, delta )
{
  if( this.isFeasible(snapshot, delta) )
  {
    var height = (this.upperBound - this.lowerBound) * delta.y;
    Browser.changePixelStyle(snapshot.element, 'height', height);
    return snapshot.delta(snapshot.left, snapshot.top, snapshot.width, height);
  }
}
AlterationHeightRelative.prototype.alterAdequate = function( snapshot, delta )
{
  return this.alter(snapshot, delta);
}
