/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>Globals</DEPENDS>
</DEPENDENCIES>
*/
function TipManager(sId, bStepByStep, bSolutable, bCycleTips, bInteractionTipEnabled)
{
  this.init(sId, bStepByStep, bSolutable, bCycleTips, bInteractionTipEnabled)
}
TipManager.prototype.init = function(sId, bStepByStep, bSolutable, bCycleTips, bInteractionTipEnabled)
{
  ;
  this.sId = sId;
  this.bStepByStep = bStepByStep;
  this.bSolutable = bSolutable;
  this.bCycleTips = bCycleTips;
  this.bInteractionTipEnabled = (bInteractionTipEnabled != undefined) ? bInteractionTipEnabled : true;
  this.currentTipHandler = null;
  this.aTipHandlers = new Array();
  this.aSolutionHandlers = new Array();
  this.aOnceSuccessfulHandlers = new Array();
  this.nTipHandlers = 0;
  this.nSolutionHandlers = 0;
  this.tipBroker = null;
  this.aInteractionTipHandlers = new Array();
  this.aIndicatingTipHandlers = new Array();
}
TipManager.prototype.start = function()
{
  ;
  for(var i in this.aIndicatingTipHandlers)
  {
    var tipHandler = this.aIndicatingTipHandlers[i];
    this.registerTipHandler(tipHandler);
    this.registerSolutionHandler(tipHandler);
  }
  for(var i in this.aInteractionTipHandlers)
  {
    var tipHandler = this.aInteractionTipHandlers[i];
    if(this.bInteractionTipEnabled)
    {
      this.registerTipHandler(tipHandler);
    }
    this.registerSolutionHandler(tipHandler);
  }
  if(this.nTipHandlers > 0)
  {
    if(this.tipBroker) {this.tipBroker.tipEnabled();}
  }
  else
  {
    if(this.tipBroker) {this.tipBroker.tipDisabled();}
  }
}
TipManager.prototype.register = function(tipHandler)
{
  this.aInteractionTipHandlers[tipHandler.getId()] = tipHandler;
}
TipManager.prototype.registerIndicatingTip = function(tipHandler)
{
  this.aIndicatingTipHandlers[tipHandler.getId()] = tipHandler;
}
TipManager.prototype.registerTipHandler = function(tipHandler)
{
  ;
  if (this.aTipHandlers[tipHandler.getId()] != null) 
  {
    ;
  }  
  else
  {
    this.aTipHandlers[tipHandler.getId()] = tipHandler;
    this.nTipHandlers++;
    ;
    tipHandler.setTipManager(this);
    if (this.currentTipHandler == null) 
    {
      this.setCurrent(tipHandler);
    }
  }
}
TipManager.prototype.registerSolutionHandler = function(tipHandler)
{
  ;
  if (this.aSolutionHandlers[tipHandler.getId()] != null)
  {
    ;
  }
  else
  {
    this.aSolutionHandlers[tipHandler.getId()] = tipHandler;
    this.nSolutionHandlers++;
    ;
    tipHandler.setTipManager(this);
  }
}
TipManager.prototype.setTipBroker = function(tipBroker)
{
  ;
  this.tipBroker = tipBroker;
}
TipManager.prototype.setCurrent = function(tipHandler)
{
  ;
  this.currentTipHandler = tipHandler;
}
TipManager.prototype.showTip = function()
{
  ;
  var bFoundOne = false;
  if (this.currentTipHandler != null)
  {
    var id = this.currentTipHandler.getId();
    if( !this.bInteractionTipEnabled || !this.aIndicatingTipHandlers[id] || !this.aOnceSuccessfulHandlers[id])
    {
      if(this.currentTipHandler.showTip(this.bStepByStep, this.bSolutable, this.bCycleTips))
      {
        ;
        this.aOnceSuccessfulHandlers[this.currentTipHandler.getId()] = true;
        bFoundOne = true;
      }
    }
  }
  if (!bFoundOne) {
    ;
    for (var id in this.aTipHandlers)
    {
      if (this.aTipHandlers[id] != this.currentTipHandler)
      {
        if( !this.bInteractionTipEnabled || !this.aIndicatingTipHandlers[id] || !this.aOnceSuccessfulHandlers[id])
        {
          if (this.aTipHandlers[id].showTip(this.bStepByStep, this.bSolutable, this.bCycleTips))
          {
            ;
            this.aOnceSuccessfulHandlers[id] = true;
            bFoundOne = true;
            break;
          }
        }
      }
    }
  }
  if (!bFoundOne) {
    ;
  }
  if (bFoundOne) {
    this.checkTipEnabled();
  }
}
TipManager.prototype.checkTipEnabled = function()
{
  ;
  var bFoundOne = false;
  for (var id in this.aTipHandlers)
  {
    if (this.aTipHandlers[id] != this.currentTipHandler)
    {
      if (this.aTipHandlers[id].hasTip(this.bStepByStep, this.bSolutable, this.bCycleTips))
      {
        bFoundOne = true;
        break;        
      }
    }
  }
  if (bFoundOne)
  {
    if (this.tipBroker) {this.tipBroker.tipAvailable();}
  }
  else
  {
    if (this.tipBroker) {this.tipBroker.tipUnavailable();}
  }
}
TipManager.prototype.getCheckTipAction = function()
{
  return new ObjectMethodAction(this, "checkTipEnabled");
}
TipManager.prototype.changed = function(workItem)
{
  ;
  this.checkTipEnabled();
}
TipManager.prototype.showSolution = function() 
{
  ;
  for (var id in this.aSolutionHandlers)
  {
    if (this.aSolutionHandlers[id].showSolution) {this.aSolutionHandlers[id].showSolution();}
  }
}
TipManager.prototype.hideSolution = function() 
{
  ;
  for (var id in this.aSolutionHandlers)
  {
    if (this.aSolutionHandlers[id].hideSolution) {this.aSolutionHandlers[id].hideSolution();}
  }
}
TipManager.prototype.getShowSolutionAction = function()
{
  return new ObjectMethodAction(this, "showSolution");
}
TipManager.prototype.getHideSolutionAction = function()
{
  return new ObjectMethodAction(this, "hideSolution");
}
TipManager.prototype.getShowTipAction = function()
{
  return new ObjectMethodAction(this, "showTip");
}
TipManager.prototype.reset = function()
{
  this.aOnceSuccessfulHandlers = new Array();
}
TipManager.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, "reset");
}
