
/* /1175261643647_35.js

Logger.js
domapi/Globals.js
common/broker/Broker.js
actions/Actions.js
toolkit/tip/TipBroker.js
toolkit/toggle/ToggleBroker.js
domapi/ServerResult.js
domapi/Browser.ie.js
domapi/Controller.js
domapi/Action.js
domapi/DomletCtr.js
domapi/Domlet.js
common/widgets/Division.js
toolkit/tip/TipManager.js
common/widgets/UniversalPushButton.js
toolkit/exercise/ExerciseManager.js
common/widgets/state/StateBroker.js
toolkit/exercise/ExerciseBroker.js
toolkit/qea/WorkItemBroker.js
common/widgets/UserInteractionController.js
common/widgets/SimpleImage.js
common/widgets/SimpleTooltip.js
common/widgets/UserInteractionBroker.js
common/widgets/ImagePushButton.js
toolkit/qea/QeaItem.js
toolkit/qea/Question.js
toolkit/staticexport/ServiceRegistryAccessor.js
toolkit/screen/ScreenBroker.js
toolkit/qea/AnswerValue.js
toolkit/qea/Answer.js
toolkit/qea/Evaluation.js
toolkit/qea/QeaWorkItem.js
toolkit/fake/FakeWorkItem.js
common/widgets/TextPushButton.js
toolkit/exercise/ComprehensionManager.js
toolkit/pager/VisibilityBroker.js
toolkit/info/InformationVisibilityManager.js
toolkit/toggle/ToggleManagerStatic.js
toolkit/screen/ScreenController.js
toolkit/staticexport/ScoreStatistics.js
toolkit/achievement/AchievementStatistic.js
toolkit/achievement/AchievementManager.js
toolkit/lms/LMSDispatcherStatic.js
toolkit/exercise/ComprehensionBroker.js
*/

/* Logger.js */
/*
<DEPENDENCIES>
</DEPENDENCIES>
*/
// Logger

/**
 * Defines the Logging class. This will usually be a single global instance
 *
 * @constructor
 */
function Logger()
{
	this.nFatal  = 0;
	this.nError =       this.nFatal + 1;
	this.nWarning =     this.nError + 1;
	this.nDebug =       this.nWarning + 1;
	this.nInfo =        this.nDebug + 1;
	this.nVerbose =     this.nInfo + 1;
	this.nTrace =       this.nVerbose + 1;
	this.nVeryVerbose = this.nTrace + 1;

	this.sDebugFrame = 'debugFrame';
	this.bActive = true;
	this.nLevel = this.nVeryVerbose;
	this.nLine = 0;
	this.sMask = '';

	this.aColor = new Array();
	this.aPrefix = new Array();
	this.aMask = new Array();

	this.aColor[this.nFatal]       = '#ff0000';
	this.aColor[this.nError]       = '#ff0000';
	this.aColor[this.nWarning]     = '#ff6666';
	this.aColor[this.nDebug]       = '#000000';
	this.aColor[this.nInfo]        = '#0000ff';
	this.aColor[this.nVerbose]     = '#588AB5';
	this.aColor[this.nTrace]       = '#785A95';
	this.aColor[this.nVeryVerbose] = '#006600';

	this.aPrefix[this.nFatal]       = 'Fatal&nbsp;&nbsp;';
	this.aPrefix[this.nError]       = 'Error&nbsp;&nbsp;';
	this.aPrefix[this.nWarning]     = 'Warning';
	this.aPrefix[this.nDebug]       = 'Debug&nbsp;&nbsp;';
	this.aPrefix[this.nInfo]        = 'Info&nbsp;&nbsp;&nbsp;';
	this.aPrefix[this.nVerbose]     = '.......';
	this.aPrefix[this.nTrace]       = 'Trace&nbsp;&nbsp;';
	this.aPrefix[this.nVeryVerbose] = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';

  this.aHex = new Array();
  this.aHex[0] = '0';
  this.aHex[1] = '1';
  this.aHex[2] = '2';
  this.aHex[3] = '3';
  this.aHex[4] = '4';
  this.aHex[5] = '5';
  this.aHex[6] = '6';
  this.aHex[7] = '7';
  this.aHex[8] = '8';
  this.aHex[9] = '9';
  this.aHex[10] = 'a';
  this.aHex[11] = 'b';
  this.aHex[12] = 'c';
  this.aHex[13] = 'd';
  this.aHex[14] = 'e';
  this.aHex[15] = 'f';

  this.aModuleColor = new Array();
  
  this.lastLine = null; // for scrolling


  try
  {
  	if(!parent.frames[this.sDebugFrame])
  		this.bActive=false;
  }
  catch(e)
  {
  	this.bActive=false;
  }

  if ( this.bActive && parent.frames[this.sDebugFrame])
	{
		parent.frames[this.sDebugFrame].document.write('<html><head>\n');
		parent.frames[this.sDebugFrame].document.write('<link rel="stylesheet" href="../af-web/js/Logger.css" type="text/css">\n');
		parent.frames[this.sDebugFrame].document.write('</head><body>\n');
		parent.frames[this.sDebugFrame].document.write('<p class="Logger_Text">\n');
	}

  if ( this.bActive && parent.frames[this.sDebugFrame]) 
  {
    for (var i = this.nFatal; i <= this.nVeryVerbose; i++) 
    {
      this.aMask[i] = true;
    }
  }

	if ( this.bActive && parent.frames[this.sDebugFrame]) 
	{
		var sQuery = parent.document.location.search;
		sQuery = sQuery.substring(1, sQuery.length);
    var aArgs = sQuery.match(/([^=]+)\=([^&]+)/);

		if (aArgs != null)
		{
			for (var i = 1; i < aArgs.length; i++)
			{
				if (false) {} 
				else if (aArgs[i] == 'logmask')
				{
					i++;

          this.setLogMask(aArgs[i]);
				}
			}
		}
	}
	
}


/**
 * Set log mask from string. 
 *
 * @param sString mask string like "f,e,d,i,v,t,vv" or "-t,vv"
 */
Logger.prototype.setLogMask = function(sMask)
{
  var aName = new Array();
  aName[this.nFatal]       = 'f';
  aName[this.nError]       = 'e';
  aName[this.nWarning]     = 'w';
  aName[this.nDebug]       = 'd';
  aName[this.nInfo]        = 'i';
  aName[this.nVerbose]     = 'v';
  aName[this.nTrace]       = 't';
  aName[this.nVeryVerbose] = 'vv';

  this.sMask = sMask;

  for (var i = this.nFatal; i <= this.nVeryVerbose; i++) 
  {
    this.aMask[i] = true;
  }

  if (sMask == '')
  { 
    //
  }
  else 
  {
    var bValue = true;

    if (false) {}
    else if (sMask.substring(0, 1) == '-')
    {
      sMask = sMask.substring(1, sMask.length);
      bValue = false;
      for (var j = this.nFatal; j <= this.nVeryVerbose; j++) this.aMask[j] = true;
    }
    else
    {
      bValue = true;
      for (var j = this.nFatal; j <= this.nVeryVerbose; j++) this.aMask[j] = false;
    }

    aLogmask = sMask.split(',');
    for (var j = 0; j < aLogmask.length; j++)
    {
      var nLevel = -1;
      for (var k = this.nFatal; k <= this.nVeryVerbose; k++) 
      {
        if (aLogmask[j] == aName[k])
        {
          nLevel = k;
          break;
        }
      }
      if (nLevel >= 0) 
      {
        this.aMask[nLevel] = bValue;
      }
    }
  }
  
} 


/**
 * Return the log mask as string. 
 *
 */
Logger.prototype.getLogMask = function()
{
  var aName = new Array();
  aName[this.nFatal]       = 'f';
  aName[this.nError]       = 'e';
  aName[this.nWarning]     = 'w';
  aName[this.nDebug]       = 'd';
  aName[this.nInfo]        = 'i';
  aName[this.nVerbose]     = 'v';
  aName[this.nTrace]       = 't';
  aName[this.nVeryVerbose] = 'vv';
  
  var sMask = '';

  for (var i = this.nFatal; i <= this.nVeryVerbose; i++) 
  {
    if (this.aMask[i])
    {
      if (sMask != '') sMask += ',';
      sMask += aName[i];
    }
  }

  return sMask;
} 


Logger.prototype.getHex = function(nValue)
{
  var sValue = '';
  var nLow = nValue % 16;
  var nHigh = (nValue - nValue % 16) / 16;
  sValue = this.aHex[nHigh] + this.aHex[nLow];
  //alert(nValue+' '+nLow+' '+nHigh+' '+sValue);
  return sValue;
}

Logger.prototype.getModuleColor = function(sModule)
{
  if (this.aModuleColor[sModule] == null)
  {
    var sColor = '#000000';
    var nSum = 0;
    for (var i = 0; i < sModule.length; i++) 
    {
      var nChar = sModule.charCodeAt(i);
      nSum += parseInt(nChar);
    }
    var nRed = nSum % 171;
    var nGreen = nSum % 215;
    var nBlue = nSum % 191;
    this.aModuleColor[sModule] = '#' + this.getHex(nRed) + this.getHex(nGreen) + this.getHex(nBlue);
  }
  return this.aModuleColor[sModule];
}

/**
 * Generic print. 
 *
 * @param nLevel The debug level
 * @param sModule module name or class name that reports
 * @param sMessage log message, usually contains current METHOD and the actual message text
 */
Logger.prototype.print = function(nLevel, sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;

	if (nLevel <= this.nLevel && this.aMask[nLevel])
	{
		this.nLine++;
    var sColor = this.aColor[nLevel];
    
    var bBold = false;
    switch (nLevel)
    {
       case this.nFatal: bBold = true; break;
       case this.nError: bBold = true; break;
       //case this.nDebug: bBold = true; break;
      default: ;
    }
        
    if (!bBold) sColor = this.getModuleColor(sModule);
    
    switch (sModule)
    {
      case 'bla': sColor = '#123456'; break;
      default: ;
    }
    
    if (nLevel == this.nDebug)
    {
      sColor = '#0000ff';
    }
    
    var sOut = '<font id="Logger_' + (this.nLine) + '" color=\"' + sColor + '\">' + this.aPrefix[nLevel] + (bBold?'<b>':'') + ' [' + sModule + '] ' + sMessage + (bBold?'</b>':'') + '</font><br>\n';
		parent.frames[this.sDebugFrame].document.write(sOut);
		
    var o = parent.frames[this.sDebugFrame].document.getElementById('Logger_'+this.nLine);
    if (o != null) 
    {
      if (this.nLine % 20 == 0)
      {
        o.scrollIntoView(true);
        this.lastLine = null;
      }
      else
      {
        this.lastLine = o;
      }
    }
	}
  
} 


/**
 * Wrapper for fatal error messages. Fatal means program will not be able to continue.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.fatal = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nFatal, sModule, sMessage);
}



/**
 * Wrapper for error messages. Means something serious happens, but program will recover
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.error = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nError, sModule, sMessage);
}


/**
 * Wrapper for warnings. A bad condition.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.warning = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nWarning, sModule, sMessage);
}


/**
 * Wrapper for debug output. Use this for temporary printf-debugging.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.debug = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nDebug, sModule, sMessage);
}


/**
 * Wrapper for info messages. Output important state changes. Expected to generate few lines per second
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.info = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nInfo, sModule, sMessage);
}


/**
 * Wrapper for verbose output. Expected to generate 50 lines per second.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.verbose = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
	this.print(this.nVerbose, sModule, sMessage);
}


/**
 * Wrapper for method entry messages, optional exit messages.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.trace = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
		this.print(this.nTrace, sModule, sMessage);
}


/**
 * Wrapper for very verbose output. Expected to generate 1000 lines per second.
 *
 * @param sModule see Logger.prototype.print
 * @param sMessage see Logger.prototype.print
 */
Logger.prototype.veryverbose = function(sModule, sMessage)
{
	if ( !this.bActive || !parent.frames[this.sDebugFrame]) return;
		this.print(this.nVeryVerbose, sModule, sMessage);
}


Logger.prototype.scrollToLastLine = function()
{
  if (this.lastLine != null)
  {
    this.lastLine.scrollIntoView(true);
    this.lastLine = null;
  }
} 

Logger.prototype.objectTypeToString = function( object )
{
  if( object == undefined )
    return 'object is UNDEFINED';
  if( object == null )
    return 'object is NULL';

  return object.constructor;
}

Logger.prototype.objectToString = function( object, mode )
{
  if( object == undefined )
    return 'UNDEFINED';
  if( object == null )
    return 'NULL';
  // ---- doesn't work in IE.
  //  if( HTMLElement.prototype.isPrototypeOf(object) )
  //    return this.htmlElementToString(object);
  // ----
  if( object.tagName != undefined )
    return this.htmlElementToString(object);

  var result = '';

  if( mode == undefined || mode == null )
  {
    for( name in object )
    {
      if( result == '' )
        result += name;
      else
        result += ', ' + name;
    }

    if( result == '' )
      result = '{ -- NO ENUMERABLE PROPERTIES -- }';
    else
      result = '{ ' + result + ' }';
  }
  else if( mode == 'v' )
  {
    for( name in object )
    {
      result += name + '=' + object[name] + ';';
    }

    if( result == '' )
      result = '{ -- NO ENUMERABLE PROPERTIES -- }';
    else
      result = '{ ' + result + ' }';
  }
  else if( mode == 'vv' )
  {
    for( name in object )
    {
      result += name + '=' + this.objectToString(object[name], 'v') + ';';
    }

    if( result == '' )
      result = '{ -- NO ENUMERABLE PROPERTIES -- }';
    else
      result = '{ ' + result + ' }';
  }

  return result;
}

Logger.prototype.htmlElementToString = function( object )
{
  if( object == undefined )
    return 'UNDEFINED';
  if( object == null )
    return 'NULL';

  return '{ tagName=' + object.tagName
          + (object.id == undefined ? '' : (object.id == null ? '' : '; id=' + object.id))
          + (object.className == undefined ? '' : (object.className == null ? '' : '; className=' + object.className))
          + ' }';
}


// Greate a static instance
Log = new Logger();
setInterval('Log.scrollToLastLine()', 1000);

// Check the logger
Log.info('main', 'Logger starting with mask=' + Log.getLogMask());


/* domapi/Globals.js */
/*
<DEPENDENCIES>
</DEPENDENCIES>
*/
function Point(x,y)
{
  this.x = x;
  this.y = y;
}
Point.prototype.toString = function()
{
  return 'Point{ x=' + this.x + ', y=' + this.y + ' }';
}
function shuffleArray(a)
{
  var aLinearValue = new Array();
  var aLinearUsed = new Array();
  for (var i = 0; i < a.length; i++)
  {
    aLinearValue[i] = a[i];
    aLinearUsed[i] = false;
  }
  var nLow = 0;
  var nHigh = a.length - 1;
  for (var i = 0; i < a.length; i++)
  {
    nRandomIndex = Math.round( (nLow - 0.5) +( ( (nHigh+0.49999) - (nLow-0.5) ) * Math.random() ) );
    var bFoundUnused = false;
    while (!bFoundUnused)
    {
      if (! aLinearUsed[nRandomIndex])
      {
        bFoundUnused = true;
        aLinearUsed[nRandomIndex] = true; 
        break;
      }
      else
      {
        nRandomIndex++;
        if (nRandomIndex > nHigh) nRandomIndex = nLow;
      }
    }
    a[i] = aLinearValue[nRandomIndex];
  }
}

/* common/broker/Broker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Globals</DEPENDS>
</DEPENDENCIES>
*/
function BrokerActionPipeline()
{
  this.callstack = new Array();
  this.pipeline = new Array();
  this.blockingTasks = new Array();
  this.finalAction = null;
  this.blockCounter = 0;
}
BrokerActionPipeline.prototype.beginAction = function(key)
{
  ;
  this.callstack.push(key);
}
BrokerActionPipeline.prototype.endAction = function(key)
{
  ;
  var closedBrokerKey = this.callstack[this.callstack.length -1];
  if(key != closedBrokerKey)
  {
    ;
  }
  else
  {
    this.callstack.pop();
  }
  this.executePipeline();
}
BrokerActionPipeline.prototype.executePipeline = function()
{
  if((this.callstack.length == 0) && (this.blockCounter == 0))
  {
    this.nextScheduledAction();
  }
}
BrokerActionPipeline.prototype.block = function(key)
{
  ;
  if(!this.blockingTasks[key])
  {
    this.blockingTasks[key] = true;
    this.blockCounter++;
  }
}
BrokerActionPipeline.prototype.unblock = function(key)
{
  ;
  if(this.blockingTasks[key])
  {
    this.blockingTasks[key] = false;
    this.blockCounter--;
  }
  this.executePipeline();
}
BrokerActionPipeline.prototype.scheduleFinalAction = function(action, parameterArray)
{
  ;
  if(this.finalAction)
  {
    ;
  }
  else
  {
    var actionObject = new Object();
    actionObject.action = action;
    actionObject.parameterArray = parameterArray;
    this.finalAction = actionObject;
  }
}
BrokerActionPipeline.prototype.scheduleAction = function(action, parameterArray)
{
  ;
  var actionObject = new Object();
  actionObject.action = action;
  actionObject.parameterArray = parameterArray;
  this.pipeline.push(actionObject);
  this.executePipeline();
}
BrokerActionPipeline.prototype.nextScheduledAction = function()
{
  ;
  if(this.pipeline.length > 0)
  {
    ;
    var actionObject = this.pipeline.shift();
    BrokerActions_callAction(actionObject.action, actionObject.parameterArray);
    this.executePipeline();
  }
  else if(this.finalAction)
  {
    ;
    var callAction = this.finalAction;
    this.finalAction = null;
    BrokerActions_callAction(callAction.action, callAction.parameterArray);
  }
}
function BrokerActions()
{
   this.init()
}
BrokerActions.prototype.init = function(name)
{
  this.actions = new Array();
}
BrokerActions.prototype.callActions = function(parameterArray)
{
  for(id in this.actions)
  {
    var action = this.actions[id];
    BrokerActions_callAction(action, parameterArray);
  }
}
BrokerActions.prototype.addAction = function(action)
{
  this.actions.push(action);
}
function BrokerActions_callAction(action, parameterArray)
{
  if(parameterArray)
  {
    action.invoke(
      parameterArray[0],
      parameterArray[1],
      parameterArray[2],
      parameterArray[3],
      parameterArray[4],
      parameterArray[5],
      parameterArray[6],
      parameterArray[7],
      parameterArray[8]);
  }
  else
  {
    action.invoke();
  }
}
function Broker(name)
{
  this.init(name)
}
Broker.actionPipeline = new BrokerActionPipeline();
Broker.prototype.init = function(name)
{
  this.name = name;
  this.callbacks = new Array();
}
Broker.prototype.getMessage = function(message, parameterArray)
{
  var key = this.name + '_' + message;
  Broker.actionPipeline.beginAction(key);
  var callback = this.callbacks[message];
  if(callback)
  {
    callback.callActions(parameterArray);
  }
  Broker.actionPipeline.endAction(key);
}
Broker.prototype.register = function(match, obj, callback)
{
  ;
}
Broker.prototype.registerAction = function(match, action)
{
  var callback = this.callbacks[match];
  if(!callback)
  {
    callback = new BrokerActions();
    this.callbacks[match] = callback;
  }
  callback.addAction(action);
}

/* actions/Actions.js */
/*
<DEPENDENCIES>
  <DEPENDS>Broker</DEPENDS>
</DEPENDENCIES>
*/
function ChangeImageAction(domId, url)
{
    this.domId = domId;
    this.url = url;
}
ChangeImageAction.prototype.invoke = function()
{
    document.getElementById(this.domId).src=this.url;
}
function SetStyleAction(domId, property, value)
{
    this.domId = domId;
    this.property = property;
    this.value = value;
}
SetStyleAction.prototype.invoke = function()
{
    document.getElementById(this.domId).style[this.property] = this.value;
}
function ObjectMethodAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
ObjectMethodAction.prototype.invoke = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
    if (typeof parameter1 != 'undefined')
        return this.objectName[this.methodname](parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7);
    else
        return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function ObjectMethodWithParameterAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
ObjectMethodWithParameterAction.prototype.invoke = function(parameter1)
{
    if (typeof parameter1 != 'undefined')
        return this.objectName[this.methodname](parameter1, this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
    else
        return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function SyncObjectMethodAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
SyncObjectMethodAction.prototype.invoke = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
  Broker.actionPipeline.scheduleAction(new ObjectMethodAction(this, "invokeTarget"), [parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7]);
}
SyncObjectMethodAction.prototype.invokeTarget = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
    if (typeof parameter1 != 'undefined')
        return this.objectName[this.methodname](parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7);
    else
        return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function FinalObjectMethodAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
FinalObjectMethodAction.prototype.invoke = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
  Broker.actionPipeline.scheduleFinalAction(new ObjectMethodAction(this, "invokeTarget"), [parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7]);
}
FinalObjectMethodAction.prototype.invokeTarget = function(parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7)
{
    if (typeof parameter1 != 'undefined')
        return this.objectName[this.methodname](parameter1, parameter2, parameter3, parameter4, parameter5, parameter6, parameter7);
    else
        return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function IgnoreRuntimeParameterAction(objectName, methodname, param1, param2, param3, param4, param5, param6, param7)
{
    this.objectName= objectName;
    this.methodname = methodname;
    this.param1 = param1;
    this.param2 = param2;
    this.param3 = param3;
    this.param4 = param4;
    this.param5 = param5;
    this.param6 = param6;
    this.param7 = param7;
}
IgnoreRuntimeParameterAction.prototype.invoke = function()
{
    return this.objectName[this.methodname](this.param1, this.param2, this.param3, this.param4, this.param5, this.param6, this.param7);
}
function AlertAction(text)
{
    this.text = text;
}
AlertAction.prototype.invoke = function()
{
    alert(this.text);
}
function SolvedAction(objectName, methodname)
{
    this.objectName = objectName;
    this.methodname = methodname;
}
SolvedAction.prototype.invoke = function(para1, para2, para3, para4, para5, para6, para7, para8)
{
    this.object[this.methodname](para1, para2, para3, para4, para5, para6, para7, para8); 
}
function DoNothingAction()
{
}
DoNothingAction.prototype.invoke = function()
{
}

/* toolkit/tip/TipBroker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
</DEPENDENCIES>
*/
TipBroker.prototype = new Broker();
TipBroker.prototype.constructor = TipBroker;
TipBroker.superclass = Broker.prototype;
function TipBroker()
{
  ;
  this.tipController = null;
  TipBroker.superclass.init.call(this, "TipBroker");
}
TipBroker.prototype.setTipController = function( tipController )
{
  this.tipController = tipController;
}
TipBroker.prototype.doTipRequested = function()
{
  ;
  this.getMessage("tipRequested");
}
TipBroker.prototype.addTipRequestedHandle = function( action )
{
  this.registerAction("tipRequested", action);
}
TipBroker.prototype.getTipRequestedAction = function( action )
{
  return new ObjectMethodAction(this, "tipRequested");
}
TipBroker.prototype.tipRequested = function()
{
  if( this.tipController )
    this.tipController.shouldTipRequested();
  else
    this.doTipRequested();
}
TipBroker.prototype.doTipDisabled = function()
{
  ;
  this.getMessage("tipDisabled");
}
TipBroker.prototype.addTipDisabledHandle = function( action )
{
  this.registerAction("tipDisabled", action);
}
TipBroker.prototype.getTipDisabledAction = function( action )
{
  return new ObjectMethodAction(this, "tipDisabled");
}
TipBroker.prototype.tipDisabled = function()
{
  if( this.tipController )
    this.tipController.shouldTipDisabled();
  else
    this.doTipDisabled();
}
TipBroker.prototype.doTipEnabled = function()
{
  ;
  this.getMessage("tipEnabled");
}
TipBroker.prototype.addTipEnabledHandle = function( action )
{
  this.registerAction("tipEnabled", action);
}
TipBroker.prototype.getTipEnabledAction = function( action )
{
  return new ObjectMethodAction(this, "tipEnabled");
}
TipBroker.prototype.tipEnabled = function()
{
  if( this.tipController )
    this.tipController.shouldTipEnabled();
  else
    this.doTipEnabled();
}
TipBroker.prototype.doTipAvailable = function()
{
  ;
  this.getMessage("tipAvailable");
}
TipBroker.prototype.addTipAvailableHandle = function( action )
{
  this.registerAction("tipAvailable", action);
}
TipBroker.prototype.getTipAvailableAction = function( action )
{
  return new ObjectMethodAction(this, "tipAvailable");
}
TipBroker.prototype.tipAvailable = function()
{
  if( this.tipController )
    this.tipController.shouldTipAvailable();
  else
    this.doTipAvailable();
}
TipBroker.prototype.doTipUnavailable = function()
{
  ;
  this.getMessage("tipUnavailable");
}
TipBroker.prototype.addTipUnavailableHandle = function( action )
{
  this.registerAction("tipUnavailable", action);
}
TipBroker.prototype.getTipUnavailableAction = function( action )
{
  return new ObjectMethodAction(this, "tipUnavailable");
}
TipBroker.prototype.tipUnavailable = function()
{
  if( this.tipController )
    this.tipController.shouldTipUnavailable();
  else
    this.doTipUnavailable();
}
TipBroker.prototype.doShowSolution = function()
{
  ;
  this.getMessage("showSolution");
}
TipBroker.prototype.addShowSolutionHandle = function( action )
{
  this.registerAction("showSolution", action);
}
TipBroker.prototype.getShowSolutionAction = function()
{
  return new ObjectMethodAction(this, "showSolution");
}
TipBroker.prototype.showSolution = function()
{
  if( this.exerciseController )
  {
    this.exerciseController.shouldShowSolution();
  }
  else
  {
    this.doShowSolution();
  }
}
TipBroker.prototype.doHideSolution = function()
{
  ;
  this.getMessage("hideSolution");
}
TipBroker.prototype.addHideSolutionHandle = function( action )
{
  this.registerAction("hideSolution", action);
}
TipBroker.prototype.getHideSolutionAction = function()
{
  return new ObjectMethodAction(this, "hideSolution");
}
TipBroker.prototype.hideSolution = function()
{
  if( this.exerciseController )
  {
    this.exerciseController.shouldHideSolution();
  }
  else
  {
    this.doHideSolution();
  }
}

/* toolkit/toggle/ToggleBroker.js */
 /*
 <DEPENDENCIES>
	 <DEPENDS>Logger</DEPENDS>
	 <DEPENDS>Actions</DEPENDS>
	 <DEPENDS>Broker</DEPENDS>
 </DEPENDENCIES>
 */
 ToggleBroker.prototype             = new Broker();
 ToggleBroker.prototype.constructor = ToggleBroker;
 ToggleBroker.superclass            = Broker.prototype;
 function ToggleBroker()
 {
	 ToggleBroker.superclass.init.call(this, 'ToggleBroker');
 }
 ToggleBroker.prototype.setToggleManager = function(togglemanager)
 {
	 this.togglemanager = togglemanager;
 }
 ToggleBroker.prototype.getId = function()
 {
	 return 'ToggleBroker';
 }
 ToggleBroker.prototype.doIsToggleOn = function()
 {
	 ;
	 this.getMessage('istoggleon');
 }
 ToggleBroker.prototype.addIsToggleOnHandle = function(action)
 {
	 ;
	 this.registerAction('istoggleon', action);
 }
 ToggleBroker.prototype.getToggleOnAction = function()
 {
	 ;
	 return new ObjectMethodAction(this, 'toggleOn');
 }
 ToggleBroker.prototype.toggleOn = function()
 {
	 ;
	 this.togglemanager.setToggleOn();
 }
 ToggleBroker.prototype.doIsToggleOff = function()
 {
	 ;
	 this.getMessage('istoggleoff');
 }
 ToggleBroker.prototype.addIsToggleOffHandle = function(action)
 {
	 ;
	 this.registerAction('istoggleoff', action);
 }
 ToggleBroker.prototype.getToggleOffAction = function()
 {
	 ;
	 return new ObjectMethodAction(this, 'toggleOff');
 }
 ToggleBroker.prototype.toggleOff = function()
 {
	 ;
	 this.togglemanager.setToggleOff();
 }

/* domapi/ServerResult.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function ServerResult( sId, xmldoc )
{
  if( sId )
  {
    this.init(sId, xmldoc)
  }
}
ServerResult.prototype.init = function( sId, xmldoc )
{
  ;
  this.sId = sId;
  this.resultMap = new Array();
  ;
  this.buildMap(xmldoc);
}
ServerResult.prototype.buildMap = function( xmldoc )
{
  ;
  if( xmldoc.nodeType == Browser.Node.DOCUMENT_NODE )
  {
    ;
    for( var i = 0; i < xmldoc.childNodes.length; i++ )
    {
      ;
      if( xmldoc.childNodes[i]
              && xmldoc.childNodes[i].nodeType == Browser.Node.ELEMENT_NODE
              && xmldoc.childNodes[i].nodeName == 'results' )
      {
        ;
        var entrylist = xmldoc.getElementsByTagName('entry');
        ;
        for( var j = 0; j < entrylist.length; j++ )
        {
          var entryElement = entrylist[j];
          ;
          this.resultMap[entryElement.getAttribute('key')] = entryElement.getAttribute('value');
        }
        break;
      }
      else
      {
        ;
      }
    }
  }
}
ServerResult.prototype.getValues = function()
{
  return this.resultMap;
}

/* domapi/Browser.ie.js */
/*
<DEPENDENCIES>
    <DEPENDS>Broker</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>ServerResult</DEPENDS>
</DEPENDENCIES>
*/
function BrowserImpl(sId)
{
	if (sId)
	{
		this.init(sId)
	}
	;
}
BrowserImpl.prototype.init = function(sId)
{
	;
	this.sId = sId;
	this.nSetToplevelCnt = 2007;
	this.nCoverElement = 0;
	this.eventNames = new EventNames();
	this.Node = new NodeTypes();
	this.aEventListeners = new Array();
	this.aUnloadHandles = new Array();
	var me = this;
	this.unloadCallback = function()
	{
		me.unloadHandler();
	}
	window.BrowserOnUnload = this.unloadCallback;
}
BrowserImpl.prototype.getId = function()
{
	;
	return this.sId;
}
BrowserImpl.prototype.getCharacterSet = function()
{
	;
	if (document.charset)
		return document.charset;
	else if (document.characterSet)
		return document.characterSet;
	else
	{
		;
		return undefined;
	}
}
BrowserImpl.prototype.getBodyScrollTop = function()
{
	;
	return document.body.scrollTop;
}
BrowserImpl.prototype.getBodyScrollLeft = function()
{
	;
	return document.body.scrollLeft;
}
BrowserImpl.prototype.unloadCallback = function()
{
	Browser.unloadHandler();
}
BrowserImpl.prototype.unloadHandler = function()
{
	for (var i = 0; i < this.aEventListeners.length; i++)
	{
		this.removeEventListenerFromDomNode(this.aEventListeners[i].domNode,
				this.aEventListeners[i].eventType,
				this.aEventListeners[i].handler
				);
	}
	for (var i = 0; i < this.aUnloadHandles.length; i++)
	{
		this.aUnloadHandles[i].invoke();
	}
}
BrowserImpl.prototype.addUnloadHandle = function(unloadHandle)
{
	this.aUnloadHandles.push(unloadHandle);
}
BrowserImpl.prototype.addEventListener = function(domId, eventType, handler)
{
	;
	var element = this.getElementById(domId);
	this.addEventListenerToDomNode(element, eventType, handler);
}
BrowserImpl.prototype.addEventListenerToDomNode = function(domNode, eventType, handler)
{
	;
	this.aEventListeners.push({ domNode: domNode,
		eventType: eventType,
		handler: handler
	}
			);
	domNode.attachEvent(eventType, handler);
}
BrowserImpl.prototype.addEventListenerToDocument = function(eventType, handler)
{
	;
	this.addEventListenerToDomNode(document, eventType, handler);
}
BrowserImpl.prototype.removeEventListener = function(domId, eventType, handler)
{
	;
	var element = this.getElementById(domId);
	this.removeEventListenerFromDomNode(element, eventType, handler);
}
BrowserImpl.prototype.removeEventListenerFromDomNode = function(domNode, eventType, handler)
{
	;
	domNode.detachEvent(eventType, handler);
}
BrowserImpl.prototype.removeEventListenerFromDocument = function(eventType, handler)
{
	;
	this.removeEventListenerFromDomNode(document, eventType, handler);
}
BrowserImpl.prototype.getElementById = function(domId)
{
	//;
	var elem = document.getElementById(domId);
	if (! elem)
	{
		;
	}
	return elem;
}
BrowserImpl.prototype.changeStyleById = function(domId, styleName, styleValue)
{
	//;
	var elem = this.getElementById(domId);
	if (elem)
	{
		elem.style[styleName] = styleValue;
	}
}
BrowserImpl.prototype.getStyleById = function(domId, styleName)
{
	//;
	var elem = this.getElementById(domId);
	var style = '';
	if (elem)
	{
		style = elem.style[styleName];
	}
	return style;
}
BrowserImpl.prototype.changeStyleClassById = function(domId, styleClass)
{
	var elem = this.getElementById(domId);
	if (elem)
	{
		elem.className = styleClass;
	}
}
BrowserImpl.prototype.getComputedStyleById = function(domId, styleName)
{
	var elem = this.getElementById(domId);
	return this.getComputedStyle(elem, styleName);
}
BrowserImpl.prototype.getComputedStyle = function(node, styleName)
{
	//;
	var style = '';
	if (node)
	{
		style = node.currentStyle[styleName];
	}
	return style;
}
BrowserImpl.prototype.getPixelComputedStyleById = function(domId, styleName)
{
	var elem = this.getElementById(domId);
	return this.getPixelComputedStyle(elem, styleName);
}
BrowserImpl.prototype.getPixelComputedStyle = function(node, styleName)
{
	var style = this.getComputedStyle(node, styleName);
	var iValue = parseInt(style);
	if (isNaN(iValue))
	{
		return 0;
	}
	else
	{
		return iValue;
	}
}
BrowserImpl.prototype.getPixelStyleById = function(domId, styleName)
{
	var elem = this.getElementById(domId);
	return this.getPixelStyle(elem, styleName);
}
BrowserImpl.prototype.getPixelStyle = function(node, styleName)
{
	var style = '';
	if (node)
	{
		style = node.style[styleName];
		var iValue = parseInt(style);
		if (isNaN(iValue))
		{
			return 0;
		}
		else
		{
			return iValue;
		}
	}
	else
	{
		return 0;
	}
}
BrowserImpl.prototype.changePixelStyleById = function(domId, styleName, iValue)
{
	var elem = this.getElementById(domId);
	return this.changePixelStyle(elem, styleName, iValue);
}
BrowserImpl.prototype.changePixelStyle = function(node, styleName, iValue)
{
	if (node)
	{
		node.style[styleName] = parseInt(iValue) + 'px';
	}
}
BrowserImpl.prototype.setImageSourceById = function(domId, srcURL)
{
	//;
	var elem = this.getElementById(domId);
	if (elem)
	{
		elem.src = srcURL;
	}
}
BrowserImpl.prototype.setImageWidthById = function(domId, width)
{
	//;
	var elem = this.getElementById(domId);
	if (elem)
	{
		elem.width = width;
	}
}
BrowserImpl.prototype.eventStopPropagation = function(event)
{
	//;
	event.cancelBubble = true;
}
BrowserImpl.prototype.eventPreventDefault = function(event)
{
	//;
	event.returnValue = false;
}
BrowserImpl.prototype.getKeyCodeFromEvent = function(event)
{
	//;
	return event.keyCode;
}
BrowserImpl.prototype.getOriginalTargetFromEvent = function(event)
{
	//;
	return event.srcElement;
}
BrowserImpl.prototype.getAbsPos = function(domNode, point)
{
	//;
	point.x += domNode.offsetLeft;
	point.y += domNode.offsetTop;
	if (domNode.offsetParent != null)
	{
		if (domNode.offsetParent.tagName != 'BODY')
		{
			this.getAbsPos(domNode.offsetParent, point);
		}
	}
	else if (domNode.tagName == 'MAP') 
	{
		if (domNode.parentNode.tagName != 'BODY')
		{
			point.x = 0;
			point.y = 0;
			this.getAbsPos(domNode.parentNode, point);
		}
	}
}
BrowserImpl.prototype.getAbsPosHtml = function(domNode, point)
{
	//;
	point.x += domNode.offsetLeft;
	point.y += domNode.offsetTop;
	if (domNode.offsetParent != null)
	{
		if (domNode.offsetParent.tagName != 'HTML')
		{
			this.getAbsPosHtml(domNode.offsetParent, point);
		}
	}
	else if (domNode.tagName == 'MAP') 
	{
		if (domNode.parentNode.tagName != 'HTML')
		{
			point.x = 0;
			point.y = 0;
			this.getAbsPosHtml(domNode.parentNode, point);
		}
	}
}
BrowserImpl.prototype.getScrollPos = function(domNode, point)
{
	//;
	point.x += domNode.scrollLeft;
	point.y += domNode.scrollTop;
	//;
	if (domNode.offsetParent != null)
	{
		if (domNode.offsetParent.tagName != 'BODY')
		{
			this.getScrollPos(domNode.offsetParent, point);
		}
	}
	else if (domNode.tagName == 'MAP') 
	{
		if (domNode.parentNode.tagName != 'BODY')
		{
			point.x = 0;
			point.y = 0;
			this.getScrollPos(domNode.parentNode, point);
		}
	}
}
BrowserImpl.prototype.getScreenPos = function(domNode, point)
{
	//;
	point.x += domNode.offsetLeft - domNode.scrollLeft;
	point.y += domNode.offsetTop - domNode.scrollTop;
	if (domNode.offsetParent != null)
	{
		if (domNode.offsetParent.tagName != 'BODY')
		{
			this.getScreenPos(domNode.offsetParent, point);
		}
	}
	else if (domNode.tagName == 'MAP') 
	{
		if (domNode.parentNode.tagName != 'BODY')
		{
			point.x = 0;
			point.y = 0;
			this.getScreenPos(domNode.parentNode, point);
		}
	}
}
BrowserImpl.prototype.getParentLocation = function(domNode, point)
{
	//;
	domNode = domNode.offsetParent;
	if (domNode != null)
	{
		if (domNode.tagName != 'BODY')
		{
			point.x += domNode.offsetLeft + domNode.clientLeft;
			point.y += domNode.offsetTop + domNode.clientTop;
			this.getParentLocation(domNode, point);
		}
	}
}
BrowserImpl.prototype.getLocation = function(domNode, point)
{
	//;
	point.x += domNode.offsetLeft;
	point.y += domNode.offsetTop;
	this.getParentLocation(domNode, point);
}
BrowserImpl.prototype.getChildById = function(domNode, sId)
{
	//;
	var childNode = null;
	for (i in domNode.childNodes)
	{
		if (domNode.childNodes[i].id == sId)
		{
			childNode = domNode.childNodes[i];
			break;
		}
	}
	return childNode;
}
BrowserImpl.prototype.moveTo = function(domNode, point)
{
	//;
	domNode.style.position = 'absolute';
	var parentPos = new Point(0, 0);
	if (domNode.offsetParent)
	{
		this.getAbsPos(domNode.offsetParent, parentPos);
	}
	domNode.style.left = (point.x - parentPos.x) + 'px';
	domNode.style.top = (point.y - parentPos.y) + 'px';
}
BrowserImpl.prototype.moveToLocation = function(domNode, point)
{
	//;
	domNode.style.position = 'absolute';
	var parentPos = new Point(0, 0);
	this.getLocation(domNode, parentPos);
	domNode.style.left = (point.x - parentPos.x) + 'px';
	domNode.style.top = (point.y - parentPos.y) + 'px';
}
BrowserImpl.prototype.setToplevel = function(domNode)
{
	;
	
	
	this.nSetToplevelCnt += 10;
	domNode.style.zIndex = this.nSetToplevelCnt;
}
BrowserImpl.prototype.loadDocument = function(url, onLoadAction, onErrorAction)
{
	var xmldoc = new ActiveXObject("Microsoft.XMLDOM");
	xmldoc.onreadystatechange = function()
	{
		if (xmldoc.readyState == 4)
		{
			if (xmldoc.parseError.errorCode != 0)
			{
				if (onErrorAction)
				{
					onErrorAction.invoke();
				}
				Broker.actionPipeline.unblock(url);
			}
			else
			{
				if (onLoadAction)
				{
					onLoadAction.invoke(xmldoc);
				}
				Broker.actionPipeline.unblock(url);
			}
		}
	}
	Broker.actionPipeline.block(url);
	xmldoc.load(url);
}
BrowserImpl.prototype.loadXMLDocument = function(url, onLoadAction, onErrorAction)
{
	var request = null;
	try
	{
		request = new ActiveXObject("Msxml2.XMLHTTP");
	}
	catch(e)
	{
		try
		{
			request = new ActiveXObject("Microsoft.XMLHTTP");
		}
		catch(e)
		{
			request = null;
		}
	}
	if (request == null)
	{
		if (onErrorAction)
			onErrorAction.invoke("Could not instantiate XMLHttpRequest !");
	}
	else
	{
		var onloadHandler = function()
		{
			if (request.readyState == 4)
			{
				if (request.status == 200)
				{
					if (onLoadAction)
						onLoadAction.invoke(request.responseXML);
				}
				else
				{
					alert(request.statusText);
					if (onErrorAction)
						onErrorAction.invoke("Errors retrieveing XML for url " + url + ". Message is: " + request.statusText);
				}
				Broker.actionPipeline.unblock(url);
			}
		};
		Broker.actionPipeline.block(url);
		request.onreadystatechange = onloadHandler;
		request.open("GET", url, true);
		request.send("");
	}
}
BrowserImpl.prototype.loadXMLDocumentPost = function(url, args, onLoadAction, onErrorAction)
{
	var request = null;
	try
	{
		request = new ActiveXObject("Msxml2.XMLHTTP");
	}
	catch(e)
	{
		try
		{
			request = new ActiveXObject("Microsoft.XMLHTTP");
		}
		catch(e)
		{
			request = null;
		}
	}
	if (request == null)
	{
		if (onErrorAction)
			onErrorAction.invoke("Could not instantiate XMLHttpRequest !");
	}
	else
	{
		var onloadHandler = function()
		{
			if (request.readyState == 4)
			{
				if (request.status == 200)
				{
					if (onLoadAction)
						onLoadAction.invoke(request.responseXML);
				}
				else
				{
					alert(request.statusText);
					if (onErrorAction)
						onErrorAction.invoke("Errors retrieveing XML for url " + url + ". Message is: " + request.statusText);
				}
				Broker.actionPipeline.unblock(url);
			}
		};
		Broker.actionPipeline.block(url);
		request.onreadystatechange = onloadHandler;
		request.open("POST", url, true);
		request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		request.send(args);
	}
}
BrowserImpl.prototype.callURL = function(url, onLoadAction, onErrorAction)
{
	var xmldoc = new ActiveXObject("Microsoft.XMLDOM");
	xmldoc.onreadystatechange = function()
	{
		if (xmldoc.readyState == 4)
		{
			if (xmldoc.parseError.errorCode != 0)
			{
				if (onErrorAction)
				{
					onErrorAction.invoke();
				}
				Broker.actionPipeline.unblock(url);
			}
			else
			{
				if (onLoadAction)
				{
					onLoadAction.invoke();
				}
				Broker.actionPipeline.unblock(url);
			}
		}
	}
	Broker.actionPipeline.block(url);
	xmldoc.load(url);
}
BrowserImpl.prototype.callServer = function(url, onLoadAction, onErrorAction)
{
	var xmldoc = new ActiveXObject("Microsoft.XMLDOM");
	xmldoc.onreadystatechange = function()
	{
		if (xmldoc.readyState == 4)
		{
			if (xmldoc.parseError.errorCode != 0)
			{
				if (onErrorAction)
				{
					onErrorAction.invoke();
				}
				Broker.actionPipeline.unblock(url);
			}
			else
			{
				if (onLoadAction)
				{
					var result = new ServerResult(this.sId + '_result', xmldoc);
					onLoadAction.invoke(result.getValues());
				}
				Broker.actionPipeline.unblock(url);
			}
		}
	}
	Broker.actionPipeline.block(url);
	xmldoc.load(url);
}
BrowserImpl.prototype.getCoverElement = function(elem)
{
	this.nCoverElement++;
	var oCoverElem = document.createElement("IMG");
	oCoverElem.id = 'Browser_sCoverWidgetDomId_' + this.nCoverElement;
	oCoverElem.src = '../af-web/js/img/trans.png';
	if (elem && elem.style)
	{
		var zindex = this.getComputedStyle(elem, 'zIndex');
		if (zindex == 'auto')
		{
			this.setToplevel(oCoverElem);
		}
		else
		{
			oCoverElem.style.zIndex = parseInt(zindex) + 1;
		}
	}
	return oCoverElem;
}
BrowserImpl.prototype.replaceRelative = function(url)
{
	document.location.href = this.getRelativeURL(url);
}
BrowserImpl.prototype.getRelativeURL = function(url)
{
	var relURL;
	var oBaseArray = document.all.tags('BASE');
	var oBase = (oBaseArray && oBaseArray.length) ? oBaseArray[0].href : null;
	if (oBase)
	{
		if (oBase.lastIndexOf('/') == (oBase.length - 1))
		{
			relURL = oBase + url;
		}
		else
		{
			relURL = oBase + '/' + url;
		}
	}
	else
		relURL = url;
	return relURL;
}
BrowserImpl.prototype.getBaseURL = function()
{
	var oBaseArray = document.all.tags('BASE');
	return (oBaseArray && oBaseArray.length) ? oBaseArray[0].href : '';
}
BrowserImpl.prototype.getEventSourceElement = function(event)
{
	return event.srcElement;
}
BrowserImpl.prototype.removeNodeFromParent = function(node)
{
	if (this.orphanageNode == null)
	{
		var bodyNodes = document.getElementsByTagName('body');
		if (bodyNodes.length > 0)
		{
			var bodyNode = bodyNodes[0];
			this.orphanageNode = document.createElement('div');
			this.orphanageNode.id = 'node_orphanage_' + new Date().getTime();
			this.orphanageNode.style.display = 'none';
			bodyNode.appendChild(this.orphanageNode);
		}
	}
	if (this.orphanageNode != null)
	{
		this.orphanageNode.appendChild(node);
	}
}
function EventNames()
{
	this.LOAD = 'onload';
	this.CLICK = 'onclick';
	this.DOUBLECLICK = 'ondblclick';
	this.DRAGDROP = 'ondragdrop';
	this.CHANGE = 'onchange';
	this.MOUSEDOWN = 'onmousedown';
	this.MOUSEUP = 'onmouseup';
	this.MOUSEOVER = 'onmouseover';
	this.MOUSEOUT = 'onmouseout';
	this.MOUSEMOVE = 'onmousemove';
	this.SELECT = 'onselect';
	this.BLUR = 'onblur';
	this.FOCUS = 'onfocus';
	this.KEYPRESS = 'onkeypress';
	this.KEYUP = 'onkeyup';
	this.KEYDOWN = 'onkeydown';
	this.DRAGSTART = 'ondragstart';
	this.SELECTIONCHANGE = 'onselectionchange';
	this.BEFOREPASTE = 'onbeforepaste';
	this.SELECTSTART = 'onselectstart';
	this.MOUSEWHEEL = 'onmousewheel';
}
function NodeTypes()
{
	this.ELEMENT_NODE = 1;
	this.ATTRIBUTE_NODE = 2;
	this.TEXT_NODE = 3;
	this.CDATA_SECTION_NODE = 4;
	this.ENTITY_REFERENCE_NODE = 5;
	this.ENTITY_NODE = 6;
	this.PROCESSING_INSTRUCTION_NODE = 7;
	this.COMMENT_NODE = 8;
	this.DOCUMENT_NODE = 9;
	this.DOCUMENT_TYPE_NODE = 10;
	this.DOCUMENT_FRAGMENT_NODE = 11;
	this.NOTATION_NODE = 12;
}
Browser = new BrowserImpl('ie');

/* domapi/Controller.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function Controller (name, parent)
{
    ;
    if (name) 
        this.init(name, parent);
} 
Controller.prototype.init = function(name, parent)
{
    ;
    this.name = name;
    this.parent = ((parent == null) ? this : parent)
    this.childs = new Array();
    ;
} 
Controller.prototype.addChild = function(child)
{
    this.childs[child.name] = child;
    ;
} 
Controller.prototype.delChild = function(child)
{
    ;
} 

/* domapi/Action.js */
/*
<DEPENDENCIES>
    <DEPENDS>Globals</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function Action (
    that,
    callback          
	
)
{
  	;
    if (arguments.length > 0)
    {
        this.init(that, callback)
    }
} 
Action.prototype.init = function(that, callback)
{
    this.that = that;
	
    this.callback = callback;
    Globals.verbose(12,"[Action.init]");
} 

/* domapi/DomletCtr.js */
/*
<DEPENDENCIES>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Controller</DEPENDS>
    <DEPENDS>Action</DEPENDS>
</DEPENDENCIES>
*/
DomletCtr.prototype             = new Controller();
DomletCtr.prototype.constructor = DomletCtr;
DomletCtr.superclass            = Controller.prototype;
function DomletCtr(domId, name, parent)
{
      ;
    if (domId) 
    {
        this.init(domId, name, parent);
    }
} 
DomletCtr.prototype.init = function(domId, name, parent)
{
    ;
    DomletCtr.superclass.init.call(this, (name == undefined ? domId : name), parent);
    this.domId = domId;
    this.oWidget = Browser.getElementById(domId);
    this.domlets = new Array();
    this.actions = new Array();
    ;
} 
DomletCtr.prototype.addDomlet = function(domlet)
{
    this.domlets[domlet.name] = domlet;
    ;
} 
DomletCtr.prototype.delDomlet = function(domlet)
{
    ;
} 
DomletCtr.prototype.getVisibleAction = function()
{
  return new ObjectMethodAction(this, "setVisible", true);
}
DomletCtr.prototype.getHiddenAction = function()
{
  return new ObjectMethodAction(this, "setVisible", false);
}
DomletCtr.prototype.setVisible = function(visible)
{
    ;
    if ( this.oWidget)
    {
      this.oWidget.style.visibility = (visible ? "visible" : "hidden");
    }
    for(var i in this.domlets)
    {
        this.domlets[i].setStyle("visibility", (visible ? "inherit" : "hidden"));
    }
}

/* domapi/Domlet.js */
/*
<DEPENDENCIES>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Action</DEPENDS>
</DEPENDENCIES>
*/
function Domlet(domId, name, parent, eventTypes)
{
    ;
    if (domId) 
    {
        this.init(domId, name, parent, eventTypes);
    }
} 
Domlet.prototype.init = function(domId, name, parent, eventTypes)
{
    if (!domId)
        ;
    this.domId   = domId;
    this.name = ((name == undefined) ? this.domId : name);
    this.controller = ((parent == null) ? this : parent)
    ;
      var me = this;
      this.eventClick = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'click', event);
      }
      this.eventDragdrop = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'dragdrop', event);
      }
      this.eventChange = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'change', event);
      }
      this.eventMousedown = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'mousedown', event);
      }
      this.eventMouseup = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'mouseup', event);
      }
      this.eventMouseover = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'mouseover', event);
      }
      this.eventMouseout = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'mouseout', event);
      }
      this.eventMousemove = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'mousemove', event);
      }
      this.eventSelect = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'select', event);
      }
      this.eventBlur = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'blur', event);
      }
      this.eventFocus = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'focus', event);
      }
      this.eventKeypress = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'keypress', event);
      }
      this.eventKeyup = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'keyup', event);
      }
      this.eventKeydown = function(eventParam)
      {
        if ( eventParam ) {  var event = eventParam; }
        me.controller.eventFromChild(me.name, 'keydown', event);
      }
    if (eventTypes)
    {
        for (var i = 0; i < eventTypes.length; i++) 
        {
            ;
            this.addEvent(eventTypes[i]);
        }
    }
} 
Domlet.prototype.addEvent = function(eventType)
{
  this.addDelEvent(eventType, true)
} 
Domlet.prototype.addDelEvent = function(eventType, bAdd)
{
  if (bAdd)
  {
    switch (eventType)
    {
  case 'click' :
    Browser.addEventListener(this.domId, Browser.eventNames.CLICK, this.eventClick);
    break;
  case 'dragdrop' :
    Browser.addEventListener(this.domId, Browser.eventNames.DRAGDROP, this.eventDragdrop);
    break;
  case 'change' :
    Browser.addEventListener(this.domId, Browser.eventNames.CHANGE, this.eventChange);
    break;
  case 'mousedown' :
    Browser.addEventListener(this.domId, Browser.eventNames.MOUSEDOWN, this.eventMousedown);
    break;
  case 'mouseup' :
    Browser.addEventListener(this.domId, Browser.eventNames.MOUSEUP, this.eventMouseup);
    break;
  case 'mouseover' :
    Browser.addEventListener(this.domId, Browser.eventNames.MOUSEOVER, this.eventMouseover);
    break;
  case 'mouseout' :
    Browser.addEventListener(this.domId, Browser.eventNames.MOUSEOUT, this.eventMouseout);
    break;
  case 'mousemove' :
    Browser.addEventListener(this.domId, Browser.eventNames.MOUSEMOVE, this.eventMousemove);
    break;
  case 'select' :
    Browser.addEventListener(this.domId, Browser.eventNames.SELECT, this.eventSelect);
    break;
  case 'blur' :
    Browser.addEventListener(this.domId, Browser.eventNames.BLUR, this.eventBlur);
    break;
  case 'focus' :
    Browser.addEventListener(this.domId, Browser.eventNames.FOCUS, this.eventFocus);
    break;
  case 'keypress' :
    Browser.addEventListener(this.domId, Browser.eventNames.KEYPRESS, this.eventKeypress);
    break;
  case 'keyup' :
    Browser.addEventListener(this.domId, Browser.eventNames.KEYUP, this.eventKeyup);
    break;
  case 'keydown' :
    Browser.addEventListener(this.domId, Browser.eventNames.KEYDOWN, this.eventKeydown);
    break;
    }
  }
  else
  {
    switch (eventType)
    {
  case 'click' :
    Browser.removeEventListener(this.domId, Browser.eventNames.CLICK, this.eventClick);
    break;
  case 'dragdrop' :
    Browser.removeEventListener(this.domId, Browser.eventNames.DRAGDROP, this.eventDragdrop);
    break;
  case 'change' :
    Browser.removeEventListener(this.domId, Browser.eventNames.CHANGE, this.eventChange);
    break;
  case 'mousedown' :
    Browser.removeEventListener(this.domId, Browser.eventNames.MOUSEDOWN, this.eventMousedown);
    break;
  case 'mouseup' :
    Browser.removeEventListener(this.domId, Browser.eventNames.MOUSEUP, this.eventMouseup);
    break;
  case 'mouseover' :
    Browser.removeEventListener(this.domId, Browser.eventNames.MOUSEOVER, this.eventMouseover);
    break;
  case 'mouseout' :
    Browser.removeEventListener(this.domId, Browser.eventNames.MOUSEOUT, this.eventMouseout);
    break;
  case 'mousemove' :
    Browser.removeEventListener(this.domId, Browser.eventNames.MOUSEMOVE, this.eventMousemove);
    break;
  case 'select' :
    Browser.removeEventListener(this.domId, Browser.eventNames.SELECT, this.eventSelect);
    break;
  case 'blur' :
    Browser.removeEventListener(this.domId, Browser.eventNames.BLUR, this.eventBlur);
    break;
  case 'focus' :
    Browser.removeEventListener(this.domId, Browser.eventNames.FOCUS, this.eventFocus);
    break;
  case 'keypress' :
    Browser.removeEventListener(this.domId, Browser.eventNames.KEYPRESS, this.eventKeypress);
    break;
  case 'keyup' :
    Browser.removeEventListener(this.domId, Browser.eventNames.KEYUP, this.eventKeyup);
    break;
  case 'keydown' :
    Browser.removeEventListener(this.domId, Browser.eventNames.KEYDOWN, this.eventKeydown);
    break;
    }
  }
}
Domlet.prototype.delEvent = function(eventType)
{
  this.addDelEvent(eventType, false);
} 
Domlet.prototype.setStyle = function(style, value)
{
    ;
    Browser.changeStyleById(this.domId, style, value);
    if (style == 'visibility')
    {
        if (value != 'hidden');
        {
            ;
            Browser.setToplevel(Browser.getElementById(this.domId));        
        }
    }
} 
Domlet.prototype.setStyleClass = function(styleclass)
{
    ;
    Browser.changeStyleClassById(this.domId, styleclass);
} 
Domlet.prototype.setText = function(value)
{
    var e = Browser.getElementById(this.domId);
    e.innerHTML = value;
}
Domlet.prototype.setImageSource = function(imgSrc)
{
    ;
    Browser.setImageSourceById(this.domId, imgSrc);
} 
Domlet.prototype.setImageWidth = function(width)
{
    //;
    Browser.setImageWidthById(this.domId, width);
}
Domlet.prototype.getVisibleAction = function()
{
  return new ObjectMethodAction(this, "setVisible", true);
}
Domlet.prototype.getHiddenAction = function()
{
  return new ObjectMethodAction(this, "setVisible", false);
}
Domlet.prototype.setVisible = function(visible)
{
    ;
    this.setStyle("visibility", (visible ? "inherit" : "hidden"));
}
Domlet.prototype.getStyle = function(style)
{
    return  Browser.getStyleById(this.domId, style)
}
Domlet.prototype.getComputedStyle = function(style)
{
    return  Browser.getComputedStyleById(this.domId, style)
}

/* common/widgets/Division.js */
/*
<DEPENDENCIES>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>DomletCtr</DEPENDS>
    <DEPENDS>Domlet</DEPENDS>
</DEPENDENCIES>
*/
Division.prototype             = new DomletCtr();
Division.prototype.constructor = Division;
Division.superclass            = DomletCtr.prototype;
function Division(domId, name, parent, initialVisible)
{
    ;
    if (domId) 
    {
        this.init(domId, name, parent, initialVisible);
    }
} 
Division.prototype.init = function(domId, name, parent, initialVisible)
{
    Division.superclass.init.call(this, domId, name, parent);
    ;
    this.addDomlet(new Domlet(
        this.domId,     
        this.name,      
        this.parent     
    ));
    this.horizontalCenter = false;
    this.verticalCenter = false;
    this.bActive = true;
    this.initialVisible = initialVisible;
    this.parentDomElement = this.getParentDomElement();
    ;
    this.oWidget = Browser.getElementById(this.domId);
    this.oCoverElem = null;
  this.assignTimerTime = 10;
  this.assignTimer = null;
  var me = this;
  function assignTimerCallback()
  {
    me.assignTimer = null;
    me.oWidget.style.visibility = 'inherit';
  }
  this.assignTimerCallback = assignTimerCallback;
} 
Division.prototype.setHorizontalCenter = function (center)
{
    this.horizontalCenter = (center == undefined) ? true : center;
}
Division.prototype.setVerticalCenter = function (center)
{
    this.verticalCenter = (center == undefined) ? true : center;
}
Division.prototype.start = function ()
{
    if(this.horizontalCenter || this.verticalCenter)
    {
      var parentX = Browser.getPixelComputedStyle(this.parentDomElement, 'width');
      var parentY = Browser.getPixelComputedStyle(this.parentDomElement, 'height');
      ;
      var thisX = this.oWidget.offsetWidth;
      var thisY = this.oWidget.offsetHeight;
      var thisStyleTop = this.oWidget.style['top'];
      var thisStyleBottom = this.oWidget.style['bottom'];
      var thisStyleLeft = this.oWidget.style['left'];
      var thisStyleRight = this.oWidget.style['right'];
      if ((thisStyleLeft == '') && (thisStyleRight == ''))
      {
          if (this.horizontalCenter)
            Browser.changePixelStyle(this.oWidget, 'left', Math.floor((parentX - thisX) / 2));
      }
      if ((thisStyleTop == '') && (thisStyleBottom == ''))
      {
          if (this.verticalCenter)
            Browser.changePixelStyle(this.oWidget, 'top', Math.floor((parentY - thisY) / 2));
      }
    }
    if (this.initialVisible)
        this.assignTimer = window.setTimeout(this.assignTimerCallback, this.assignTimerTime);
}
Division.prototype.setInactive = function()
{
  if(this.bActive)
  {
    this.bActive = false;
    if(this.oWidget.style.visibility != 'hidden')
    {
      this.cover();
    }
  }
}
Division.prototype.cover = function()
{
  if ( !this.oCoverElem )
  {
    this.oCoverElem = Browser.getCoverElement(this.oWidget);
    Browser.changePixelStyle(this.oCoverElem, 'width', this.oWidget.offsetWidth);
    Browser.changePixelStyle(this.oCoverElem, 'height', this.oWidget.offsetHeight);
    this.oCoverElem.style.position = 'absolute';
    this.oCoverElem.style.visibility = 'hidden';
    if (Browser.getId() == 'moz')
    {
        var range = document.createRange();
        range.setStartBefore(this.oWidget);
        range.insertNode(this.oCoverElem);
    }
    else
    {
        this.oWidget.parentNode.appendChild(this.oCoverElem);
    }
    var pAbsPos = new Point(0,0);
    Browser.getAbsPos(this.oWidget, pAbsPos);
    Browser.moveTo(this.oCoverElem, pAbsPos);
    this.oCoverElem.style.visibility = 'inherit';
   }
}
Division.prototype.setActive = function()
{
  if(!this.bActive)
  {
    this.bActive = true;
    if(this.oWidget.style.visibility != 'hidden')
    {
      this.uncover();
    }
  }
}
Division.prototype.uncover = function()
{
  if ( this.oCoverElem )
  {
    Browser.removeNodeFromParent(this.oCoverElem);
    this.oCoverElem = null;
  }
}
Division.prototype.setVisible = function(visible)
{
  Division.superclass.setVisible.call(this, visible);
  if(!this.bActive)
  {
    if(visible)
    {
      this.cover();
    }
    else
    {
      this.uncover();
    }
  }
}
Division.prototype.getSetInactiveAction = function()
{
  return new ObjectMethodAction(this, "setInactive");
}
Division.prototype.getSetActiveAction = function()
{
  return new ObjectMethodAction(this, "setActive");
}
Division.prototype.getParentDomElement = function()
{
    return Browser.getElementById(this.domId).parentNode;
}
Division.prototype.getChangeBorderColorAction = function(borderColor)
{
    return new ObjectMethodAction(this, "changeBorderColor", borderColor);
}
Division.prototype.changeBorderColor = function(borderColor)
{
    ;
    this.domlets[this.name].setStyle("borderColor", borderColor);
}
Division.prototype.getChangeBorderStyleAction = function(borderStyle)
{
    return new ObjectMethodAction(this, "changeBorderStyle", borderStyle);
}
Division.prototype.changeBorderStyle = function(borderStyle)
{
    ;
    this.domlets[this.name].setStyle("borderStyle", borderStyle);
}
Division.prototype.getChangeColorAction = function(color)
{
    return new ObjectMethodAction(this, "changeColor", color);
}
Division.prototype.changeColor = function(color)
{
    ;
    this.domlets[this.name].setStyle("color", color);
}
Division.prototype.getChangeBgColorAction = function(color)
{
    return new ObjectMethodAction(this, "changeBgColor", color);
}
Division.prototype.changeBgColor = function(color)
{
    ;
    this.domlets[this.name].setStyle("backgroundColor", color);
}
Division.prototype.getChangeDisplayStyleAction = function(displayStyle)
{
    return new ObjectMethodAction(this, "changeDisplayStyle", displayStyle);
}
Division.prototype.changeDisplayStyle = function(displayStyle)
{
    ;
    this.domlets[this.name].setStyle("display", displayStyle);
}
Division.prototype.getChangeStyleClassAction = function(styleclass)
{
    return new ObjectMethodAction(this, "changeStyleClass", styleclass);
}
Division.prototype.changeStyleClass= function(styleclass)
{
    this.domlets[this.name].setStyleClass(styleclass);
}

/* toolkit/tip/TipManager.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>Globals</DEPENDS>
</DEPENDENCIES>
*/
function TipManager(sId, bStepByStep, bSolutable, bCycleTips, bInteractionTipEnabled)
{
  this.init(sId, bStepByStep, bSolutable, bCycleTips, bInteractionTipEnabled)
}
TipManager.prototype.init = function(sId, bStepByStep, bSolutable, bCycleTips, bInteractionTipEnabled)
{
  ;
  this.sId = sId;
  this.bStepByStep = bStepByStep;
  this.bSolutable = bSolutable;
  this.bCycleTips = bCycleTips;
  this.bInteractionTipEnabled = (bInteractionTipEnabled != undefined) ? bInteractionTipEnabled : true;
  this.currentTipHandler = null;
  this.aTipHandlers = new Array();
  this.aSolutionHandlers = new Array();
  this.aOnceSuccessfulHandlers = new Array();
  this.nTipHandlers = 0;
  this.nSolutionHandlers = 0;
  this.tipBroker = null;
  this.aInteractionTipHandlers = new Array();
  this.aIndicatingTipHandlers = new Array();
}
TipManager.prototype.start = function()
{
  ;
  for(var i in this.aIndicatingTipHandlers)
  {
    var tipHandler = this.aIndicatingTipHandlers[i];
    this.registerTipHandler(tipHandler);
    this.registerSolutionHandler(tipHandler);
  }
  for(var i in this.aInteractionTipHandlers)
  {
    var tipHandler = this.aInteractionTipHandlers[i];
    if(this.bInteractionTipEnabled)
    {
      this.registerTipHandler(tipHandler);
    }
    this.registerSolutionHandler(tipHandler);
  }
  if(this.nTipHandlers > 0)
  {
    if(this.tipBroker) {this.tipBroker.tipEnabled();}
  }
  else
  {
    if(this.tipBroker) {this.tipBroker.tipDisabled();}
  }
}
TipManager.prototype.register = function(tipHandler)
{
  this.aInteractionTipHandlers[tipHandler.getId()] = tipHandler;
}
TipManager.prototype.registerIndicatingTip = function(tipHandler)
{
  this.aIndicatingTipHandlers[tipHandler.getId()] = tipHandler;
}
TipManager.prototype.registerTipHandler = function(tipHandler)
{
  ;
  if (this.aTipHandlers[tipHandler.getId()] != null) 
  {
    ;
  }  
  else
  {
    this.aTipHandlers[tipHandler.getId()] = tipHandler;
    this.nTipHandlers++;
    ;
    tipHandler.setTipManager(this);
    if (this.currentTipHandler == null) 
    {
      this.setCurrent(tipHandler);
    }
  }
}
TipManager.prototype.registerSolutionHandler = function(tipHandler)
{
  ;
  if (this.aSolutionHandlers[tipHandler.getId()] != null)
  {
    ;
  }
  else
  {
    this.aSolutionHandlers[tipHandler.getId()] = tipHandler;
    this.nSolutionHandlers++;
    ;
    tipHandler.setTipManager(this);
  }
}
TipManager.prototype.setTipBroker = function(tipBroker)
{
  ;
  this.tipBroker = tipBroker;
}
TipManager.prototype.setCurrent = function(tipHandler)
{
  ;
  this.currentTipHandler = tipHandler;
}
TipManager.prototype.showTip = function()
{
  ;
  var bFoundOne = false;
  if (this.currentTipHandler != null)
  {
    var id = this.currentTipHandler.getId();
    if( !this.bInteractionTipEnabled || !this.aIndicatingTipHandlers[id] || !this.aOnceSuccessfulHandlers[id])
    {
      if(this.currentTipHandler.showTip(this.bStepByStep, this.bSolutable, this.bCycleTips))
      {
        ;
        this.aOnceSuccessfulHandlers[this.currentTipHandler.getId()] = true;
        bFoundOne = true;
      }
    }
  }
  if (!bFoundOne) {
    ;
    for (var id in this.aTipHandlers)
    {
      if (this.aTipHandlers[id] != this.currentTipHandler)
      {
        if( !this.bInteractionTipEnabled || !this.aIndicatingTipHandlers[id] || !this.aOnceSuccessfulHandlers[id])
        {
          if (this.aTipHandlers[id].showTip(this.bStepByStep, this.bSolutable, this.bCycleTips))
          {
            ;
            this.aOnceSuccessfulHandlers[id] = true;
            bFoundOne = true;
            break;
          }
        }
      }
    }
  }
  if (!bFoundOne) {
    ;
  }
  if (bFoundOne) {
    this.checkTipEnabled();
  }
}
TipManager.prototype.checkTipEnabled = function()
{
  ;
  var bFoundOne = false;
  for (var id in this.aTipHandlers)
  {
    if (this.aTipHandlers[id] != this.currentTipHandler)
    {
      if (this.aTipHandlers[id].hasTip(this.bStepByStep, this.bSolutable, this.bCycleTips))
      {
        bFoundOne = true;
        break;        
      }
    }
  }
  if (bFoundOne)
  {
    if (this.tipBroker) {this.tipBroker.tipAvailable();}
  }
  else
  {
    if (this.tipBroker) {this.tipBroker.tipUnavailable();}
  }
}
TipManager.prototype.getCheckTipAction = function()
{
  return new ObjectMethodAction(this, "checkTipEnabled");
}
TipManager.prototype.changed = function(workItem)
{
  ;
  this.checkTipEnabled();
}
TipManager.prototype.showSolution = function() 
{
  ;
  for (var id in this.aSolutionHandlers)
  {
    if (this.aSolutionHandlers[id].showSolution) {this.aSolutionHandlers[id].showSolution();}
  }
}
TipManager.prototype.hideSolution = function() 
{
  ;
  for (var id in this.aSolutionHandlers)
  {
    if (this.aSolutionHandlers[id].hideSolution) {this.aSolutionHandlers[id].hideSolution();}
  }
}
TipManager.prototype.getShowSolutionAction = function()
{
  return new ObjectMethodAction(this, "showSolution");
}
TipManager.prototype.getHideSolutionAction = function()
{
  return new ObjectMethodAction(this, "hideSolution");
}
TipManager.prototype.getShowTipAction = function()
{
  return new ObjectMethodAction(this, "showTip");
}
TipManager.prototype.reset = function()
{
  this.aOnceSuccessfulHandlers = new Array();
}
TipManager.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, "reset");
}

/* common/widgets/UniversalPushButton.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>DomletCtr</DEPENDS>
    <DEPENDS>Domlet</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
UniversalPushButton.prototype = new DomletCtr();
UniversalPushButton.prototype.constructor = UniversalPushButton;
UniversalPushButton.superclass = DomletCtr.prototype;
function UniversalPushButton( domId, enabled, name, parent )
{
  ;
  if( domId ) 
  {
    this.init(domId, enabled, name, parent);
  }
}
UniversalPushButton.prototype.init = function( domId, enabled, name, parent )
{
  UniversalPushButton.superclass.init.call(this, domId, name, parent);
  ;
  this.addDomlet(new Domlet(
          this.domId, 
          this.name, 
          this.parent, 
          new Array(
          'mouseout',
          'mouseover',
          'mousedown',
          'mouseup',
          'click'
          )
          ));
  this.state = {
    pressed : false,       
    highlighted : false   
  };
  this.state.enabled = ((enabled == undefined) ? true : enabled);
  this.state.suspend = false;
  this.broker = new Broker('pushButtonBroker');
  ;
} 
UniversalPushButton.prototype.registerBroker = function( broker )
{
  this.broker = broker;
}
UniversalPushButton.prototype.addNormalAction = function( action )
{
  this.broker.registerAction("normalAction", action);
}
UniversalPushButton.prototype.addMouseoverNormalAction = function( action )
{
  this.broker.registerAction("mouseoverNormalAction", action);
}
UniversalPushButton.prototype.addPressedAction = function( action )
{
  this.broker.registerAction("pressedAction", action);
}
UniversalPushButton.prototype.addMouseoverPressedAction = function( action )
{
  this.broker.registerAction("mouseoverPressedAction", action);
}
UniversalPushButton.prototype.addDisabledAction = function( action )
{
  this.broker.registerAction("disabledAction", action);
}
UniversalPushButton.prototype.addDisabledPressedAction = function( action )
{
  this.broker.registerAction("disabledPressedAction", action);
}
UniversalPushButton.prototype.addEnabledAction = function( action )
{
  this.broker.registerAction("enabledAction", action);
}
UniversalPushButton.prototype.addEnabledPressedAction = function( action )
{
  this.broker.registerAction("enabledPressedAction", action);
}
UniversalPushButton.prototype.getEnableAction = function()
{
  return new ObjectMethodAction(this, "setEnabled", true);
}
UniversalPushButton.prototype.getDisableAction = function()
{
  return new ObjectMethodAction(this, "setEnabled", false);
}
UniversalPushButton.prototype.setPressedAction = function( action )
{
  this.pressedAction = action;
}
UniversalPushButton.prototype.setDownAction = function( action )
{
  this.downAction = action;
}
UniversalPushButton.prototype.setUpAction = function( action )
{
  this.upAction = action;
}
UniversalPushButton.prototype.setEnabled = function( enabled )
{
  this.state.enabled = enabled;
  if( this.state.enabled )
  {
    if( this.state.pressed ) this.broker.getMessage("enabledPressedAction");
    else this.broker.getMessage("enabledAction");
  }
  else
  {
    this.resetMouseStatus();
    if( this.state.pressed ) this.broker.getMessage("disabledPressedAction");
    else this.broker.getMessage("disabledAction");
  }
}
UniversalPushButton.prototype.setSuspend = function( suspend )
{
  this.state.suspend = suspend;
}
UniversalPushButton.prototype.resetMouseStatus = function()
{
  this.state.highlighted = false;
  this.state.pressed = false;
  this.broker.getMessage("normalAction");
}
UniversalPushButton.prototype.eventFromChild = function( childName, eventType )
{
  ;
  if( this.state.enabled && (this.broker != null) && !this.state.suspend )
  {
    switch( eventType )
            {
      case "mouseover" :
        this.state.highlighted = true;
        if( this.state.pressed ) this.broker.getMessage("mouseoverPressedAction");
        else this.broker.getMessage("mouseoverNormalAction");
        break;
      case "mouseout" :
        if( this.state.pressed == true )
        {
          this.state.pressed = false;
          this.broker.getMessage("mouseoverNormalAction");
          if( this.upAction )
          {
            ;
            this.upAction.invoke();
          }
        }
        this.resetMouseStatus();
        break;
      case "mousedown" :
        this.state.pressed = true;
        this.broker.getMessage("mouseoverPressedAction");
        if( this.downAction )
        {
          ;
          this.downAction.invoke();
        }
        break;
      case "mouseup" :
        this.state.pressed = false;
        this.broker.getMessage("mouseoverNormalAction");
        if( this.upAction )
        {
          ;
          this.upAction.invoke();
        }
        break;
      case "click" :
        this.state.pressed = false;
        if( this.buttonGroupManager )
          this.buttonGroupManager.buttonPushed(this);
        if( this.pressedAction )
        {
          ;
          this.pressedAction.invoke();
        }
        break;
    }
  }
} 
UniversalPushButton.prototype.getSetPressedAction = function()
{
  return new ObjectMethodAction(this, "setPressed", true);
}
UniversalPushButton.prototype.setPressed = function( bPressed )
{
  ;
  if( bPressed )
  {
    if( this.buttonGroupManager )
      this.buttonGroupManager.buttonPushed(this);
    if( this.pressedAction )
    {
      this.pressedAction.invoke();
    }
  }
}
UniversalPushButton.prototype.setButtonGroupManager = function( buttonGroupManager )
{
  this.buttonGroupManager = buttonGroupManager;
}

/* toolkit/exercise/ExerciseManager.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>ExerciseManagerStatus</DEPENDS>
</DEPENDENCIES>
*/
function ExerciseManagerStatus( nTouched, nTotal, nCorrect, nWrong, nCorrectStatus, nWrongStatus, nCorrectSystem,
                                nWrongSystem, sOrigin )
{
  this.nTouched = nTouched;
  this.nTotal = nTotal;
  this.nCorrect = nCorrect;
  this.nWrong = nWrong;
  this.nCorrectStatus = nCorrectStatus;
  this.nWrongStatus = nWrongStatus;
  this.nCorrectSystem = nCorrectSystem;
  this.nWrongSystem = nWrongSystem;
  this.oWorkItem = null;
  this.sOrigin = sOrigin;
}
ExerciseManagerStatus.prototype.toString = function()
{
  return '[' +
         'items:touched=' + this.nTouched +
         '/total=' + this.nTotal +
         ' ' +
         'total:correct=' + this.nCorrect +
         '/wrong=' + this.nWrong +
         ' ' +
         'status:correct=' + this.nCorrectStatus +
         '/wrong=' + this.nWrongStatus +
         ' ' +
         'sys:const=' + this.nCorrectSystem +
         '/corr=' + this.nWrongSystem +
         ']';
}
ExerciseHandlerStatus.prototype = new ExerciseManagerStatus();
ExerciseHandlerStatus.prototype.constructor = ExerciseHandlerStatus;
ExerciseHandlerStatus.superclass = ExerciseManagerStatus.prototype;
function ExerciseHandlerStatus( nCorrect, nWrong, nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem, oWorkItem,
                                sOrigin )
{
  this.nTouched = -1;
  this.nTotal = -1;
  this.nCorrect = nCorrect;
  this.nWrong = nWrong;
  this.nCorrectStatus = nCorrectStatus;
  this.nWrongStatus = nWrongStatus;
  this.nCorrectSystem = nCorrectSystem;
  this.nWrongSystem = nWrongSystem;
  this.oWorkItem = oWorkItem;
  this.sOrigin = sOrigin;
}
function ExerciseManager( sId )
{
  this.init(sId)
}
ExerciseManager.prototype.init = function( sId )
{
  ;
  this.sId = sId;
  this.aExcHandlers = new Array();
  this.aStatus = new Array();
  this.excStatus = new ExerciseManagerStatus(0, 0, 0, 0, 0, 0, 0, 0);
  this.aTouchedHandler = new Array();
  this.bWorkStarted = false;
  this.bWorkStopped = false;
  this.bHelpApplied = false;
  this.bActivated = false;
  this.bReset = false;
  this.currentExecHandler = null;
  this.oExerciseBroker = null;
  this.aCorrectionStrategys = new Array();
}
ExerciseManager.prototype.setExerciseBroker = function( oExerciseBroker )
{
  ;
  this.oExerciseBroker = oExerciseBroker;
  this.oExerciseBroker.addResetHandle(this.getResetAction());
  this.oExerciseBroker.addEvaluateHandle(this.getEvaluateAction());
}
ExerciseManager.prototype.register = function( excHandler )
{
  ;
  if( this.aExcHandlers[excHandler.getId()] != null )
  {
    ;
  }
  else
  {
    this.aExcHandlers[excHandler.getId()] = excHandler;
    this.excStatus.nTotal++;
    ;
    this.setCurrent(excHandler);
    excHandler.setExerciseManager(this);
  }
}
ExerciseManager.prototype.start = function()
{
  ;
  for( var id in this.aExcHandlers )
  {
    this.aExcHandlers[id].start();
  }
  this.initExcStatus();
}
ExerciseManager.prototype.registerCorrectionStrategy = function( oCorrectionStrategy )
{
  ;
  this.aCorrectionStrategys.push(oCorrectionStrategy);
  oCorrectionStrategy.setExerciseManager(this);
}
ExerciseManager.prototype.initExcStatus = function()
{
  ;
  if( this.excStatus.nTotal == 0 )  
  {
    if( this.oExerciseBroker != null )
    {
      this.oExerciseBroker.solved(
              new ExerciseManagerStatus(1, 1, 1, 0, 1, 0, 0, 0));
    }
  }
  this.excStatus.nTouched = 0;
  this.excStatus.nCorrect = 0;
  this.excStatus.nWrong = 0;
  this.excStatus.nCorrectStatus = 0;
  this.excStatus.nWrongStatus = 0;
  this.excStatus.nCorrectSystem = 0;
  this.excStatus.nWrongSystem = 0;
  for( var id in this.aExcHandlers )
  {
    this.excStatus.nCorrect += this.aExcHandlers[id].getNumCorrect();
    this.excStatus.nWrong += this.aExcHandlers[id].getNumWrong();
  }
}
ExerciseManager.prototype.setCurrent = function( execHandler )
{
  ;
  this.currentExecHandler = execHandler;
}
ExerciseManager.prototype.solved = function( excHandler, excHandlerStatusNew )
{
  ;
  if( this.aExcHandlers[excHandler.getId()] == null )
  {
    ;
  }
  else
  {
    if( !this.aTouchedHandler[excHandler.getId()] )
    {
      if( excHandlerStatusNew.oWorkItem.bWorkStarted )
      {
        ;
        this.aTouchedHandler[excHandler.getId()] = true;
        this.excStatus.nTouched++;
      }
    }
    var excHandlerStatusOld = this.aStatus[excHandler.getId()];
    if( excHandlerStatusOld != null )
    {
      ;
      this.excStatus.nCorrectStatus -= excHandlerStatusOld.nCorrectStatus;
      this.excStatus.nWrongStatus -= excHandlerStatusOld.nWrongStatus;
      this.excStatus.nCorrectSystem -= excHandlerStatusOld.nCorrectSystem;
      this.excStatus.nWrongSystem -= excHandlerStatusOld.nWrongSystem;
      if( this.excStatus.nCorrectStatus < 0 )
        this.excStatus.nCorrectStatus = 0;
      if( this.excStatus.nWrongStatus < 0 )
        this.excStatus.nWrongStatus = 0;
      if( this.excStatus.nCorrectSystem < 0 )
        this.excStatus.nCorrectSystem = 0;
      if( this.excStatus.nWrongSystem < 0 )
        this.excStatus.nWrongSystem = 0;
    }
    if( excHandlerStatusNew.oWorkItem.bWorkStarted )
    {
      this.aStatus[excHandler.getId()] = excHandlerStatusNew;
      this.excStatus.nCorrectStatus += excHandlerStatusNew.nCorrectStatus;
      this.excStatus.nWrongStatus += excHandlerStatusNew.nWrongStatus;
      this.excStatus.nCorrectSystem += excHandlerStatusNew.nCorrectSystem;
      this.excStatus.nWrongSystem += excHandlerStatusNew.nWrongSystem;
      this.excStatus.sOrigin = excHandlerStatusNew.sOrigin;
    }
    this.setCurrent(excHandler);
    ;
    if( !this.bWorkStarted && !this.bReset )
    {
      if( excHandlerStatusNew.oWorkItem.bWorkStarted )
      {
        this.bWorkStarted = true;
        if( this.oExerciseBroker )
        {
          this.oExerciseBroker.doWorkStarted();
        }
      }
    }
    if( this.oExerciseBroker )
    {
      if( !this.bWorkStopped )
      {
        var bWorkStopped = true;
        for( var i in this.aExcHandlers )
        {
          if( ! this.aExcHandlers[i].bWorkStopped )
          {
            bWorkStopped = false;
          }
        }
        if( bWorkStopped )
        {
          this.oExerciseBroker.doWorkStopped();
        }
        this.bWorkStopped = bWorkStopped;
      }
      if( this.bWorkStarted && !this.bWorkStopped )
      {
        this.oExerciseBroker.doSolved(this.excStatus);
        this.oExerciseBroker.doWorkItemSolved(excHandlerStatusNew);
      }
      if( ! this.bHelpApplied && ((this.excStatus.nCorrectSystem > 0) || (this.excStatus.nWrongSystem > 0)) )
      {
        this.bHelpApplied = true;
        this.oExerciseBroker.doHelpApplied();
      }
    }
  }
}
ExerciseManager.prototype.evaluate = function()
{
  ;
  for( var id in this.aExcHandlers )
  {
    if( this.aExcHandlers[id].evaluate )
    {
      this.aExcHandlers[id].evaluate();
    }
  }
}
ExerciseManager.prototype.reset = function()
{
  ;
  this.bReset = true;
  if( this.oExerciseBroker )
  {
    var bWorkStarted = false;
    for( var id in this.aExcHandlers )
    {
      if( this.aExcHandlers[id].bWorkStarted )
      {
        bWorkStarted = true;
        break;
      }
    }
    if( bWorkStarted )
    {
      this.oExerciseBroker.doWorkAborted();
    }
  }
  for( var id in this.aExcHandlers )
  {
    if( this.aExcHandlers[id].reset )
    {
      this.aExcHandlers[id].reset();
    }
  }
  this.initExcStatus();
  this.aStatus = new Array();
  this.aTouchedHandler = new Array();
  this.bHelpApplied = false;
  this.bActivated = false;
  this.bWorkStarted = false;
  this.bWorkStopped = false;
  this.bReset = false;
}
ExerciseManager.prototype.beginCorrection = function()
{
  ;
  if( this.oExerciseBroker )
  {
    this.oExerciseBroker.beginCorrection();
  }
}
ExerciseManager.prototype.endCorrection = function()
{
  ;
  if( this.oExerciseBroker )
  {
    this.oExerciseBroker.endCorrection();
  }
}
ExerciseManager.prototype.activateExercise = function()
{
  if( !this.bActivated )
  {
    this.bActivated = true;
    if( this.oExerciseBroker )
    {
      this.oExerciseBroker.activated();
    }
  }
}
ExerciseManager.prototype.workItemStarted = function( exerciseHandler )
{
  if( this.oExerciseBroker )
  {
    this.oExerciseBroker.workItemStarted();
  }
}
ExerciseManager.prototype.workItemStopped = function( exerciseHandler )
{
  if( this.oExerciseBroker )
  {
    this.oExerciseBroker.workItemStopped();
  }
}
ExerciseManager.prototype.workItemAborted = function( exerciseHandler )
{
  if( this.oExerciseBroker )
  {
    this.oExerciseBroker.workItemAborted();
  }
}
ExerciseManager.prototype.workItemActivated = function( exerciseHandler )
{
  if( this.oExerciseBroker )
  {
    this.oExerciseBroker.workItemActivated();
  }
  this.activateExercise();
}
ExerciseManager.prototype.getResetAction = function()
{
  ;
  return new ObjectMethodAction(this, "reset");
}
ExerciseManager.prototype.getEvaluateAction = function()
{
  return new ObjectMethodAction(this, 'evaluate');
}

/* common/widgets/state/StateBroker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Globals</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
</DEPENDENCIES>
*/
StateBroker.prototype             = new Broker();
StateBroker.prototype.constructor = StateBroker;
StateBroker.superclass            = Broker.prototype;
function StateBroker(sId, conjunction, affirmation)
{
  if (!sId) sId = "StateBroker";
  this.init(sId, conjunction, affirmation);
}
StateBroker.prototype.init = function(sId, conjunction, affirmation)
{
  ;
  this.sId = sId;
  this.globalState = false;
  this.subStates = new Array();
  if(affirmation != undefined)
    this.affirmation = affirmation;
  else
    this.affirmation = false;
  if(conjunction != undefined)
    this.conjunction = conjunction;
  else
    this.conjunction = false;
  StateBroker.superclass.init.call(this, sId);
}
StateBroker.prototype.initSet = function(setState)
{
    this.globalState = setState;
}
StateBroker.prototype.setSubstate = function(name)
{
  this.subStates[name] = true;
  this.doSetSubstate(name);
  this.checkGlobalState();
}
StateBroker.prototype.resetSubstate = function(name)
{
  this.subStates[name] = false;
  this.doResetSubstate(name);
  this.checkGlobalState();
}
StateBroker.prototype.initSubstate = function(name)
{
  ;
  var bFound = false;
  for(var key in this.subStates)
  {
     if(key == name)
     {
        bFound = true;
        break;
     }
  }
  if(!bFound)
  {
    this.subStates[name] = false;
    ;
  }
}
StateBroker.prototype.checkGlobalState = function()
{
  var localState = false;
  if(this.conjunction)
  {
      localState = true;
      for (i in this.subStates)
      {
        localState = this.subStates[i] && localState;
        ;
      }
      ;
  }
  else
  {
      localState = false;
      for (i in this.subStates)
      {
        localState = this.subStates[i] || localState;
        ;
      }
      ;
  }
  if (localState)
  {
    if (!this.globalState || this.affirmation)
    {
        this.setGlobalState();
    }
  }
  else
  {
    if (this.globalState  || this.affirmation)
    {
        this.resetGlobalState();
    }
  }
}
StateBroker.prototype.setGlobalState = function()
{
  this.globalState = true;
  this.doSetState();
}
StateBroker.prototype.resetGlobalState = function()
{
  this.globalState = false;
  this.doResetState();
}
StateBroker.prototype.doSetSubstate = function(name)
{
  this.getMessage('setSubstate', [name]);
}
StateBroker.prototype.addSetSubstateHandle = function(action)
{
  this.registerAction('setSubstate', action);
}
StateBroker.prototype.getSetSubstateAction = function(name)
{
  this.initSubstate(name);
  return new ObjectMethodAction(this, 'setSubstate', name);
}
StateBroker.prototype.doResetSubstate = function(name)
{
  this.getMessage('resetSubstate', [name]);
}
StateBroker.prototype.addResetSubstateHandle = function(action)
{
  this.registerAction('resetSubstate', action);
}
StateBroker.prototype.getResetSubstateAction = function(name)
{
  this.initSubstate(name);
  return new ObjectMethodAction(this, 'resetSubstate', name);
}
StateBroker.prototype.doSetState = function()
{
  this.getMessage('setState');
}
StateBroker.prototype.addSetStateHandle = function(action)
{
  this.registerAction('setState', action);
}
StateBroker.prototype.getSetStateAction = function()
{
  return new ObjectMethodAction(this, 'setGlobalState');
}
StateBroker.prototype.doResetState = function()
{
  this.getMessage('resetState');
}
StateBroker.prototype.addResetStateHandle = function(action)
{
  this.registerAction('resetState', action);
}
StateBroker.prototype.getResetStateAction = function()
{
  return new ObjectMethodAction(this, 'resetGlobalState');
}

/* toolkit/exercise/ExerciseBroker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
ExerciseBroker.prototype             = new Broker();
ExerciseBroker.prototype.constructor = ExerciseBroker;
ExerciseBroker.superclass            = Broker.prototype;
function ExerciseBroker()
{
    ;
    this.exerciseController = null;
     ExerciseBroker.superclass.init.call(this, "ExerciseBroker");
}
ExerciseBroker.prototype.setExerciseController = function(exerciseController)
{
    this.exerciseController = exerciseController;
}
ExerciseBroker.prototype.doWorkAborted = function()
{
    ;
    this.getMessage("workAborted");
}
ExerciseBroker.prototype.addWorkAbortedHandle = function(action)
{
    this.registerAction("workAborted", action);
}
ExerciseBroker.prototype.getWorkAbortedAction = function()
{
    return new ObjectMethodAction(this, "workAborted");
}
ExerciseBroker.prototype.workAborted = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldWorkAborted();
    else
        this.doWorkAborted();
}
ExerciseBroker.prototype.doHelpApplied = function()
{
    ;
    this.getMessage("helpApplied");
}
ExerciseBroker.prototype.addHelpAppliedHandle = function(action)
{
    this.registerAction("helpApplied", action);
}
ExerciseBroker.prototype.getHelpAppliedAction = function(action)
{
    return new ObjectMethodAction(this, "helpApplied");
}
ExerciseBroker.prototype.helpApplied = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldHelpApplied();
    else
        this.doHelpApplied();
}
ExerciseBroker.prototype.doWorkStarted = function()
{
    ;
    this.getMessage("workStarted");
}
ExerciseBroker.prototype.addWorkStartedHandle = function(action)
{
    this.registerAction("workStarted", action);
}
ExerciseBroker.prototype.getWorkStartedAction = function(action)
{
    return new ObjectMethodAction(this, "workStarted");
}
ExerciseBroker.prototype.workStarted = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldWorkStarted();
    else
        this.doWorkStarted();
}
ExerciseBroker.prototype.doSolved = function(para1, para2)
{
    ;
    var paramArray = [para1, para2];
    ;
    this.getMessage("solved", paramArray);
}
ExerciseBroker.prototype.addSolvedHandle = function(action)
{
    this.registerAction("solved", action);
}
ExerciseBroker.prototype.getSolvedAction = function(action)
{
    return new ObjectMethodAction(this, "solved");
}
ExerciseBroker.prototype.solved = function(para1, para2)
{
    if (this.exerciseController)
        this.exerciseController.shouldSolved(para1, para2);
    else
        this.doSolved(para1, para2);
}
ExerciseBroker.prototype.doWorkItemSolved = function(para1, para2)
{
    ;
    var paramArray = [para1, para2];
    ;
    this.getMessage("workItemWorkItemSolved", paramArray);
}
ExerciseBroker.prototype.addWorkItemSolvedHandle = function(action)
{
    this.registerAction("workItemWorkItemSolved", action);
}
ExerciseBroker.prototype.getWorkItemSolvedAction = function(action)
{
    return new ObjectMethodAction(this, "workItemWorkItemSolved");
}
ExerciseBroker.prototype.workItemWorkItemSolved = function(para1, para2)
{
    if (this.exerciseController)
        this.exerciseController.shouldWorkItemSolved(para1, para2);
    else
        this.doWorkItemSolved(para1, para2);
}
ExerciseBroker.prototype.doWorkStopped = function()
{
    ;
    this.getMessage("workStopped");
}
ExerciseBroker.prototype.addWorkStoppedHandle = function(action)
{
    this.registerAction("workStopped", action);
}
ExerciseBroker.prototype.getWorkStoppedAction = function(action)
{
    return new ObjectMethodAction(this, "workStopped");
}
ExerciseBroker.prototype.workStopped = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldWorkStopped();
    else
        this.doWorkStopped();
}
ExerciseBroker.prototype.doActivated = function()
{
    ;
    this.getMessage("activated");
}
ExerciseBroker.prototype.addActivatedHandle = function(action)
{
    this.registerAction("activated", action);
}
ExerciseBroker.prototype.getActivatedAction = function(action)
{
    return new ObjectMethodAction(this, "activated");
}
ExerciseBroker.prototype.activated = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldActivated();
    else
        this.doActivated();
}
ExerciseBroker.prototype.doWorkItemStarted = function()
{
    ;
    this.getMessage("workItemStarted");
}
ExerciseBroker.prototype.addWorkItemStartedHandle = function(action)
{
    this.registerAction("workItemStarted", action);
}
ExerciseBroker.prototype.getWorkItemStartedAction = function(action)
{
    return new ObjectMethodAction(this, "workItemStarted");
}
ExerciseBroker.prototype.workItemStarted = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldWorkItemStarted();
    else
        this.doWorkItemStarted();
}
ExerciseBroker.prototype.doWorkItemStopped = function()
{
    ;
    this.getMessage("workItemStopped");
}
ExerciseBroker.prototype.addWorkItemStoppedHandle = function(action)
{
    this.registerAction("workItemStopped", action);
}
ExerciseBroker.prototype.getWorkItemStoppedAction = function(action)
{
    return new ObjectMethodAction(this, "workItemStopped");
}
ExerciseBroker.prototype.workItemStopped = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldWorkItemStopped();
    else
        this.doWorkItemStopped();
}
ExerciseBroker.prototype.doWorkItemAborted = function()
{
    ;
    this.getMessage("workItemAborted");
}
ExerciseBroker.prototype.addWorkItemAbortedHandle = function(action)
{
    this.registerAction("workItemAborted", action);
}
ExerciseBroker.prototype.getWorkItemAbortedAction = function(action)
{
    return new ObjectMethodAction(this, "workItemAborted");
}
ExerciseBroker.prototype.workItemAborted = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldWorkItemAborted();
    else
        this.doWorkItemAborted();
}
ExerciseBroker.prototype.doWorkItemActivated = function()
{
    ;
    this.getMessage("workItemActivated");
}
ExerciseBroker.prototype.addWorkItemActivatedHandle = function(action)
{
    this.registerAction("workItemActivated", action);
}
ExerciseBroker.prototype.getWorkItemActivatedAction = function(action)
{
    return new ObjectMethodAction(this, "workItemActivated");
}
ExerciseBroker.prototype.workItemActivated = function()
{
    if (this.exerciseController)
        this.exerciseController.shouldWorkItemActivated();
    else
        this.doWorkItemActivated();
}
ExerciseBroker.prototype.doEvaluate = function()
{
  ;
  this.getMessage("evaluate");
}
ExerciseBroker.prototype.addEvaluateHandle = function(action)
{
  this.registerAction("evaluate", action);
}
ExerciseBroker.prototype.getEvaluateAction = function()
{
  return new ObjectMethodAction(this, "evaluate");
}
ExerciseBroker.prototype.evaluate = function()
{
  if (this.exerciseController)
  {
    this.exerciseController.shouldEvaluate();
  }
  else
  {
    this.doEvaluate();
  }
}
ExerciseBroker.prototype.doReset = function()
{
  ;
  this.getMessage("reset");
}
ExerciseBroker.prototype.addResetHandle = function(action)
{
  this.registerAction("reset", action);
}
ExerciseBroker.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, "reset");
}
ExerciseBroker.prototype.reset = function()
{
  if (this.exerciseController)
  {
    this.exerciseController.shouldReset();
  }
  else
  {
    this.doReset();
  }
}
ExerciseBroker.prototype.doBeginCorrection = function()
{
  ;
  this.getMessage("beginCorrection");
}
ExerciseBroker.prototype.addBeginCorrectionHandle = function(action)
{
  this.registerAction("beginCorrection", action);
}
ExerciseBroker.prototype.getBeginCorrectionAction = function()
{
  return new ObjectMethodAction(this, "beginCorrection");
}
ExerciseBroker.prototype.beginCorrection = function()
{
  if (this.exerciseController)
  {
    this.exerciseController.shouldBeginCorrection();
  }
  else
  {
    this.doBeginCorrection();
  }
}
ExerciseBroker.prototype.doEndCorrection = function()
{
  ;
  this.getMessage("endCorrection");
}
ExerciseBroker.prototype.addEndCorrectionHandle = function(action)
{
  this.registerAction("endCorrection", action);
}
ExerciseBroker.prototype.getEndCorrectionAction = function()
{
  return new ObjectMethodAction(this, "endCorrection");
}
ExerciseBroker.prototype.endCorrection = function()
{
  if (this.exerciseController)
  {
    this.exerciseController.shouldEndCorrection();
  }
  else
  {
    this.doEndCorrection();
  }
}

/* toolkit/qea/WorkItemBroker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
WorkItemBroker.prototype             = new Broker();
WorkItemBroker.prototype.constructor = WorkItemBroker;
WorkItemBroker.superclass            = Broker.prototype;
function WorkItemBroker(name)
{
  this.name = name;
  ;
  this.workItemController = null;
  WorkItemBroker.superclass.init.call(this, name);
}
WorkItemBroker.prototype.setWorkItemController = function(workItemController)
{
    this.workItemController = workItemController;
}
WorkItemBroker.prototype.doWorkAborted = function()
{
    ;
    this.getMessage("workAborted");
}
WorkItemBroker.prototype.addWorkAbortedHandle = function(action)
{
    this.registerAction("workAborted", action);
}
WorkItemBroker.prototype.getWorkAbortedAction = function(action)
{
    return new ObjectMethodAction(this, "workAborted");
}
WorkItemBroker.prototype.workAborted = function()
{
    if (this.workItemController)
        this.workItemController.shouldWorkAborted();
    else
        this.doWorkAborted();
}
WorkItemBroker.prototype.doHelpApplied = function()
{
    ;
    this.getMessage("helpApplied");
}
WorkItemBroker.prototype.addHelpAppliedHandle = function(action)
{
    this.registerAction("helpApplied", action);
}
WorkItemBroker.prototype.getHelpAppliedAction = function(action)
{
    return new ObjectMethodAction(this, "helpApplied");
}
WorkItemBroker.prototype.helpApplied = function()
{
    if (this.workItemController)
        this.workItemController.shouldHelpApplied();
    else
        this.doHelpApplied();
}
WorkItemBroker.prototype.doWorkStarted = function()
{
    ;
    this.getMessage("workStarted");
}
WorkItemBroker.prototype.addWorkStartedHandle = function(action)
{
    this.registerAction("workStarted", action);
}
WorkItemBroker.prototype.getWorkStartedAction = function(action)
{
    return new ObjectMethodAction(this, "workStarted");
}
WorkItemBroker.prototype.workStarted = function()
{
    if (this.workItemController)
        this.workItemController.shouldWorkStarted();
    else
        this.doWorkStarted();
}
WorkItemBroker.prototype.doSolved = function(para1, para2)
{
    ;
    ;
    var paramArray = [para1, para2];
    ;
    this.getMessage("solved", paramArray);
}
WorkItemBroker.prototype.addSolvedHandle = function(action)
{
    this.registerAction("solved", action);
}
WorkItemBroker.prototype.getSolvedAction = function(action)
{
    return new ObjectMethodAction(this, "solved");
}
WorkItemBroker.prototype.solved = function(para1, para2)
{
    if (this.workItemController)
        this.workItemController.shouldSolved(para1, para2);
    else
        this.doSolved(para1, para2);
}
WorkItemBroker.prototype.doWorkStopped = function()
{
    ;
    this.getMessage("workStopped");
}
WorkItemBroker.prototype.addWorkStoppedHandle = function(action)
{
    this.registerAction("workStopped", action);
}
WorkItemBroker.prototype.getWorkStoppedAction = function(action)
{
    return new ObjectMethodAction(this, "workStopped");
}
WorkItemBroker.prototype.workStopped = function()
{
    if (this.workItemController)
        this.workItemController.shouldWorkStopped();
    else
        this.doWorkStopped();
}
WorkItemBroker.prototype.doActivated = function()
{
    ;
    this.getMessage("activated");
}
WorkItemBroker.prototype.addActivatedHandle = function(action)
{
    this.registerAction("activated", action);
}
WorkItemBroker.prototype.getActivatedAction = function(action)
{
    return new ObjectMethodAction(this, "activated");
}
WorkItemBroker.prototype.activated = function()
{
    if (this.workItemController)
        this.workItemController.shouldActivated();
    else
        this.doActivated();
}
WorkItemBroker.prototype.doNoticed = function()
{
    ;
    this.getMessage("noticed");
}
WorkItemBroker.prototype.addNoticedHandle = function(action)
{
    this.registerAction("noticed", action);
}
WorkItemBroker.prototype.getNoticedAction = function(action)
{
    return new ObjectMethodAction(this, "noticed");
}
WorkItemBroker.prototype.noticed = function()
{
    if (this.workItemController)
        this.workItemController.shouldNoticed();
    else
        this.doNoticed();
}
WorkItemBroker.prototype.doEvaluate = function()
{
    ;
  this.getMessage("evaluate");
}
WorkItemBroker.prototype.addEvaluateHandle = function(action)
{
  this.registerAction("evaluate", action);
}
WorkItemBroker.prototype.getEvaluateAction = function()
{
  return new ObjectMethodAction(this, "evaluate");
}
WorkItemBroker.prototype.evaluate = function()
{
  if (this.workItemController)
  {
    this.workItemController.shouldEvaluate();
  }
  else
  {
    this.doEvaluate();
  }
}
WorkItemBroker.prototype.doReset = function()
{
    ;
  this.getMessage("reset");
}
WorkItemBroker.prototype.addResetHandle = function(action)
{
  this.registerAction("reset", action);
}
WorkItemBroker.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, "reset");
}
WorkItemBroker.prototype.reset = function()
{
  if (this.workItemController)
  {
    this.workItemController.shouldReset();
  }
  else
  {
    this.doReset();
  }
}
WorkItemBroker.prototype.doShowSolution = function()
{
    ;
  this.getMessage("showSolution");
}
WorkItemBroker.prototype.addShowSolutionHandle = function(action)
{
  this.registerAction("showSolution", action);
}
WorkItemBroker.prototype.getShowSolutionAction = function()
{
  return new ObjectMethodAction(this, "showSolution");
}
WorkItemBroker.prototype.showSolution = function()
{
  if (this.workItemController)
  {
    this.workItemController.shouldShowSolution();
  }
  else
  {
    this.doShowSolution();
  }
}
WorkItemBroker.prototype.doHideSolution = function()
{
    ;
  this.getMessage("hideSolution");
}
WorkItemBroker.prototype.addHideSolutionHandle = function(action)
{
  this.registerAction("hideSolution", action);
}
WorkItemBroker.prototype.getHideSolutionAction = function()
{
  return new ObjectMethodAction(this, "hideSolution");
}
WorkItemBroker.prototype.hideSolution = function()
{
  if (this.workItemController)
  {
    this.workItemController.shouldHideSolution();
  }
  else
  {
    this.doHideSolution();
  }
}

/* common/widgets/UserInteractionController.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
function UserInteractionController(sId, exerciseBroker, tipBroker, workItemBroker)
{
  this.init(sId, exerciseBroker, tipBroker, workItemBroker);
}
UserInteractionController.prototype.init = function(sId, exerciseBroker, tipBroker, workItemBroker)
{
  ;
  this.sId = sId;
  this.exerciseBroker = exerciseBroker;
  this.workItemBroker = workItemBroker;
  this.tipBroker = tipBroker;
  this.bSoluted = false;
  this.bStopped = false;
  this.bInteractionAllowed = true;
  this.oBroker = null;
  if(this.exerciseBroker)
  {
    this.exerciseBroker.addWorkStoppedHandle(this.getSetStoppedAction());
    this.exerciseBroker.addResetHandle(this.getResetStoppedAction());
    this.exerciseBroker.addResetHandle(this.getResetSolutedAction());
  }
  if(this.tipBroker)
  {
    this.tipBroker.addShowSolutionHandle(this.getSetSolutedAction());
    this.tipBroker.addHideSolutionHandle(this.getResetSolutedAction());
  }
  if(this.workItemBroker)
  {
    this.workItemBroker.addWorkStoppedHandle(this.getSetStoppedAction());
  }
}
UserInteractionController.prototype.setUserInteractionBroker = function(userInteractionBroker)
{
  this.oBroker = userInteractionBroker;
}
UserInteractionController.prototype.getSetSolutedAction = function()
{
  return new ObjectMethodAction(this, "setSoluted", true);
}
UserInteractionController.prototype.getResetSolutedAction = function()
{
  return new ObjectMethodAction(this, "setSoluted", false);
}
UserInteractionController.prototype.getSetStoppedAction = function()
{
  return new ObjectMethodAction(this, "setStopped", true);
}
UserInteractionController.prototype.getResetStoppedAction = function()
{
  return new ObjectMethodAction(this, "setStopped", false);
}
UserInteractionController.prototype.setSoluted = function(bSoluted)
{
  this.bSoluted = bSoluted;
  this.checkState();
}
UserInteractionController.prototype.setStopped = function(bStopped)
{
  this.bStopped = bStopped;
  this.checkState();
}
UserInteractionController.prototype.checkState = function()
{
  if(this.bSoluted || this.bStopped)
  {
    this.forbidInteraction();
  }
  else
  {
    this.allowInteraction();
  }
}
UserInteractionController.prototype.allowInteraction = function()
{
  if(!this.bInteractionAllowed)
  {
    this.bInteractionAllowed = true;
    if(this.oBroker)
    {
      this.oBroker.allowInteraction();
    }
  }
}
UserInteractionController.prototype.forbidInteraction = function()
{
  if(this.bInteractionAllowed)
  {
    this.bInteractionAllowed = false;
    if(this.oBroker)
    {
      this.oBroker.forbidInteraction();
    }
  }
}

/* common/widgets/SimpleImage.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>DomletCtr</DEPENDS>
    <DEPENDS>Domlet</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
SimpleImage.prototype             = new DomletCtr();
SimpleImage.prototype.constructor = SimpleImage;
SimpleImage.superclass            = DomletCtr.prototype;
function SimpleImage(domId, name, parent)
{
    ;
    if (domId) 
    {
        this.init(domId, name, parent);
    }
} 
SimpleImage.prototype.init = function(domId, name, parent)
{
    SimpleImage.superclass.init.call(this, domId, name, parent);
    ;
    this.addDomlet(new Domlet(
        this.domId,     
        this.name,      
        this.parent
    ));
    ;
  this.eventhandler = function(event)
  {
    ;
    Browser.eventPreventDefault(event);
  };
  Browser.addEventListener(domId, 'mousedown', this.eventhandler, false);
}
SimpleImage.prototype.getChangeSourceAction = function(srcURL)
{
    return new ObjectMethodAction(this, "setImage", srcURL);
}
SimpleImage.prototype.setImage = function(srcURL)
{
	;
	this.domlets[this.name].setImageSource(srcURL);
}

/* common/widgets/SimpleTooltip.js */
/*
<DEPENDENCIES>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Domlet</DEPENDS>
    <DEPENDS>DomletCtr</DEPENDS>
</DEPENDENCIES>
*/
function Tooltip(groupName, sText, sDomId, sCssClass)
{
	if (!groupName)
		groupName = 'DEFAULT';
	this.tooltip = Tooltip_getInstance('SimpleTooltipGroup_' + groupName);
	this.tooltip.setPosOffset(2, -2);
	this.sDomId = sDomId;
	this.sCssClass = sCssClass;
	if (sText && sDomId && sCssClass)
		this.tooltip.registerTooltip(sText, sDomId, sCssClass);
}
Tooltip.prototype.setHideTimeout = function(nHideTimeout)
{
	this.tooltip.setHideTimeout(nHideTimeout);
}
Tooltip.prototype.setShowTimeout = function(nShowTimeout)
{
	this.tooltip.setShowTimeout(nShowTimeout);
}
Tooltip.prototype.getChangeTooltipTextAction = function(tooltiptext)
{
	return new ObjectMethodAction(this, "changeTooltipText", tooltiptext);
}
Tooltip.prototype.changeTooltipText = function(tooltiptext)
{
	this.sText = tooltiptext;
	if (this.sText && this.sDomId && this.sCssClass)
	{
		var domlet = this.tooltip.domlets[this.sDomId];
		if (domlet != null && domlet != undefined)
		{
			function Container(sText, sDomId, sCssClass, nIndex)
			{
				this.sText = sText;
				this.sDomId = sDomId;
				this.sCssClass = sCssClass;
				this.nIndex = nIndex;
			}
			domlet.tooltipData = new Container(this.sText, this.sDomId, this.sCssClass);
			this.tooltip.oWidget.innerHTML = tooltiptext;
		}
	}
}
SimpleTooltip.prototype = new DomletCtr();
SimpleTooltip.prototype.constructor = SimpleTooltip;
SimpleTooltip.superclass = DomletCtr.prototype;
var staticTooltip = new Array;
Tooltip_getInstance = function(sId)
{
	var Instance = null;
	if (staticTooltip[sId])
	{
		Instance = staticTooltip[sId];
	}
	else
	{
		Instance = new SimpleTooltip(sId);
	}
	return Instance;
}
function SimpleTooltip(sId)
{
	if (sId)
	{
		this.init(sId);
	}
}
SimpleTooltip.prototype.init = function(sId)
{
	;
	this.sId = sId;
	this.XOffset = null;
	this.YOffset = null;
	this.nWidth = null;
	this.nHeight = null;
	this.nHideTipTimeoutId = null;
	this.nShowTipTimeoutId = null;
	this.nShowTimeout = 1000;
	this.nHideTimeout = 4000;
	staticTooltip[sId] = this;
	this.sWidgetDomId = sId + 'tooltip_dyn_div';
	this.oWidget = document.createElement("DIV");
	this.oWidget.id = this.sWidgetDomId;
	this.oWidget.style.visibility = 'hidden';
	this.oWidget.style.position = 'absolute';
	document.body.appendChild(this.oWidget);
	SimpleTooltip.superclass.init.call(this, this.sWidgetDomId, this.sId, this);
}
SimpleTooltip.prototype.getId = function()
{
	;
	return this.sId;
}
SimpleTooltip.prototype.setShowTimeout = function(nShowTimeout)
{
	;
	this.nShowTimeout = nShowTimeout;
}
SimpleTooltip.prototype.registerTooltip = function(sText, sDomId, sCssClass)
{
	;
	function Container(sText, sDomId, sCssClass, nIndex)
	{
		this.sText = sText;
		this.sDomId = sDomId;
		this.sCssClass = sCssClass;
		this.nIndex = nIndex;
	}
	var tooltipWidget = document.getElementById(sDomId);
	if (tooltipWidget == null)
	{
		;
	}
	else
	{
		var Container = new Container(sText, sDomId, sCssClass)
		var domlet = new Domlet(
				sDomId, 
				sDomId, 
				this, 
				new Array('mouseover', 'mouseout', 'mousemove', 'click'))	
		domlet.tooltipData = Container;
		this.addDomlet(domlet);
	}
}
SimpleTooltip.prototype.setPosOffset = function(nXOffset, nYOffset)
{
	;
	this.XOffset = nXOffset;
	this.YOffset = nYOffset;
}
SimpleTooltip.prototype.setHideTimeout = function(nHideTimeout)
{
	;
	this.nHideTimeout = nHideTimeout;
}
SimpleTooltip.prototype.eventFromChild = function(childName, eventType, event)
{
	////;
	var domlet = this.domlets[childName];
	switch (eventType)
			{
		case "mouseover" :
			;
			this.mouseoverHandler(domlet, event);
			break;
		case "mousemove" :
			;
			this.mousemoveHandler(domlet, event);
			break;
		case "mouseout" :
			;
			this.mouseoutHandler(domlet, event);
			break;
	}
}
SimpleTooltip.prototype.mouseoverHandler = function(domlet, event)
{
	;
	this.oWidget.style.top = "0px";
	this.oWidget.style.left = "0px";
	this.oWidget.innerHTML = domlet.tooltipData.sText;
	this.oWidget.className = domlet.tooltipData.sCssClass;
	this.nWidth = this.oWidget.offsetWidth;
	this.nHeight = this.oWidget.offsetHeight;
	this.moveTooltip(event.clientX, event.clientY);
	
	
	this.nShowTipTimeoutId = window.setTimeout('Tooltip_getInstance(\'' + this.sId + '\').showTooltip();', this.nShowTimeout);
	if (this.nHideTipTimeoutId)
	{
		clearTimeout(this.nHideTipTimeoutId);
		this.nHideTipTimeoutId = null;
	}
}
SimpleTooltip.prototype.showTooltip = function()
{
	;
	if (this.nHideTipTimeoutId)
	{
		clearTimeout(this.nHideTipTimeoutId);
		this.nHideTipTimeoutId = null;
	}
	if (this.nShowTipTimeoutId)
	{
		clearTimeout(this.nShowTipTimeoutId);
		this.nShowTipTimeoutId = null;
	}
	this.oWidget.style.visibility = 'visible';
	Browser.setToplevel(this.oWidget);
	if (this.nHideTimeout)
	{
		this.nHideTipTimeoutId = window.setTimeout('Tooltip_getInstance(\'' + this.sId + '\').mouseoutHandler();', this.nHideTimeout);
	}
}
SimpleTooltip.prototype.moveTooltip = function(cursorX, cursorY)
{
	;
	var firstDiv = document.getElementsByTagName("div")[0];
	
	var body = document.body;
	
	var top = 0;
	top = cursorY + this.YOffset - this.nHeight + parseInt(Browser.getBodyScrollTop()) - 5;
	if (top < 0)
	{
		top = cursorY + this.YOffset + parseInt(Browser.getBodyScrollTop()) + 3;
	}
	
	var left = 0;
	if ((cursorX + this.XOffset + this.nWidth + 5) > firstDiv.offsetWidth + body.scrollLeft) 
	{
		
		left = cursorX - this.XOffset - this.nWidth - 5;
		if (left < 0)
			left = 0;
	}
	else
	{
		
		left = cursorX + this.XOffset;
	}
	Browser.changePixelStyle(this.oWidget, 'top', top);
	Browser.changePixelStyle(this.oWidget, 'left', left);
}
SimpleTooltip.prototype.mousemoveHandler = function(domlet, event)
{
	;
	this.moveTooltip(event.clientX, event.clientY);
}
SimpleTooltip.prototype.mouseoutHandler = function(domlet, event)
{
	;
	if (this.nHideTipTimeoutId)
	{
		clearTimeout(this.nHideTipTimeoutId);
		this.nHideTipTimeoutId = null;
	}
	if (this.nShowTipTimeoutId)
	{
		clearTimeout(this.nShowTipTimeoutId);
		this.nShowTipTimeoutId = null;
	}
	this.oWidget.style.visibility = 'hidden';
}

/* common/widgets/UserInteractionBroker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
UserInteractionBroker.prototype             = new Broker();
UserInteractionBroker.prototype.constructor = UserInteractionBroker;
UserInteractionBroker.superclass            = Broker.prototype;
function UserInteractionBroker()
{
    ;
    this.userInteractionController = null;
     UserInteractionBroker.superclass.init.call(this, "UserInteractionBroker");
}
UserInteractionBroker.prototype.setUserInteractionController = function(userInteractionController)
{
    this.userInteractionController = userInteractionController;
}
UserInteractionBroker.prototype.doAllowInteraction = function()
{
    this.getMessage("allowInteraction");
}
UserInteractionBroker.prototype.addAllowInteractionHandle = function(action)
{
    this.registerAction("allowInteraction", action);
}
UserInteractionBroker.prototype.getAllowInteractionAction = function(action)
{
    return new ObjectMethodAction(this, "allowInteraction");
}
UserInteractionBroker.prototype.allowInteraction = function()
{
    if (this.userInteractionController)
        this.userInteractionController.shouldAllowInteraction();
    else
        this.doAllowInteraction();
}
UserInteractionBroker.prototype.doForbidInteraction = function()
{
    this.getMessage("forbidInteraction");
}
UserInteractionBroker.prototype.addForbidInteractionHandle = function(action)
{
    this.registerAction("forbidInteraction", action);
}
UserInteractionBroker.prototype.getForbidInteractionAction = function(action)
{
    return new ObjectMethodAction(this, "forbidInteraction");
}
UserInteractionBroker.prototype.forbidInteraction = function()
{
    if (this.userInteractionController)
        this.userInteractionController.shouldForbidInteraction();
    else
        this.doForbidInteraction();
}

/* common/widgets/ImagePushButton.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>DomletCtr</DEPENDS>
    <DEPENDS>Domlet</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
ImagePushButton.prototype = new DomletCtr();
ImagePushButton.prototype.constructor = ImagePushButton;
ImagePushButton.superclass = DomletCtr.prototype;
function ImagePushButton( domId, imgNorm, imgNormMo, imgNormDisabled, imgPressMo, enabled, name, parent )
{
  ;
  if( domId ) 
  {
    this.init(domId, imgNorm, imgNormMo, imgNormDisabled, imgPressMo, enabled, name, parent);
  }
}
ImagePushButton.prototype.init = function( domId, imgNorm, imgNormMo, imgNormDisabled, imgPressMo, enabled, name,
                                           parent )
{
  ImagePushButton.superclass.init.call(this, domId, name, parent);
  ;
  this.state = {
    pressed : false,       
    highlighted : false,   
    images : {             
      normal : imgNorm,
      normalMousover : imgNormMo,
      normalDisabled : imgNormDisabled,
      pressedMouseover : imgPressMo
    }
  };
  if( !this.state.images.normalMouseover ) this.state.images.normalMouseover = this.state.images.normal;
  if( !this.state.images.normalDisabled ) this.state.images.normalDisabled = this.state.images.normal;
  if( !this.state.images.pressedMouseover ) this.state.images.pressedMouseover = this.state.images.normal;
  this.state.enabled = ((enabled == undefined) ? true : enabled);
  ;
  this.addDomlet(new Domlet(
          this.domId, 
          this.name, 
          this.parent, 
          new Array(
          'mouseout',
          'mouseover',
          'mousedown',
          'mouseup',
          'click'
          )
          ));
  this.activeCursor = this.domlets[this.name].getStyle('cursor');
  this.inactiveCursor = this.activeCursor;
  this.displayState(this.name);
  ;
} 
ImagePushButton.prototype.eventFromChild = function( childName, eventType )
{
  ;
  switch( eventType )
          {
    case "mouseover" :
      this.state.highlighted = true;
      this.displayState(this.name);
      break;
    case "mouseout" :
      var shouldDoUp = false;
      if( this.state.pressed == true )
        shouldDoUp = true;
      this.state.highlighted = false;
      this.state.pressed = false;
      this.displayState(this.name);
      if( shouldDoUp && this.upAction )
      {
        ;
        this.upAction.invoke();
      }
      break;
    case "mousedown" :
      this.state.pressed = true;
      this.displayState(this.name);
      if( this.downAction )
      {
        ;
        this.downAction.invoke();
      }
      break;
    case "mouseup" :
      this.state.pressed = false;
      this.displayState(this.name);
      if( this.upAction )
      {
        ;
        this.upAction.invoke();
      }
      break;
    case "click" :
      this.state.pressed = false;
      this.displayState(this.name);
      if( this.state.enabled )
      {
        if( this.buttonGroupManager )
          this.buttonGroupManager.buttonPushed(this);
        if( this.pressedAction )
        {
          ;
          this.pressedAction.invoke();
        }
      }
      break;
  }
} 
ImagePushButton.prototype.setPressedAction = function( action )
{
  this.pressedAction = action;
}
ImagePushButton.prototype.setDownAction = function( action )
{
  this.downAction = action;
}
ImagePushButton.prototype.setUpAction = function( action )
{
  this.upAction = action;
}
ImagePushButton.prototype.getDisableAction = function()
{
  return new ObjectMethodAction(this, "setEnabled", false);
}
ImagePushButton.prototype.getEnableAction = function()
{
  return new ObjectMethodAction(this, "setEnabled", true);
}
ImagePushButton.prototype.setEnabled = function( enabled )
{
  this.state.enabled = enabled;
  if( !enabled )
  {
    this.state.highlighted = false;
    this.state.down = false;
  }
  this.displayState(this.name);
}
ImagePushButton.prototype.displayState = function( domletName )
{
  var imgSrc;
  if( this.state.highlighted )
  {
    imgSrc = (this.state.pressed ? this.state.images.pressedMouseover : this.state.images.normalMousover);
  }
  else
  {
    imgSrc = this.state.images.normal;
  }
  if( !this.state.enabled ) 
  {
    imgSrc = this.state.images.normalDisabled;
    this.domlets[domletName].setStyle('cursor', this.inactiveCursor);
  }
  else
  {
    this.domlets[domletName].setStyle('cursor', this.activeCursor);
  }
  if( imgSrc != null ) 
  {
    ;
    this.domlets[domletName].setImageSource(imgSrc);
  }
} 
ImagePushButton.prototype.setActiveCursor = function( activeCursor )
{
  this.activeCursor = activeCursor;
  this.displayState(this.name);
}
ImagePushButton.prototype.setInactiveCursor = function( inactiveCursor )
{
  this.inactiveCursor = inactiveCursor;
  this.displayState(this.name);
}
ImagePushButton.prototype.setButtonGroupManager = function( buttonGroupManager )
{
  this.buttonGroupManager = buttonGroupManager;
}

/* toolkit/qea/QeaItem.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function QeaItem(sId, nMax)
{
  if (sId) { this.init(sId, nMax); }
}
QeaItem.prototype.init = function(sId, nMax)
{
  ;
  this.sId = sId;
  this.nMax = nMax;
  this.bSwapSingle = (this.nMax == 1);
  this.workItem = null; 
  this.bEvaluated = false;
  this.bSolved = false;
  this.lastEvaluation = null;
  this.nCorrectEvaluations = 0;
  this.bUncovered = false;
  this.bUserFocus = false; 
  this.aSCListener = new Array(); 
  this.nSCListener = 0;
}
QeaItem.prototype.getId = function()
{
  return this.sId;
}
QeaItem.prototype.hasUserFocus = function()
{
  return this.bUserFocus;
}
QeaItem.prototype.setWorkItem = function(workItem)
{
  ;
  this.workItem = workItem;
}
QeaItem.prototype.registerStatusChangeListener = function(scListener)
{
  ;
  if (this.aSCListener[scListener.getId()] != null)
  {
    ;
  }
  else
  {
    this.aSCListener[scListener.getId()] = scListener;
    this.nSCListener++;
    ;
    scListener.setModelItem(this);
  }
  return null;
}
QeaItem.prototype.start = function()
{
  ;
}
QeaItem.prototype.canSetActive = function(bActive)
{
  ;
  var bCanSetActive = true;
  if (bActive == true)
  {
    var nCount = this.countActiveEvaluations()
    if (nCount >= this.nMax && !this.bSwapSingle)
    {
      ;
      bCanSetActive = false;
    }
  }
  return bCanSetActive;
}
QeaItem.prototype.assign = function(question, answer)
{
  ;
  return this.workItem.assign(question, answer);
}
QeaItem.prototype.assignByText = function(question, sText)
{
  ;
  return this.workItem.assignByText(question, sText);
}
QeaItem.prototype.assignOrConfirmByText = function(question, sText)
{
  ;
  return this.workItem.assignOrConfirmByText(question, sText);
}
QeaItem.prototype.assignByTextAndGridPos = function(question, sText, nX, nY)
{
  ;
  return this.workItem.assignByText(question, sText, nX, nY);
}
QeaItem.prototype.unassign = function(question, answer)
{
  ;
  return this.workItem.unassign(question, answer);
}
QeaItem.prototype.changedEvaluation = function(evaluation)
{
  ;
  this.bEvaluated = false;
  if (this.countCorrectEvaluations() == 0 && this.lastEvaluation && this.getComplementItem(evaluation) != this.getComplementOffItem())
  {
    if (evaluation.isActive())
    {
      if (this.getComplementOffItem() == this.getComplementItem(this.lastEvaluation))
      {
        ;
        var bManagerStatus = this.workItem.getNotifyManagers();
        this.workItem.setNotifyManagers(false);
        this.lastEvaluation.setActive(false, 'system_off');
        this.workItem.setNotifyManagers(bManagerStatus);
      }
    }
    else
    {
      if (this.countActiveEvaluations() == 0)
      {
        var bManagerStatus = this.workItem.getNotifyManagers();
        this.workItem.setNotifyManagers(false);
        this.activateOffEvaluation();
        this.workItem.setNotifyManagers(bManagerStatus);
      }
    }
  }
  if (evaluation.isActive())
  {
    if (this.nMax == 1 && this.bSwapSingle)
    {
      for (var id in this.workItem.aEvaluations)
      {
        var e = this.workItem.aEvaluations[id];
        if (e.isActive() && e != evaluation && this.getSameItem(e) == this)
        {
          ;
          e.setActive(false, 'system_swapsingle');
        }
      }
    }
  }
  this.lastEvaluation = evaluation;
  this.notify();
}
QeaItem.prototype.notify = function(evaluation)
{
  ;
  if (this.workItem.getNotifyView())
  {
    for (var id in this.aSCListener)
    {
      this.aSCListener[id].changed(this);
    }
  }
}
QeaItem.prototype.getSameItem = function(evaluation)
{
  ;
  return null;
}
QeaItem.prototype.getComplementItem = function(evaluation)
{
  ;
  return null;
}
QeaItem.prototype.getActiveEvaluations = function()
{
  ;
  var aEvaluations = new Array();
  for (var id in this.workItem.aEvaluations)
  {
    var e = this.workItem.aEvaluations[id];
    if (e.isActive() && this.getSameItem(e) == this && this.getComplementItem(e) != this.getComplementOffItem())
    {
      aEvaluations.push(e)
    }  
  }
  return aEvaluations;
}
QeaItem.prototype.countActiveEvaluations = function()
{
  ;
  var nCount = 0;
  for (var id in this.workItem.aEvaluations)
  {
    var e = this.workItem.aEvaluations[id];
    if (e.isActive() && this.getSameItem(e) == this && this.getComplementItem(e) != this.getComplementOffItem())
    {
      nCount++;
    }
  }
  ;
  return nCount;
}
QeaItem.prototype.countActiveCorrectEvaluations = function()
{
  ;
  var nCount = 0;
  for (var id in this.workItem.aEvaluations)
  {
    var e = this.workItem.aEvaluations[id];
    if (e.isActive() && e.isCorrect() && this.getSameItem(e) == this && this.getComplementItem(e) != this.getComplementOffItem())
    {
      nCount++;
    }
  }
  ;
  return nCount;
}
QeaItem.prototype.getActiveComplement = function()
{
  ;
  return this.getNextActiveComplement(null);
}
QeaItem.prototype.getNextActiveComplement = function(lastComplement)
{
  ;
  var nextComplement = null;
  for (var id in this.workItem.aEvaluations)
  {
    var e = this.workItem.aEvaluations[id];
    if (e.isActive() && this.getSameItem(e) == this && this.getComplementItem(e) != this.getComplementOffItem())
    {
      if (lastComplement == null)
      {
        nextComplement = this.getComplementItem(e);
        break;
      }
      else
      {
        if (this.getComplementItem(e) == lastComplement) { lastComplement = null; }
      }
    }
  }
  ;
  return nextComplement;
}
QeaItem.prototype.getSolutionComplement = function()
{
  ;
  return this.getNextSolutionComplement(null);
}
QeaItem.prototype.getNextSolutionComplement = function(lastComplement)
{
  ;
  var nextComplement = null;
  for (var id in this.workItem.aEvaluations)
  {
    var e = this.workItem.aEvaluations[id];
    if (e.isSolution() && this.getSameItem(e) == this && this.getComplementItem(e) != this.getComplementOffItem())
    {
      if (lastComplement == null)
      {
        nextComplement = this.getComplementItem(e);
        break;
      }
      else
      {
        if (this.getComplementItem(e) == lastComplement) { lastComplement = null; }
      }
    }
  }
  ;
  return nextComplement;
}
QeaItem.prototype.getMax = function()
{
  //;
  return this.nMax;
}
QeaItem.prototype.isSolved = function()
{
  //;
  return this.bSolved;
}
QeaItem.prototype.isEvaluated = function()
{
  //;
  return this.bEvaluated;
}
QeaItem.prototype.evaluated = function(bSolved)
{
  ;
  this.bSolved = bSolved;
  this.bEvaluated = true;
}
QeaItem.prototype.incCorrectEvaluations = function()
{
  ;
  this.nCorrectEvaluations++;
}
QeaItem.prototype.countCorrectEvaluations = function()
{
  //;
  return this.nCorrectEvaluations;
}
QeaItem.prototype.uncoverEvaluations = function()
{
  ;
  for (var id in this.workItem.aEvaluations)
  {
    var e = this.workItem.aEvaluations[id];
    if (this.getSameItem(e) == this)
    {
      e.setUncovered(true);
    }
  }
  this.setUncovered(true);
}
QeaItem.prototype.isUncovered = function()
{
  //;
  return this.bUncovered;
}
QeaItem.prototype.setUncovered = function(bUncovered)
{
  //;
  this.bUncovered = bUncovered;
}
QeaItem.prototype.checkUncovered = function()
{
  ;
  return this.isUncovered();
}
QeaItem.prototype.getEvaluation = function()
{
  //;
  return this.lastEvaluation;
}
QeaItem.prototype.match = function(sId)
{
  //;
  return false;
}
QeaItem.prototype.isAnswer = function()
{
  return false;
}
QeaItem.prototype.isQuestion = function()
{
  return false;
}
QeaItem.prototype.reset = function()
{
  ;
  this.bUncovered = false;
  this.bEvaluated = false;
  this.bSolved = false;
  this.bUserFocus = false;
}

/* toolkit/qea/Question.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>QeaItem</DEPENDS>
</DEPENDENCIES>
*/
Question.prototype             = new QeaItem();
Question.prototype.constructor = Question;
Question.superclass            = QeaItem.prototype;
function Question(sId, nMax)
{
  if (sId) { this.init(sId, nMax); }
}
Question.prototype.getSameItem = function(evaluation)
{
  ;
  return evaluation.getQuestion();
}
Question.prototype.getComplementItem = function(evaluation)
{
  ;
  return evaluation.getAnswer();
}
Question.prototype.getComplementOffItem = function()
{
  //;
  return this.workItem.offAnswer;
}
Question.prototype.isOffItem = function()
{
  //;
  return this == this.workItem.offQuestion;
}
Question.prototype.activateOffEvaluation = function()
{
  ;
  var e = this.workItem.getEvaluation(this, this.workItem.offAnswer);
  if (e)
  {
    e.setActive(true, 'system_off');
  }
}
Question.prototype.getActiveAnswer = function()
{
  ;
  return this.getActiveComplement();
}
Question.prototype.evaluate = function()
{
  ;
  if (!this.bEvaluated)
  {
    ;
    if ( this.workItem.bWithAlternatives )
    {
      var nCorrectAvailable = 0;
      var nCorrectActive = 0;
      var nWrongActive = 0;
      this.bSolved = undefined;
      for (var id in this.workItem.aEvaluations)
      {
        var e = this.workItem.aEvaluations[id];
        if (this.getSameItem(e) == this)
        {
          if (e.isCorrect())
          {
            nCorrectAvailable++;
            if (e.isActive())
            {
              nCorrectActive++;
            }
          }
        }
      }
      if (nCorrectActive > 0)
      {
        this.bSolved = true;
      }
      else
      {
        if (nCorrectAvailable > 0)
        {
          this.bSolved = false;
        }
      }
      if (nCorrectAvailable == 0)
      {
        this.bSolved = true;
      }
      else
      {
        if (nCorrectActive > 0 && nCorrectActive < nCorrectAvailable)
        {
          for (var id in this.workItem.aEvaluations)
          {
            var e = this.workItem.aEvaluations[id];
            if (this.getSameItem(e) == this)
            {
              if (e.isCorrect() && !e.isActive())
              {
                if (this.getComplementItem(e).countCorrectEvaluations() <= 1)
                {
                  this.bSolved = false;
                  break;
                }
              }
            }
          }
        }
        if (nCorrectActive == 0)
        {
          var bLeftUnassigned = 0;
          for (var id in this.workItem.aEvaluations)
          {
            var e = this.workItem.aEvaluations[id];
            if (this.getSameItem(e) == this)
            {
              if (e.isCorrect())
              {
                if (this.nMax == 1)
                {
                  if (this.getComplementItem(e).countActiveCorrectEvaluations() == 0)
                  {
                    bLeftUnassigned = true;
                    break;
                  }
                }
                else
                {
                  if (this.getComplementItem(e).countActiveEvaluations() == 0)
                  {
                    bLeftUnassigned = true;
                    break;
                  }
                }
              }
            }
          }
          if (!bLeftUnassigned)
          {
            this.bSolved = undefined;
          }
        }
      }
    }
    else
    {
      var countCorrect = 0;
      this.bSolved = undefined;
      for (var id in this.workItem.aEvaluations)
      {
        var e = this.workItem.aEvaluations[id];
        if (this.getSameItem(e) == this)
        {
          if (!e.isActive() && e.isCorrect())
          {
            this.bSolved = false;
            break;
          }
          if (e.isActive() && e.isCorrect())
          {
            countCorrect++;
          }
        }
      }
      if ( this.bSolved == undefined )
      {
        if (countCorrect == this.nCorrectEvaluations )
        {
          this.bSolved = true;
        }
        else
        {
          this.bSolved = false;
        }
      }
    }      
  }
  if (this.bSolved != undefined)
  {
    this.bEvaluated = true;
    this.notify();
  }
}
Question.prototype.isQuestion = function()
{
  return true;
}

/* toolkit/staticexport/ServiceRegistryAccessor.js */
 /*
 <DEPENDENCIES>
	 <DEPENDS>Logger</DEPENDS>
 </DEPENDENCIES>
 */
 function ServiceRegistryAccessor()
 {
 }
 ServiceRegistryAccess = new ServiceRegistryAccessor();
 ServiceRegistryAccessor.prototype.getServiceRegistry= function() {
	var serviceRegistry = null;
	
	if (parent.frames['services'] && parent.frames['services'].serviceregistry)
		serviceRegistry =  parent.frames['services'].serviceregistry;
	else if(opener && opener.parent && opener.parent.frames['services'] && opener.parent.frames['services'].serviceregistry)
		serviceRegistry =  opener.parent.frames['services'].serviceregistry;
	return serviceRegistry; 	
 }

/* toolkit/screen/ScreenBroker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
    <DEPENDS>IgnoreRuntimeParameterAction</DEPENDS>
    <DEPENDS>ServiceRegistryAccessor</DEPENDS>
</DEPENDENCIES>
*/
ScreenBroker.prototype             = new Broker();
ScreenBroker.prototype.constructor = ScreenBroker;
ScreenBroker.superclass            = Broker.prototype;
function ScreenBroker()
{
    ;
    this.screenController = null;
    this.bookmarkable = false;
     ScreenBroker.superclass.init.call(this, 'ScreenBroker');
     window.screenBroker = this;
     this.beforeLeavePageAction = null;
}
ScreenBroker.prototype.setScreenController = function(screenController)
{
    this.screenController = screenController;
}
ScreenBroker.prototype.getId = function()
{
    return 'ScreenBroker';
}
ScreenBroker.prototype.doReplace = function(extURL)
{
    ;
    var paramArray = new Array();
    paramArray[0] = extURL;
    this.getMessage('replace', paramArray);
}
ScreenBroker.prototype.addReplaceHandle = function(action)
{
    this.registerAction('replace', action);
}
ScreenBroker.prototype.getReplaceAction = function(extURL)
{
    return new ObjectMethodAction(this, 'replace', extURL);
}
ScreenBroker.prototype.replace = function(extURL)
{
    if (this.screenController)
        this.screenController.shouldReplace(extURL);
    else
        this.doReplace(extURL);
}
ScreenBroker.prototype.doReload = function()
{
    ;
    this.getMessage('reload');
}
ScreenBroker.prototype.addReloadHandle = function(action)
{
    this.registerAction('reload', action);
}
ScreenBroker.prototype.getReloadAction = function()
{
    return new ObjectMethodAction(this, 'reload');
}
ScreenBroker.prototype.reload = function()
{
    if (this.screenController)
        this.screenController.shouldReload();
    else
        this.doReload();
}
ScreenBroker.prototype.doPopup = function(extURL, windowName, width, height, scrollbars, resizable)
{
    ;
    var paramArray = new Array();
    paramArray[0] = extURL;
    paramArray[1] = windowName;
    paramArray[2] = width;
    paramArray[3] = height;
    paramArray[4] = scrollbars;
    paramArray[5] = resizable;
    this.getMessage('popup', paramArray);
}
ScreenBroker.prototype.addPopupHandle = function(action)
{
    this.registerAction('popup', action);
}
ScreenBroker.prototype.getPopupAction = function(id, extURL, windowName, width, height, scrollbars, resizable)
{
    return new ObjectMethodAction(this, 'popup',
                id, extURL, windowName, width, height, scrollbars, resizable);
}
ScreenBroker.prototype.popup = function(id, extURL, windowName, width, height, scrollbars, resizable)
{
    extURL = this.getURL(id, extURL);
    if (this.screenController)
        this.screenController.shouldPopup(extURL, windowName, width, height, scrollbars, resizable)
    else
        this.doPopup(extURL, windowName, width, height, scrollbars, resizable)
}
ScreenBroker.prototype.doIFrame = function(extURL, windowName)
{
    ;
    var paramArray = new Array();
    paramArray[0] = extURL;
    paramArray[1] = windowName;
    this.getMessage('iframe', paramArray);
}
ScreenBroker.prototype.addIFrameHandle = function(action)
{
    this.registerAction('iframe', action);
}
ScreenBroker.prototype.getIFrameAction = function(extURL, windowName)
{
    return new ObjectMethodAction(this, 'iframe',
                extURL, windowName);
}
ScreenBroker.prototype.iframe = function(extURL, windowName)
{
    if (this.screenController)
        this.screenController.shouldIFrame(extURL, windowName)
    else
        this.doIFrame(extURL, windowName)
}
ScreenBroker.prototype.doInvokeServletMethod = function(extURL)
{
    ;
    var paramArray = new Array();
    paramArray[0] = extURL;
    this.getMessage('invokeServletMethod', paramArray);
}
ScreenBroker.prototype.addInvokeServletMethodHandle = function(action)
{
    this.registerAction('invokeServletMethod', action);
}
ScreenBroker.prototype.getInvokeServletMethodAction = function(extURL, successAction, errorAction)
{
    return new ObjectMethodAction(this, 'invokeServletMethod', extURL, successAction, errorAction);
}
ScreenBroker.prototype.invokeServletMethod = function(extURL, callBackAction, errorAction)
{
    if (this.screenController)
        this.screenController.shouldInvokeServletMethod(extURL, callBackAction, errorAction)
    else
        this.doInvokeServletMethod(extURL)
}
ScreenBroker.prototype.doCallServer = function(extURL)
{
    ;
    var paramArray = new Array();
    paramArray[0] = extURL;
    this.getMessage('callServer', paramArray);
}
ScreenBroker.prototype.addCallServerHandle = function(action)
{
    this.registerAction('callServer', action);
}
ScreenBroker.prototype.getCallServerAction = function(extURL, successAction, errorAction)
{
    return new ObjectMethodAction(this, 'callServer', extURL, successAction, errorAction);
}
ScreenBroker.prototype.callServer = function(extURL, callBackAction, errorAction)
{
    if (this.screenController)
        this.screenController.shouldCallServer(extURL, callBackAction, errorAction)
    else
        this.doCallServer(extURL)
}
ScreenBroker.prototype.doClosePopup = function(windowName)
{
    ;
    var paramArray = new Array();
    paramArray[0] = windowName;
    this.getMessage('closePopup', paramArray);
}
ScreenBroker.prototype.addClosePopupHandle = function(action)
{
    this.registerAction('closePopup', action);
}
ScreenBroker.prototype.getClosePopupAction = function(windowName)
{
    return new ObjectMethodAction(this, 'closePopup', windowName);
}
ScreenBroker.prototype.closePopup = function(windowName)
{
    if (this.screenController)
        this.screenController.shouldClosePopup(windowName)
    else
        this.doClosePopup(windowName)
}
ScreenBroker.prototype.doReloadPopup = function(extURL, windowName)
{
    ;
    var paramArray = new Array();
    paramArray[0] = extURL;
    paramArray[1] = windowName;
    this.getMessage('reloadPopup', paramArray);
}
ScreenBroker.prototype.addReloadPopupHandle = function(action)
{
    this.registerAction('reloadPopup', action);
}
ScreenBroker.prototype.getReloadPopupAction = function(extURL, windowName)
{
    return new ObjectMethodAction(this, 'reloadPopup', extURL, windowName);
}
ScreenBroker.prototype.reloadPopup = function(extURL, windowName)
{
    if (this.screenController)
        this.screenController.shouldReloadPopup(extURL, windowName)
    else
        this.doReloadPopup(extURL, windowName)
}
ScreenBroker.prototype.doLoaded = function()
{
    ;
    this.getMessage('loaded');
}
ScreenBroker.prototype.addLoadedHandle = function(action)
{
    this.registerAction('loaded', action);
}
ScreenBroker.prototype.getLoadedAction = function()
{
    return new ObjectMethodAction(this, 'loaded');
}
ScreenBroker.prototype.loaded = function()
{
    if (this.screenController)
        this.screenController.shouldLoaded();
    else
        this.doLoaded();
}
ScreenBroker.prototype.doUnload = function()
{
    ;
    this.getMessage('unloaded');
}
ScreenBroker.prototype.addUnloadHandle = function(action)
{
    this.registerAction('unloaded', action);
}
ScreenBroker.prototype.getUnloadAction = function()
{
    return new ObjectMethodAction(this, 'unloaded');
}
ScreenBroker.prototype.unloaded = function()
{
    if (this.screenController)
        this.screenController.shouldUnload();
    else
        this.doUnload();
}
ScreenBroker.prototype.doOpenWindow = function(url, target)
{
    ;
    var paramArray = new Array();
    paramArray[0] = url;
    paramArray[1] = target;
    this.getMessage('openWindow', paramArray);
}
ScreenBroker.prototype.addOpenWindowHandle = function(action)
{
    this.registerAction('openWindow', action);
}
ScreenBroker.prototype.getOpenWindowAction = function(id, url, target)
{
    return new ObjectMethodAction(this, 'openWindow', id, url, target);
}
ScreenBroker.prototype.openWindow = function(id, url, target)
{
    url = this.getURL(id, url);
    if (this.screenController)
        this.screenController.shouldOpenWindow(url, target);
    else
        this.doOpenWindow(url, target);
}
ScreenBroker.prototype.doOpenerReplace = function(internalURL)
{
    ;
    var paramArray = new Array();
    paramArray[0] = internalURL;
    this.getMessage('openerReplace', paramArray);
}
ScreenBroker.prototype.addOpenerReplaceHandle = function(action)
{
    this.registerAction('openerReplace', action);
}
ScreenBroker.prototype.getOpenerReplaceAction = function(internalURL)
{
    return new ObjectMethodAction(this, 'openerReplace', internalURL);
}
ScreenBroker.prototype.openerReplace = function(internalURL)
{
    if (this.screenController)
        this.screenController.shouldOpenerReplace(internalURL);
    else
        this.doOpenerReplace(internalURL);
}
ScreenBroker.prototype.getAddUserMarkerOpenerAction = function(markerId)
{
    var obj = new Object();
    obj.invoke = function()
    {
        if (opener && opener.screenBroker)
            opener.screenBroker.addUserMarker(markerId);
    };
    return obj;
}
ScreenBroker.prototype.getAddUserMarkerAction = function(markerId)
{
    return new ObjectMethodAction(this, 'addUserMarker', markerId);
}
ScreenBroker.prototype.addUserMarker = function(markerId)
{
    ;
    if (this.screenController)
        this.screenController.shouldAddUserMarker(markerId);
    else
        this.doAddUserMarker(markerId);
}
ScreenBroker.prototype.doAddUserMarker = function(markerId)
{
    var paramArray = new Array();
    paramArray[0] = markerId;
    this.getMessage('addUserMarker', paramArray);
}
ScreenBroker.prototype.addAddUserMarkerHandle = function(action)
{
    this.registerAction('addUserMarker', action);
}
ScreenBroker.prototype.setBookmarkable = function(bookmarkEnable)
{
    ;
    this.bookmarkable = bookmarkEnable;
}
ScreenBroker.prototype.doNotifyPopupAware = function(screenBroker)
{
    ;
    var paramArray = new Array();
    paramArray[0] = screenBroker;
    this.getMessage('notifyPopupAware', paramArray);
}
ScreenBroker.prototype.addNotifyPopupAwareHandle = function(action)
{
    this.registerAction('notifyPopupAware', action);
}
ScreenBroker.prototype.getNotifyPopupAwareAction = function(screenBroker)
{
    return new ObjectMethodAction(this, 'notifyPopupAware', screenBroker);
}
ScreenBroker.prototype.notifyPopupAware = function(screenBroker)
{
    if (this.screenController)
        this.screenController.shouldNotifyPopupAware(screenBroker)
    else
        this.doNotifyPopupAware(screenBroker)
}
ScreenBroker.prototype.doRequestLogout = function()
{
    ;
    this.getMessage('requestLogout');
}
ScreenBroker.prototype.addRequestLogoutHandle = function(action)
{
    this.registerAction('requestLogout', action);
    window.Quit = function()
    {
        window.screenBroker.getRequestLogoutAction().invoke();
    }
}
ScreenBroker.prototype.getRequestLogoutAction = function()
{
    return new ObjectMethodAction(this, 'requestLogout');
}
ScreenBroker.prototype.requestLogout = function()
{
    this.doRequestLogout();
}
ScreenBroker.prototype.doLogout = function()
{
    ;
    this.getMessage('logout');
}
ScreenBroker.prototype.getLogoutAction = function()
{
    return new ObjectMethodAction(this, 'logout');
}
ScreenBroker.prototype.addLogoutHandle = function(action)
{
    this.registerAction('logout', action);
}
ScreenBroker.prototype.logout = function()
{
    if (this.screenController)
        this.screenController.shouldLogout();
    else
        this.doLogout();
}
ScreenBroker.prototype.doLeave = function(screenBroker)
{
    ;
    var paramArray = new Array();
    paramArray[0] = screenBroker;
    this.getMessage('leave', paramArray);
}
ScreenBroker.prototype.addLeaveHandle = function(action)
{
    this.registerAction('leave', action);
}
ScreenBroker.prototype.getLeaveAction = function(screenBroker)
{
    return new ObjectMethodAction(this, 'leave', screenBroker);
}
ScreenBroker.prototype.leave = function(screenBroker)
{
    if (this.screenController)
        this.screenController.shouldLeave(screenBroker)
    else
        this.doLeave(screenBroker)
}
ScreenBroker.prototype.getURL = function(id, defaulturl)
{
	
	if (id)
    {
		var serviceregistry = ServiceRegistryAccess.getServiceRegistry();
		if (serviceregistry)
		{
			var linkservice = serviceregistry.getService("linkservice");
			if (linkservice)
			{
				var url = linkservice.getLink(id);
				if (url)
					defaulturl = url;
			}
		}
   	}
   	return defaulturl;
}

/* toolkit/qea/AnswerValue.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function AnswerValue(sId)
{
  if (sId) { this.init(sId); }
}
AnswerValue.prototype.init = function(sId)
{
  ;
  this.sId = sId;
}
AnswerValue.prototype.match = function(value)
{
  return false;
}
AnswerValue.prototype.getText = function()
{
  return 'X';
}
AnswerValue.prototype.getGapSampleText = function()
{
  return this.getText();
}
EmptyTextAnswerValue.prototype             = new AnswerValue();
EmptyTextAnswerValue.prototype.constructor = EmptyTextAnswerValue;
EmptyTextAnswerValue.superclass            = AnswerValue.prototype;
function EmptyTextAnswerValue(sId)
{
  if (sId) { this.init(sId); }
}
EmptyTextAnswerValue.prototype.init = function(sId)
{
  ;
  EmptyTextAnswerValue.superclass.init.call(this, sId);
}
EmptyTextAnswerValue.prototype.match = function(value)
{
  return ('' == value);
}
EmptyTextAnswerValue.prototype.getText = function()
{
  return '';
}
CasesensitiveTextAnswerValue.prototype             = new AnswerValue();
CasesensitiveTextAnswerValue.prototype.constructor = CasesensitiveTextAnswerValue;
CasesensitiveTextAnswerValue.superclass            = AnswerValue.prototype;
function CasesensitiveTextAnswerValue(sId, text)
{
  if (sId) { this.init(sId, text); }
}
CasesensitiveTextAnswerValue.prototype.init = function(sId, text)
{
  ;
  CasesensitiveTextAnswerValue.superclass.init.call(this, sId);
  this.text = text;
}
CasesensitiveTextAnswerValue.prototype.match = function(value)
{
  return (this.text == value);
}
CasesensitiveTextAnswerValue.prototype.getText = function()
{
  return this.text;
}
UpperBoundNumberAnswerValue.prototype             = new AnswerValue();
UpperBoundNumberAnswerValue.prototype.constructor = UpperBoundNumberAnswerValue;
UpperBoundNumberAnswerValue.superclass            = AnswerValue.prototype;
function UpperBoundNumberAnswerValue(sId, valueFloat)
{
  if (sId) { this.init(sId, valueFloat); }
}
UpperBoundNumberAnswerValue.prototype.init = function(sId, valueFloat)
{
  ;
  UpperBoundNumberAnswerValue.superclass.init.call(this, sId);
  this.valueFloat = parseFloat(valueFloat);
}
UpperBoundNumberAnswerValue.prototype.match = function(value)
{
  value = value.replace (",", ".");
  if(isNaN(value))
    return false;
  var inputValueFloat = parseFloat(value);
  return (value <= this.valueFloat);
}
UpperBoundNumberAnswerValue.prototype.getText = function()
{
  return this.valueFloat + '';
}
LowerBoundNumberAnswerValue.prototype             = new AnswerValue();
LowerBoundNumberAnswerValue.prototype.constructor = LowerBoundNumberAnswerValue;
LowerBoundNumberAnswerValue.superclass            = AnswerValue.prototype;
function LowerBoundNumberAnswerValue(sId, valueFloat)
{
  if (sId) { this.init(sId, valueFloat); }
}
LowerBoundNumberAnswerValue.prototype.init = function(sId, valueFloat)
{
  ;
  LowerBoundNumberAnswerValue.superclass.init.call(this, sId);
  this.valueFloat = parseFloat(valueFloat);
}
LowerBoundNumberAnswerValue.prototype.match = function(value)
{
  value = value.replace (",", ".");
  if(isNaN(value))
    return false;
  var inputValueFloat = parseFloat(value);
  return (value >= this.valueFloat);
}
LowerBoundNumberAnswerValue.prototype.getText = function()
{
  return this.valueFloat + '';
}
RangeNumberAnswerValue.prototype             = new AnswerValue();
RangeNumberAnswerValue.prototype.constructor = RangeNumberAnswerValue;
RangeNumberAnswerValue.superclass            = AnswerValue.prototype;
function RangeNumberAnswerValue(sId, lowerBoundFloat, upperBoundFloat)
{
  if (sId) { this.init(sId, lowerBoundFloat, upperBoundFloat); }
}
RangeNumberAnswerValue.prototype.init = function(sId, lowerBoundFloat, upperBoundFloat)
{
  ;
  RangeNumberAnswerValue.superclass.init.call(this, sId);
  this.lowerBoundFloat = parseFloat(lowerBoundFloat);
  this.upperBoundFloat = parseFloat(upperBoundFloat);
}
RangeNumberAnswerValue.prototype.match = function(value)
{
  value = value.replace (",", ".");
  if(isNaN(value))
    return false;
  var inputValueFloat = parseFloat(value);
  return ((this.lowerBoundFloat <= value) && (value <= this.upperBoundFloat));
}
RangeNumberAnswerValue.prototype.getText = function()
{
  return this.lowerBoundFloat.toString();
}
RangeNumberAnswerValue.prototype.getGapSampleText = function()
{
  ;
  var intPlaces = Math.max(
    getIntegerPlaces(this.lowerBoundFloat), getIntegerPlaces(this.upperBoundFloat) );
  var decPlaces = Math.max(
    getDecimalPlaces(this.lowerBoundFloat), getDecimalPlaces(this.upperBoundFloat) );
  ;
  var result = '';
  for ( var i = 0; i < intPlaces; i++ )
    result += '9';
  result += '.';
  for ( var i = 0; i < decPlaces; i++ )
    result += '9';
  ;
  return result;
}
function getDecimalPlaces( value )
{
  var pos = value.toString().indexOf('.');
  if( pos == -1 )
    return 0;
  else
    return value.toString().length - ( pos + 1 );
}
function getIntegerPlaces( value )
{
  var pos = value.toString().indexOf('.');
  if( pos == -1 )
    return value.toString().length;
  else
    return pos;
}

/* toolkit/qea/Answer.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>QeaItem</DEPENDS>
    <DEPENDS>EmptyTextAnswerValue</DEPENDS>
    <DEPENDS>CasesensitiveTextAnswerValue</DEPENDS>
</DEPENDENCIES>
*/
Answer.prototype             = new QeaItem();
Answer.prototype.constructor = Answer;
Answer.superclass            = QeaItem.prototype;
function Answer(sId, nMax)
{
  if (sId) { this.init(sId, nMax); }
}
Answer.prototype.getSameItem = function(evaluation)
{
  //;
  return evaluation.getAnswer();
}
Answer.prototype.getComplementItem = function(evaluation)
{
  //;
  return evaluation.getQuestion();
}
Answer.prototype.getComplementOffItem = function()
{
  //;
  return this.workItem.offQuestion;
}
Answer.prototype.isOffItem = function()
{
  //;
  return this == this.workItem.offAnswer;
}
Answer.prototype.activateOffEvaluation = function()
{
  ;
  var e = this.workItem.getEvaluation(this.workItem.offQuestion, this);
  if (e)
  {
    e.setActive(true, 'system_off');
  }
}
Answer.prototype.getActiveQuestion = function()
{
  ;
  return this.getActiveComplement();
}
Answer.prototype.evaluate = function()
{
  ;
  if (!this.bEvaluated)
  {
    ;
    if ( this.workItem.bWithAlternatives )
    {
      var unusedCorrect = false;
      var correctEvaluationAvailable = false;
      this.bSolved = undefined;
      if (this.nMax == 1)
      {
        for (var id in this.workItem.aEvaluations)
        {
          var e = this.workItem.aEvaluations[id];
          if (this.getSameItem(e) == this)
          {
            if (!e.isActive() && e.isCorrect())
            {
              unusedCorrect = true;
            }
            if (e.isActive() && e.isCorrect())
            {
              unusedCorrect = false;
              this.bSolved = true;
              break;
            }
          }
        }
        if (!correctEvaluationAvailable)
        {
          this.bSolved = true;
        }
        if (unusedCorrect)
        {
          this.bSolved = false;
        }
      }
    }
    else
    {
      var countCorrect = 0;
      this.bSolved = undefined;
      for (var id in this.workItem.aEvaluations)
      {
        var e = this.workItem.aEvaluations[id];
        if (this.getSameItem(e) == this)
        {
          if (!e.isActive() && e.isCorrect())
          {
            this.bSolved = false;
            break;
          }
          if (e.isActive() && e.isCorrect())
          {
            countCorrect++;
          }
        }
      }
      if ( this.bSolved == undefined )
      {
        if (countCorrect == this.nCorrectEvaluations )
        {
          this.bSolved = true;
        }
        else
        {
          this.bSolved = false;
        }
      }
    }
  }
  if (this.bSolved != undefined)
  {
    this.bEvaluated = true;
    this.notify();
  }
}
Answer.prototype.isAnswer = function()
{
  return true;
}
Answer.prototype.getYPos = function()
{
  return null;
}
Answer.prototype.getXPos = function()
{
  return null;
}
TextAnswer.prototype             = new Answer();
TextAnswer.prototype.constructor = TextAnswer;
TextAnswer.superclass            = Answer.prototype;
function TextAnswer(sId, sText, nMax, bEmpty, answerValue)
{
  if (sId) { this.init(sId, sText, nMax, bEmpty, answerValue); }
}
TextAnswer.prototype.init = function(sId, sText, nMax, bEmpty, answerValue)
{
  ;
  TextAnswer.superclass.init.call(this, sId, nMax);
  this.value = answerValue;
  if(!answerValue)
  {
    if(bEmpty)
    {
      this.value = new EmptyTextAnswerValue(sId + '_value');
    }
    else
    {
      this.value = new CasesensitiveTextAnswerValue(sId + '_value', sText);
    }
  }
  this.text = this.value.getText();
}
TextAnswer.prototype.match = function(sText)
{
  ;
  return this.value.match(sText);
}
TextAnswer.prototype.getText = function()
{
  ;
  return this.value.getText();
}
TextAnswer.prototype.getGapSampleText = function()
{
  ;
  return this.value.getGapSampleText();
}
GridAnswer.prototype             = new Answer();
GridAnswer.prototype.constructor = GridAnswer;
GridAnswer.superclass            = Answer.prototype;
function GridAnswer(sId, nMax, nX, nY)
{
  if (sId) { this.init(sId, nMax, nX, nY); }
}
GridAnswer.prototype.init = function(sId, nMax, nX, nY)
{
  ;
  GridAnswer.superclass.init.call(this, sId, nMax);
  this.nCol = nX;
  this.nRow = nY;
}
GridAnswer.prototype.getCol = function()
{
  return this.nCol;
}
GridAnswer.prototype.getRow = function()
{
  return this.nRow;
}
TextGridAnswer.prototype             = new TextAnswer();
TextGridAnswer.prototype.constructor = TextGridAnswer;
TextGridAnswer.superclass            = TextAnswer.prototype;
function TextGridAnswer(sId, nMax, nX, nY, sText)
{
  if (sId) { this.init(sId, nMax, nX, nY, sText); }
}
TextGridAnswer.prototype.init = function(sId, nMax, nX, nY, sText)
{
  ;
  TextGridAnswer.superclass.init.call(this, sId, sText, nMax, false);
  this.nX = nX;
  this.nY = nY;
  this.sText = sText;
}
TextGridAnswer.prototype.getYPos = function()
{
  return this.nY;
}
TextGridAnswer.prototype.getXPos = function()
{
  return this.nX;
}
TextGridAnswer.prototype.match = function(sText, nX, nY)
{
  ;
  if(sText == null || sText == undefined)
    return false;
  return (this.sText.toLowerCase() == sText.toLowerCase() && this.nX == nX && this.nY == nY)
}
TextGridAnswer.prototype.changedEvaluation = function(evaluation)
{
  if (evaluation.isActive())
  {
    for (var id in this.workItem.aEvaluations)
    {
      var e = this.workItem.aEvaluations[id];
      if (e.isActive() && e != evaluation)
      {
        var a = e.getAnswer();
        if((a != this) && (a.nX == this.nX) && (a.nY == this.nY))
        {
          ;
          e.setActive(false, 'system_swapsingle');
        }
      }
    }
  }
  TextGridAnswer.superclass.changedEvaluation.call(this, evaluation);
}
HighlighterAnswer.prototype             = new Answer();
HighlighterAnswer.prototype.constructor = HighlighterAnswer;
HighlighterAnswer.superclass            = Answer.prototype;
function HighlighterAnswer(sId, nMax)
{
  if (sId) { this.init(sId, nMax); }
}
HighlighterAnswer.prototype.init = function(sId, nMax)
{
  ;
  this.nStartPos = -1;
  this.nLength = -1;
  HighlighterAnswer.superclass.init.call(this, sId, nMax);
}
HighlighterAnswer.prototype.match = function(nStartPos, nLength)
{
  ;
  return this.nStartPos == nStartPos && this.nLength == nLength && nStartPos != -1 && nLength != -1;
}
HighlighterAnswer.prototype.getStartPos = function()
{
  ;
  return this.nStartPos;
}
HighlighterAnswer.prototype.getLength = function()
{
  ;
  return this.nLength;
}
HighlighterAnswer.prototype.setStartPos = function(nStartPos)
{
  ;
  this.nStartPos = nStartPos;
}
HighlighterAnswer.prototype.setLength = function(nLength)
{
  ;
  this.nLength = nLength;
}
HighlighterAnswer.prototype.isAnswer = function()
{
  return 'Highlighter Answer';
}

/* toolkit/qea/Evaluation.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function Evaluation( sId, question, answer, bCorrect, bSolution )
{
  if( sId )
  {
    this.init(sId, question, answer, bCorrect, bSolution);
  }
}
Evaluation.prototype.init = function( sId, question, answer, bCorrect, bSolution )
{
  ;
  this.sId = sId;
  this.workItem = null;
  this.bActive = false;
  this.bCorrect = bCorrect;
  this.bSolution = bCorrect;
  if( bSolution == false )
  {
    this.bSolution = false;
  } 
  this.question = question;
  this.answer = answer;
  this.sOrigin = '';
  this.bUserActivated = false;
  this.bUserDeactivated = false;
  this.bSystemConstructive = false;
  this.bSystemCorrective = false;
  this.bSystemPartialConstructive = false;
  this.bSystemPartialCorrective = false;
  this.bChangeForDisabledOnReset = true;
  this.bUncovered = false;
  this.bEvaluated = false;
  this.aSCListener = new Array();
  this.nSCListener = 0;
  this.bIsInitialActive = false;
}
Evaluation.prototype.getId = function()
{
  return this.sId;
}
Evaluation.prototype.toString = function()
{
  var s = '' +
          ' q=' + this.question.getId() +
          ' a=' + this.answer.getId() +
          ' active=' + (this.bActive   ? '1'  : '0') +
          ' correct=' + (this.bCorrect  ? '1' : '0') +
          ' solution=' + (this.bSolution ? '1': '0') +
          ' id=' + this.sId +
          ' ';
  return s;
}
Evaluation.prototype.setSilentlyActive = function( bActive, sOrigin )
{
  var notifyFlagBackup = this.workItem.getNotifyManagers();
  this.workItem.setNotifyManagers(false);
  this.setActive(bActive, sOrigin);
  this.workItem.setNotifyManagers(notifyFlagBackup);
}
Evaluation.prototype.setActive = function( bActive, sOrigin )
{
  ;
  if( this.canSetActive(bActive) )
  {
    this.workItem.willChange(this);
    this.sOrigin = (sOrigin == undefined) ? 'user' : sOrigin;
    if( bActive )
    {
      switch( this.sOrigin )
              {
        case 'user':
          this.bUserActivated = true;
          break;
        case 'system_brand':
          this.bSystemCorrective = true;
          break;
        case 'system_off':
          break;
        case 'system_setpartial':
          break;
        default:
          this.bSystemConstructive = true;
      }
    }
    else
    {
      switch( this.sOrigin )
              {
        case 'user':
          this.bUserDeactivated = true;
          break;
        case 'system_off':
        case 'system_swapsingle':
          break;
        default:
          this.bSystemCorrective = true;
      }
    }
    this.bActive = bActive;
    this.question.changedEvaluation(this);
    this.answer.changedEvaluation(this);
    this.notify();
  }
}
Evaluation.prototype.canSetActive = function( bActive )
{
  var bcanSetActive = true;
  if( this.isActive() != bActive ) 
  {
    bcanSetActive = this.answer.canSetActive(bActive) && this.question.canSetActive(bActive);
  }
  return bcanSetActive;
}
Evaluation.prototype.hasUserContribution = function()
{
  return this.bUserActivated || this.bUserDeactivated;
}
Evaluation.prototype.hasSystemContribution = function()
{
  return this.bSystemConstructive || this.bSystemCorrective;
}
Evaluation.prototype.hasSystemConstuction = function()
{
  return this.bSystemConstructive;
}
Evaluation.prototype.hasSystemCorrection = function()
{
  return this.bSystemCorrective;
}
Evaluation.prototype.setSystemConstuction = function( bSysCon )
{
  this.bSystemConstructive = bSysCon;
}
Evaluation.prototype.setChangeForDisabledOnReset = function( bChangeForDisabledOnReset )
{
  this.bChangeForDisabledOnReset = bChangeForDisabledOnReset;
}
Evaluation.prototype.setSystemCorrection = function( bSysCor )
{
  this.bSystemCorrective = bSysCor;
}
Evaluation.prototype.hasSystemPartialConstuction = function()
{
  return this.bSystemPartialConstructive;
}
Evaluation.prototype.hasSystemPartialCorrection = function()
{
  return this.bSystemPartialCorrective;
}
Evaluation.prototype.setSystemPartialConstuction = function( bSysCon )
{
  this.bSystemPartialConstructive = bSysCon;
}
Evaluation.prototype.setSystemPartialCorrection = function( bSysCor )
{
  this.bSystemPartialCorrective = bSysCor;
}
Evaluation.prototype.isActive = function()
{
  return this.bActive;
}
Evaluation.prototype.isCorrect = function()
{
  return this.bCorrect;
}
Evaluation.prototype.isSolution = function()
{
  return this.bSolution;
}
Evaluation.prototype.isSolved = function()
{
  //;
  var bSolved = undefined;
  if( this.bActive )
  {
    bSolved = this.bCorrect;
  }
  else
  {
    if( this.workItem.bWithAlternatives )
    {
      if( this.bCorrect && this.question.nMax > 1 )
      {
        bSolved = false;
      }
    }
    else
    {
      bSolved = this.question.isSolved() && this.answer.isSolved();
    }
  }
  return bSolved;
}
Evaluation.prototype.isEvaluated = function()
{
  return this.bEvaluated;
}
Evaluation.prototype.getOrigin = function()
{
  return this.sOrigin;
}
Evaluation.prototype.getQuestion = function()
{
  return this.question;
}
Evaluation.prototype.getAnswer = function()
{
  return this.answer;
}
Evaluation.prototype.registerStatusChangeListener = function( scListener )
{
  ;
  if( this.aSCListener[scListener.getId()] != null )
  {
    ;
  }
  else
  {
    this.aSCListener[scListener.getId()] = scListener;
    this.nSCListener++;
    ;
    scListener.setModelItem(this);
  }
  return null;
}
Evaluation.prototype.setWorkItem = function( workItem )
{
  ;
  this.workItem = workItem;
}
Evaluation.prototype.start = function()
{
  ;
}
Evaluation.prototype.evaluate = function()
{
  ;
  if( this.workItem.bWithAlternatives )
  {
    if( this.bActive )
    {
      if( !this.bCorrect )
      {
        this.answer.evaluated(false);
        this.question.evaluated(false);
      }
      else
      {
        if( this.answer.nMax == 1 )
        {
          this.answer.evaluated(true);
        }
        if( this.question.nMax == 1 )
        {
          this.question.evaluated(true);
        }
      }
    }
  }
  else
  {
    this.answer.evaluate();
    this.question.evaluate();
  }
  this.bEvaluated = true;
  this.notify();
}
Evaluation.prototype.notify = function()
{
  ;
  if( this.workItem.getNotifyView() )
  {
    for( var id in this.aSCListener )
    {
      this.aSCListener[id].changed(this);
    }
    this.workItem.changed(this);
  }
}
Evaluation.prototype.isUncovered = function()
{
  //;
  return this.bUncovered;
}
Evaluation.prototype.setUncovered = function( bUncovered )
{
  //;
  this.bUncovered = bUncovered;
}
Evaluation.prototype.checkUncovered = function()
{
  ;
  if( this.hasSystemConstuction() )
  {
    this.setUncovered(true);
    if( this.answer.getMax() == 1 )
    {
      this.answer.uncoverEvaluations();
    }
    if( this.question.getMax() == 1 )
    {
      this.question.uncoverEvaluations();
    }
  }
  if( this.hasSystemCorrection() )
  {
    this.setUncovered(true);
  }
  return this.isUncovered();
}
Evaluation.prototype.hasOffItem = function()
{
  ;
  return this.answer.isOffItem() || this.question.isOffItem();
}
Evaluation.prototype.hasUserFocus = function()
{
  ;
  return this.answer.hasUserFocus() || this.question.hasUserFocus();
  ;
}
Evaluation.prototype.getEvaluation = function()
{
  //;
  return this;
}
Evaluation.prototype.countActiveEvaluations = function()
{
  //;
  return this.isActive() ? 1 : 0;
}
Evaluation.prototype.reset = function()
{
  ;
  this.bUserActivated = false;
  this.bUserDeactivated = false;
  this.bSystemConstructive = false;
  this.bSystemCorrective = false;
  this.bSystemPartialConstructive = false;
  this.bSystemPartialCorrective = false;
  this.bUncovered = false;
  this.bEvaluated = false;
  if( this.bChangeForDisabledOnReset )
  {
    this.setActive(false, 'user');
  }
  else
  {
    if( this.isActive() )
    {
      this.setActive(false, 'user');
    }
  }
}

/* toolkit/qea/QeaWorkItem.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Answer</DEPENDS>
    <DEPENDS>Question</DEPENDS>
    <DEPENDS>Evaluation</DEPENDS>
    <DEPENDS>TextAnswer</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>ExerciseHandlerStatus</DEPENDS>
</DEPENDENCIES>
*/
function QeaWorkItem( sId )
{
  if( sId )
  {
    this.init(sId);
  }
}
QeaWorkItem.prototype.init = function( sId )
{
  ;
  this.sId = sId;
  this.bAlreadyStarted = false;
  this.bIsShowSolutionActive = false;
  this.bHasShownSolution = false;
  this.aQuestions = new Array();
  this.nQuestions = 0;
  this.aAnswers = new Array();
  this.nAnswers = 0;
  this.aEvaluations = new Array();
  this.nEvaluations = 0;
  this.aTipStrategies = new Array();
  this.nTipStrategies = 0;
  this.nCorrect = 0;
  this.nWrong = 0;
  this.tipManager = null;
  this.excManager = null;
  this.visibilityBroker = null;
  this.bActive = true;
  this.bCountDynamicAnswersWrong = false;
  this.bEvaluated = false;
  this.bWorkStarted = false;
  this.bHelpApplied = false;
  this.oUserFocusQeaItem = null;
  this.bWorkStopped = false;
  this.bActivated = false;
  this.bWithAlternatives = true;
  this.lastEvaluation = null;
  this.lastUserActivatedEvaluation = null;
  this.oWorkItemBroker = null;
  this.bNotifyView = true;
  this.bNotifyManagers = true;
  this.qeaUndoStack = new QeaUndoStack(this.sId);
  this.offAnswer = new Answer('OffAnswer', 1000);
  this.offQuestion = new Question('OffQuestion', 1000);
}
QeaWorkItem.prototype.getId = function()
{
  return this.sId;
}
QeaWorkItem.prototype.getNumCorrect = function()
{
  return this.nCorrect;
}
QeaWorkItem.prototype.getNumWrong = function()
{
  return this.nWrong;
}
QeaWorkItem.prototype.setWithAlternatives = function( bWithAlternatives )
{
  ;
  this.bWithAlternatives = bWithAlternatives;
}
QeaWorkItem.prototype.setAnswerBasedEvaluation = function( bAnswerBasedEvaluation )
{
  ;
  this.bAnswerBasedEvaluation = bAnswerBasedEvaluation;
}
QeaWorkItem.prototype.setWorkItemBroker = function( oWorkItemBroker )
{
  ;
  this.oWorkItemBroker = oWorkItemBroker;
  this.oWorkItemBroker.addResetHandle(this.getResetAction());
  this.oWorkItemBroker.addEvaluateHandle(this.getEvaluateAction());
  this.oWorkItemBroker.addShowSolutionHandle(this.getShowSolutionAction());
  this.oWorkItemBroker.addHideSolutionHandle(this.getHideSolutionAction());
  this.oWorkItemBroker.addNoticedHandle(this.getNoticedAction());
}
QeaWorkItem.prototype.registerQuestion = function( question )
{
  ;
  if( this.aQuestions[question.getId()] != null )
  {
    ;
  }
  else
  {
    this.aQuestions[question.getId()] = question;
    this.nQuestions++;
    ;
    question.setWorkItem(this);
  }
}
QeaWorkItem.prototype.registerAnswer = function( answer )
{
  ;
  if( this.aAnswers[answer.getId()] != null )
  {
    ;
  }
  else
  {
    this.aAnswers[answer.getId()] = answer;
    this.nAnswers++;
    ;
    answer.setWorkItem(this);
  }
}
QeaWorkItem.prototype.registerEvaluation = function( evaluation )
{
  ;
  if( this.aEvaluations[evaluation.getId()] != null )
  {
    ;
  }
  else
  {
    this.aEvaluations[evaluation.getId()] = evaluation;
    this.nEvaluations++;
    ;
    evaluation.setWorkItem(this);
  }
  if( evaluation.isSolution() && !(evaluation.getAnswer() == this.offAnswer || evaluation.getQuestion()
          == this.offQuestion ) )
  {
    this.nCorrect++;
  }
  if( evaluation.isCorrect() && !(evaluation.getAnswer() == this.offAnswer || evaluation.getQuestion()
          == this.offQuestion ) )
  {
    evaluation.getAnswer().incCorrectEvaluations();
    evaluation.getQuestion().incCorrectEvaluations();
  }
}
QeaWorkItem.prototype.registerTipStrategy = function( tipStrategy )
{
  ;
  if( this.aTipStrategies[tipStrategy.getId()] != null )
  {
    ;
  }
  else
  {
    this.aTipStrategies[tipStrategy.getId()] = tipStrategy;
    this.nTipStrategies++;
    ;
    tipStrategy.setWorkItem(this);
  }
}
QeaWorkItem.prototype.setTipManager = function( tipManager )
{
  ;
  this.tipManager = tipManager;
}
QeaWorkItem.prototype.setExerciseManager = function( excManager )
{
  ;
  this.excManager = excManager;
}
QeaWorkItem.prototype.setVisibilityBroker = function( broker )
{
  ;
  this.visibilityBroker = broker;
}
QeaWorkItem.prototype.setActive = function( bActive )
{
  ;
  this.bActive = bActive;
  if( this.tipManager ) this.tipManager.changed(this);
}
QeaWorkItem.prototype.getActivateAction = function()
{
  return new ObjectMethodAction(this, 'setActive', true);
}
QeaWorkItem.prototype.getDeactivateAction = function()
{
  return new ObjectMethodAction(this, 'setActive', false);
}
QeaWorkItem.prototype.start = function()
{
  ;
  if( !this.bAlreadyStarted )
  {
    this.bAlreadyStarted = true;
    if( this.bAnswerBasedEvaluation )
      this.nWrong = this.nAnswers;
    else
      this.nWrong = this.calculateWrong();
    this.offAnswer.incCorrectEvaluations();
    this.registerAnswer(this.offAnswer);
    this.offAnswer.start();
    this.offQuestion.incCorrectEvaluations();
    this.registerQuestion(this.offQuestion);
    this.offQuestion.start();
    this.qeaUndoStack.lock();
    this.setNotifyView(false);
    for( var id in this.aQuestions )
    {
      if( this.aQuestions[id].countCorrectEvaluations() == 0 )
      {
        ;
        evaluation = new Evaluation(this.sId + '_' + this.aQuestions[id].getId() + '_'
                + this.offAnswer.getId(), this.aQuestions[id], this.offAnswer, true, true);
        this.registerEvaluation(evaluation);
        evaluation.setActive(true, 'system_off');
      }
    }
    for( var id in this.aAnswers )
    {
      if( this.aAnswers[id].countCorrectEvaluations() == 0 )
      {
        ;
        evaluation = new Evaluation(this.sId + '_' + this.offQuestion.getId() + '_'
                + this.aAnswers[id].getId(), this.offQuestion, this.aAnswers[id], true, true);
        this.registerEvaluation(evaluation);
        evaluation.setActive(true, 'system_off');
      }
    }
    this.setNotifyView(true);
    this.qeaUndoStack.unlock();
  }
}
QeaWorkItem.prototype.makeFakedUserActivity = function()
{
  this.workStarted();
  if( this.bNotifyManagers )
  {
    var excStatus = new ExerciseHandlerStatus(this.nCorrect, 0, this.nCorrect, 0, 0, 0, this);
    if( this.excManager ) this.excManager.solved(this, excStatus);
    if( this.tipManager ) this.tipManager.changed(this);
    if( this.oWorkItemBroker )
    {
      this.oWorkItemBroker.solved(excStatus);
      if( !this.bHelpApplied && (excStatus.nCorrectSystem || excStatus.nWrongSystem) )
      {
        this.bHelpApplied = true;
        this.oWorkItemBroker.doHelpApplied();
      }
    }
  }
}
QeaWorkItem.prototype.hasShowTip = function( bShow, bStepByStep, bSolutable, bCycleTips )
{
  if( bShow )
  {
    ;
  }
  var bHasTip = false;
  for( var id in this.aTipStrategies )
  {
    var tipStrategy = this.aTipStrategies[id];
    if( tipStrategy )
    {
      ;
      bHasTip = tipStrategy.hasShowTip(bShow, bStepByStep, bSolutable, bCycleTips);
      if( bHasTip )
      {
        ;
        break;
      }
    }
  }
  if( !bHasTip )
  {
    ;
  }
  return bHasTip;
}
QeaWorkItem.prototype.hasTip = function( bStepByStep, bSolutable, bCycleTips )
{
  ;
  if( this.bActive )
  {
    return this.hasShowTip(false, bStepByStep, bSolutable, bCycleTips);
  }
  else
  {
    return false;
  }
}
QeaWorkItem.prototype.showTip = function( bStepByStep, bSolutable, bCycleTips )
{
  ;
  var bHasTip = false;
  if( this.bActive )
  {
    var bLocalCycleTips = bCycleTips;
    bHasTip = true;
    if( !bStepByStep )
    {
      bLocalCycleTips = false;
    }
    while( bHasTip )
    {
      bHasTip = this.hasShowTip(true, bStepByStep, bSolutable, bLocalCycleTips);
      this.bUserAction = false;
      if( bStepByStep )
      {
        break;
      }
    }
  }
  return bHasTip;
}
QeaWorkItem.prototype.assign = function( question, answer )
{
  ;
  var bAssigned = false;
  var evaluation = this.getOrCreateEvaluation(question, answer);
  if( evaluation != null )
  {
    if( !evaluation.isActive() )
    {
      evaluation.setActive(true);
    }
    bAssigned = evaluation.isActive();
  }
  else
  {
    ;
  }
  return bAssigned;
}
QeaWorkItem.prototype.assignOrConfirmByText = function( question, sText )
{
  this.assignByText(question, sText, true);
}
QeaWorkItem.prototype.assignByText = function( question, sText, bConfirmActive )
{
  ;
  var bAssigned = false;
  var answer = this.getOrCreateAnswerByText(sText);
  if( answer != null )
  {
    var evaluation = this.getOrCreateEvaluation(question, answer);
    if( evaluation != null )
    {
      if( !evaluation.isActive() || bConfirmActive )
      {
        evaluation.setActive(true);
      }
      bAssigned = evaluation.isActive();
    }
    else
    {
      ;
    }
  }
  else
  {
    ;
  }
  return bAssigned;
}
QeaWorkItem.prototype.unassign = function( question, answer )
{
  ;
  var bUnassigned = false;
  var evaluation = this.getEvaluation(question, answer);
  if( evaluation != null )
  {
    if( evaluation.isActive() )
    {
      evaluation.setActive(false);
    }
    bUnassigned = !evaluation.isActive();
  }
  else
  {
    ;
    bUnassigned = true;
  }
  return bUnassigned;
}
QeaWorkItem.prototype.getOrCreateEvaluation = function( question, answer )
{
  ;
  var evaluation = this.getEvaluation(question, answer);
  if( evaluation == null )
  {
    ;
    evaluation = new Evaluation(this.sId + '_' + question.getId() + '_'
            + answer.getId(), question, answer, false, false);
    if( evaluation != null )
    {
      this.registerEvaluation(evaluation);
    }
  }
  return evaluation;
}
QeaWorkItem.prototype.getEvaluation = function( question, answer )
{
  ;
  var evaluation = null;
  for( var id in this.aEvaluations )
  {
    if( this.aEvaluations[id].getQuestion() == question && this.aEvaluations[id].getAnswer() == answer )
    {
      evaluation = this.aEvaluations[id];
      break;
    }
  }
  return evaluation;
}
QeaWorkItem.prototype.getEvaluations = function( question, answer )
{
  ;
  var evaluations = new Array();
  for( var id in this.aEvaluations )
  {
    if( this.aEvaluations[id].getQuestion() == question && this.aEvaluations[id].getAnswer() == answer )
    {
      evaluations.push(this.aEvaluations[id]);
    }
  }
  return evaluations;
}
QeaWorkItem.prototype.getOrCreateAnswerByText = function( sText )
{
  ;
  var answer = this.getAnswerByText(sText);
  if( answer == null )
  {
    ;
    answer = new TextAnswer(this.sId + '_' + sText, sText, 1000);
    if( answer != null )
    {
      this.registerAnswer(answer);
    }
  }
  return answer;
}
QeaWorkItem.prototype.getAnswerByText = function( sText )
{
  ;
  var answer = null;
  for( var id in this.aAnswers )
  {
    if( this.aAnswers[id].match(sText) )
    {
      answer = this.aAnswers[id];
      break;
    }
  }
  return answer;
}
QeaWorkItem.prototype.setUserFocus = function( oUserFocusQeaItem )
{
  ;
  if( this.oUserFocusQeaItem )
  {
    this.oUserFocusQeaItem.bUserFocus = false;
  }
  this.oUserFocusQeaItem = oUserFocusQeaItem;
  if( this.oUserFocusQeaItem )
  {
    this.oUserFocusQeaItem.bUserFocus = true;
  }
}
QeaWorkItem.prototype.isCurrentWorkItem = function()
{
  if( this.excManager )
  {
    var lastWorkItem = this.excManager.currentExecHandler;
    return (lastWorkItem == this);
  }
  else
  {
    return false;
  }
}
QeaWorkItem.prototype.getOrCreateAnswerByTextAndGridPos = function( sText, nX, nY )
{
  ;
  var answer = this.getAnswerByTextAndGridPos(sText, nX, nY);
  if( answer == null )
  {
    ;
    answer = new TextGridAnswer('TextGridAnswer_' + this.sId + sText + '_X_' + nX + '_Y_' + nY, 1000, nX, nY, sText);
    if( answer != null )
    {
      this.registerAnswer(answer);
    }
  }
  return answer;
}
QeaWorkItem.prototype.getAnswerByTextAndGridPos = function( sText, nX, nY )
{
  ;
  var answer = null;
  for( var id in this.aAnswers )
  {
    if( this.aAnswers[id].match(sText, nX, nY) )
    {
      answer = this.aAnswers[id];
      break;
    }
  }
  return answer;
}
QeaWorkItem.prototype.getOrCreateAnswerByPosAndLength = function( nStartPos, nLength )
{
  ;
  var answer = this.getAnswerByPosAndLength(nStartPos, nLength);
  if( answer == null )
  {
    ;
    answer = new HighlighterAnswer('HighlighterAnswer_' + this.sId + '_Start_' + nStartPos + '_length_'
            + nLength, nStartPos, nLength, 1000);
    if( answer != null )
    {
      this.registerAnswer(answer);
    }
  }
  return answer;
}
QeaWorkItem.prototype.getAnswerByPosAndLength = function( nStartPos, nLength )
{
  ;
  var answer = null;
  for( var id in this.aAnswers )
  {
    if( this.aAnswers[id].match(nStartPos, nLength) )
    {
      answer = this.aAnswers[id];
      break;
    }
  }
  return answer;
}
QeaWorkItem.prototype.willChange = function( evaluation )
{
  ;
  this.qeaUndoStack.push(new QeaUndoItem(evaluation.getId(), evaluation));
}
QeaWorkItem.prototype.setUndoBreak = function( sTag )
{
  ;
  this.qeaUndoStack.push(new QeaUndoItem(sTag, null));
}
QeaWorkItem.prototype.undo = function( bRestoreSystemContribution, sTag )
{
  ;
  var sLastId = null;
  this.qeaUndoStack.lock();
  var bDone = false;
  while( !bDone )
  {
    var undoItem = this.qeaUndoStack.pop();
    if( undoItem != null )
    {
      sLastId = undoItem.getId();
      if( undoItem.evaluation )
      {
        undoItem.evaluation.setActive(undoItem.bActive, undoItem.sOrigin);
        if( bRestoreSystemContribution )
        {
          undoItem.evaluation.setSystemConstuction(undoItem.bSystemConstructive);
          undoItem.evaluation.setSystemCorrection(undoItem.bSystemCorrective);
          undoItem.evaluation.setSystemPartialConstuction(undoItem.bSystemPartialConstructive);
          undoItem.evaluation.setSystemPartialCorrection(undoItem.bSystemPartialCorrective);
        }
      }
    }
    else
    {
      ;
      bDone = true;
    }
    if( sTag )
    {
      if( sTag == sLastId )
      {
        bDone = true;
      }
    }
    else
    {
      bDone = true;
    }
  }
  this.qeaUndoStack.unlock();
  return sLastId;
}
QeaWorkItem.prototype.changed = function( evaluation )
{
  ;
  if( this.bNotifyManagers )
  {
    this.lastEvaluation = evaluation;
    if( evaluation )
    {
      switch( evaluation.getOrigin() )
              {
        case 'user':
          this.lastUserActivatedEvaluation = evaluation;
          break;
      }
      if( evaluation.isActive() )
      {
        switch( evaluation.getOrigin() )
                {
          case 'system_set':
            this.lastUserActivatedEvaluation = evaluation;
            break;
        }
      }
    }
    if( !this.bWorkStarted && !this.bEvaluated )
    {
      if( this.bHasShownSolution )
      {
        this.bHasShownSolution = false;
      }
      else
      {
        if( evaluation )
        {
          switch( evaluation.getOrigin() )
                  {
            case 'system_solution':
              this.noticed();
              break;
            case 'user':
            case 'system_set':
            case 'system_approve':
            case 'system_brand':
              this.workStarted();
              break;
          }
        }
      }
    }
    this.propagateStatus(evaluation);
    if( evaluation && evaluation.getOrigin() == 'user' )
    {
      this.qeaUndoStack.flush();
    }
  }
}
QeaWorkItem.prototype.workStarted = function()
{
  if( !this.bWorkStarted )
  {
    this.noticed();
    this.bWorkStarted = true;
    if( this.oWorkItemBroker )
    {
      this.oWorkItemBroker.doWorkStarted();
    }
    this.excManager.workItemStarted(this);
  }
}
QeaWorkItem.prototype.propagateStatus = function( evaluation )
{
  ;
  if( this.bNotifyManagers )
  {
    var excStatus = this.getExerciseStatus();
    ;
    if( this.excManager ) this.excManager.solved(this, excStatus);
    if( this.tipManager ) this.tipManager.changed(this);
    if( this.oWorkItemBroker )
    {
      this.oWorkItemBroker.solved(excStatus);
      if( !this.bHelpApplied && (excStatus.nCorrectSystem || excStatus.nWrongSystem) )
      {
        this.bHelpApplied = true;
        this.oWorkItemBroker.doHelpApplied();
      }
    }
    if( evaluation && evaluation.getOrigin() == 'system_solution' )
    {
      if( this.visibilityBroker ) this.visibilityBroker.activity();
    }
  }
}
QeaWorkItem.prototype.calculateWrong = function()
{
  var questionAssignments = 0;
  var answerAssignments = 0;
  for( var id in this.aQuestions )
  {
    questionAssignments += this.aQuestions[id].getMax();
  }
  for( var id in this.aAnswers )
  {
    answerAssignments += this.aAnswers[id].getMax();
  }
  var assignments = (questionAssignments < answerAssignments) ? questionAssignments : answerAssignments;
  return (assignments < (this.nAnswers * this.nQuestions)) ? assignments : (this.nAnswers * this.nQuestions);
}
QeaWorkItem.prototype.getExerciseStatus = function()
{
  ;
  if( this.bCountDynamicAnswersWrong )
  {
    this.nWrong = this.calculateWrong();
  }
  var nCorrectStatus = 0;
  var nWrongStatus = 0;
  var nCorrectSystem = 0;
  var nWrongSystem = 0;
  for( var id in this.aEvaluations )
  {
    var e = this.aEvaluations[id];
    if( e.isActive() )
    {
      if( e.isCorrect() )
      {
        if( !e.hasOffItem() )
        {
          nCorrectStatus++;
        }
      }
      else
      {
        nWrongStatus++;
      }
    }
    else
    {
      if( !this.bWithAlternatives && e.isCorrect() )
      {
        if( !e.hasOffItem() )
        {
          nWrongStatus++;
        }
      }
    }
    if( e.hasSystemConstuction() || e.hasSystemPartialConstuction() )
    {
      nCorrectSystem++;
    }
    if( e.hasSystemCorrection() || e.hasSystemPartialCorrection() )
    {
      nWrongSystem++;
    }
  }
  nWrongStatus = nWrongStatus > this.nWrong ? this.nWrong : nWrongStatus;
  nWrongSystem = nWrongSystem > this.nWrong ? this.nWrong : nWrongSystem;
  var sOrigin = undefined;
  if( this.lastEvaluation && this.lastEvaluation.sOrigin )
  {
    sOrigin = this.lastEvaluation.sOrigin;
  }
  return new ExerciseHandlerStatus(this.nCorrect, this.nWrong, nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem, this, sOrigin);
}
QeaWorkItem.prototype.evaluate = function()
{
  ;
  this.workStarted();
  this.propagateStatus();
  this.bEvaluated = true;
  this.bNotifyManagers = false;
  for( var id in this.aEvaluations )
  {
    this.aEvaluations[id].evaluate();
  }
  for( var id in this.aQuestions )
  {
    this.aQuestions[id].evaluate();
  }
  for( var id in this.aAnswers )
  {
    this.aAnswers[id].evaluate();
  }
  var excStatus = this.getExerciseStatus();
  if( excStatus.nCorrect == excStatus.nCorrectStatus && excStatus.nWrongStatus == 0 )
  {
  }
  else
  {
    if( this.visibilityBroker )
      this.visibilityBroker.evaluatesWrong();
  }
  this.bNotifyManagers = true;
  this.workStopped();
}
QeaWorkItem.prototype.reset = function()
{
  ;
  if( this.oWorkItemBroker && this.bWorkStarted && !this.bWorkStopped ) 
  {
    this.oWorkItemBroker.doWorkAborted();
  }
  this.excManager.workItemAborted(this);
  this.bNotifyManagers = false;
  this.bNotifyView = true;
  this.bEvaluated = false;
  this.bActivated = false;
  this.bIsShowSolutionActive = false;
  this.bHasShownSolution = false;
  for( var id in this.aQuestions )
  {
    this.aQuestions[id].reset();
  }
  for( var id in this.aAnswers )
  {
    this.aAnswers[id].reset();
  }
  for( var id in this.aEvaluations )
  {
    this.aEvaluations[id].reset();
  }
  for( var id in this.aEvaluations )
  {
    if( this.aEvaluations[id].bIsInitialActive )
    {
      this.aEvaluations[id].setActive(true, 'system_off');
    }
  }
  for( var id in this.aQuestions )
  {
    if( this.aQuestions[id].countCorrectEvaluations() == 0 )
    {
      evaluation = this.getEvaluation(this.aQuestions[id], this.offAnswer);
      if( evaluation )
      {
        evaluation.setActive(true, 'system_off');
      }
    }
  }
  for( var id in this.aAnswers )
  {
    if( this.aAnswers[id].countCorrectEvaluations() == 0 )
    {
      evaluation = this.getEvaluation(this.offQuestion, this.aAnswers[id]);
      if( evaluation )
      {
        evaluation.setActive(true, 'system_off');
      }
    }
  }
  this.bNotifyManagers = true;
  this.qeaUndoStack.flush();
  this.bWorkStarted = false;
  this.bHelpApplied = false;
  this.bWorkStopped = false;
  this.bActive = true;
  this.propagateStatus();
}
QeaWorkItem.prototype.workStopped = function()
{
  ;
  if( this.oWorkItemBroker && ! this.bWorkStopped )
  {
    this.oWorkItemBroker.doWorkStopped();
  }
  this.excManager.workItemStopped(this);
  this.bActive = false;
  this.bWorkStopped = true;
  this.propagateStatus();
}
QeaWorkItem.prototype.isUncovered = function()
{
  ;
  var bIsUncovered = false;
  for( var id in this.aEvaluations )
  {
    this.aEvaluations[id].setUncovered(false);
  }
  for( var id in this.aQuestions )
  {
    this.aQuestions[id].setUncovered(false);
  }
  for( var id in this.aAnswers )
  {
    this.aAnswers[id].setUncovered(false);
  }
  for( var id in this.aEvaluations )
  {
    this.aEvaluations[id].checkUncovered();
  }
  var nUncoveredQuestions = 0;
  for( var id in this.aQuestions )
  {
    if( this.aQuestions[id].isUncovered() )
    {
      nUncoveredQuestions++;
    }
  }
  if( nUncoveredQuestions == this.nQuestions - 1 )
  {
    bIsUncovered = true;
    ;
  }
  if( !bIsUncovered )
  {
    var nUncoveredAnswers = 0;
    for( var id in this.aAnswers )
    {
      if( this.aAnswers[id].isUncovered() )
      {
        nUncoveredAnswers++;
      }
    }
    if( nUncoveredAnswers == this.nAnswers - 1 )
    {
      bIsUncovered = true;
      ;
    }
  }
  if( !bIsUncovered )
  {
    var nCovered = 0;
    for( var id in this.aEvaluations )
    {
      if( this.aEvaluations[id].isUncovered() )
      {
        ;
      }
      else
      {
        nCovered++;
        ;
      }
    }
    ;
    bIsUncovered = nCovered == 0;
  }
  return bIsUncovered;
}
QeaWorkItem.prototype.showSolution = function()
{
  ;
  this.bIsShowSolutionActive = true;
  this.setUndoBreak('system_solution_sentinel');
  this.bNotifyManagers = false;
  var aInactiveEvaluations = new Array();
  for( var id in this.aEvaluations )
  {
    var e = this.aEvaluations[id];
    e.bEvaluated = false;
    if( !e.isSolution() )
    {
      if( e.isActive() )
      {
        e.setActive(false, 'system_solution');
      }
      else
      {
        aInactiveEvaluations[id] = e;
      }
    }
  }
  for( var id in this.aEvaluations )
  {
    var e = this.aEvaluations[id];
    if( e.isSolution() )
    {
      e.setActive(true, 'system_solution');
    }
  }
  for( var id in aInactiveEvaluations )
  {
    var e = aInactiveEvaluations[id];
    e.setActive(false, 'system_solution');
  }
  this.changed();
}
QeaWorkItem.prototype.hideSolution = function()
{
  ;
  if( this.bIsShowSolutionActive )
  {
    this.bIsShowSolutionActive = false;
    this.bHasShownSolution = true;
  }
  this.bNotifyManagers = false;
  var bRestoreSystemContribution = false;
  var sentinel = this.undo(bRestoreSystemContribution, 'system_solution_sentinel');
  if( sentinel != 'system_solution_sentinel' )
  {
    ;
  }
  if( this.bEvaluated )
  {
    this.evaluate();
  }
  this.bNotifyManagers = true;
  this.changed();
}
QeaWorkItem.prototype.dumpStatus = function()
{
  ;
  ;
  for( var id in this.aEvaluations )
  {
    var e = this.aEvaluations[id];
    ;
  }
  this.isUncovered();
  var nCovered = 0;
  for( var id in this.aEvaluations )
  {
    if( this.aEvaluations[id].isUncovered() )
    {
      ;
    }
    else
    {
      nCovered++;
      ;
    }
  }
  ;
}
QeaWorkItem.prototype.noticed = function()
{
  if( !this.bActivated )
  {
    this.bActivated = true;
    if( this.oWorkItemBroker )
    {
      this.oWorkItemBroker.activated();
    }
    this.excManager.workItemActivated(this);
  }
}
QeaWorkItem.prototype.setNotifyView = function( bNotifyView )
{
  //;
  this.bNotifyView = bNotifyView;
}
QeaWorkItem.prototype.getNotifyView = function()
{
  //;
  return this.bNotifyView;
}
QeaWorkItem.prototype.setNotifyManagers = function( bNotifyManagers )
{
  //;
  this.bNotifyManagers = bNotifyManagers;
}
QeaWorkItem.prototype.getNotifyManagers = function()
{
  //;
  return this.bNotifyManagers;
}
QeaWorkItem.prototype.setCountDynamicAnswersWrong = function( bCountDynamicAnswersWrong )
{
  //;
  this.bCountDynamicAnswersWrong = bCountDynamicAnswersWrong;
}
QeaWorkItem.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, "reset");
}
QeaWorkItem.prototype.getEvaluateAction = function()
{
  return new ObjectMethodAction(this, "evaluate");
}
QeaWorkItem.prototype.getShowSolutionAction = function()
{
  return new ObjectMethodAction(this, "showSolution");
}
QeaWorkItem.prototype.getHideSolutionAction = function()
{
  return new ObjectMethodAction(this, "hideSolution");
}
QeaWorkItem.prototype.getNoticedAction = function()
{
  return new ObjectMethodAction(this, "noticed");
}
function QeaUndoItem( sId, evaluation )
{
  if( sId )
  {
    this.init(sId, evaluation);
  }
}
QeaUndoItem.prototype.init = function( sId, evaluation )
{
  ;
  this.sId = sId;
  this.evaluation = evaluation;
  if( evaluation )
  {
    this.bActive = evaluation.isActive();
    this.sOrigin = evaluation.getOrigin();
    this.bSystemConstructive = evaluation.hasSystemConstuction();
    this.bSystemCorrective = evaluation.hasSystemCorrection();
    this.bSystemPartialConstructive = evaluation.hasSystemPartialConstuction();
    this.bSystemPartialCorrective = evaluation.hasSystemPartialCorrection();
  }
  else
  {
    this.bActive = false;
    this.sOrigin = '';
    this.bSystemConstructive = false;
    this.bSystemCorrective = false;
    this.bSystemPartialConstructive = false;
    this.bSystemPartialCorrective = false;
    this.bSystemPartialConstructive = false;
    this.bSystemPartialCorrective = false;
  }
}
QeaUndoItem.prototype.getId = function()
{
  return this.sId;
}
function QeaUndoStack( sId )
{
  if( sId )
  {
    this.init(sId);
  }
}
QeaUndoStack.prototype.init = function( sId )
{
  ;
  this.sId = sId;
  this.bLock = false;
  this.aUndoItems = new Array();
}
QeaUndoStack.prototype.getId = function()
{
  return this.sId;
}
QeaUndoStack.prototype.hasUndoSentinel = function( sTag )
{
  ;
  var bFound = false;
  for( var i = 0; i < this.aUndoItems.length; i++ )
  {
    if( sTag == this.aUndoItems[i].getId() )
    {
      bFound = true;
      break;
    }
  }
  return bFound;
}
QeaUndoStack.prototype.push = function( undoItem )
{
  ;
  if( !this.bLock )
  {
    this.aUndoItems.push(undoItem);
  }
}
QeaUndoStack.prototype.pop = function()
{
  ;
  return this.aUndoItems.pop();
}
QeaUndoStack.prototype.flush = function()
{
  ;
  if( !this.bLock )
  {
    this.aUndoItems = new Array();
  }
}
QeaUndoStack.prototype.lengh = function()
{
  //;
  return this.aUndoItems.length;
}
QeaUndoStack.prototype.isLocked = function()
{
  //;
  return this.bLock;
}
QeaUndoStack.prototype.lock = function()
{
  ;
  this.bLock = true;
}
QeaUndoStack.prototype.unlock = function()
{
  ;
  this.bLock = false;
}
QeaUndoStack.prototype.setLock = function( bLock )
{
  ;
  this.bLock = bLock;
}

/* toolkit/fake/FakeWorkItem.js */
/*
<DEPENDENCIES>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Question</DEPENDS>
    <DEPENDS>Answer</DEPENDS>
    <DEPENDS>Evaluation</DEPENDS>
    <DEPENDS>QeaWorkItem</DEPENDS>
</DEPENDENCIES>
*/
FakeWorkItem.prototype             = new QeaWorkItem();
FakeWorkItem.prototype.constructor = FakeWorkItem;
FakeWorkItem.superclass            = QeaWorkItem.prototype;
function FakeWorkItem(sId) 
{
  if (sId) { this.init(sId); }
}
FakeWorkItem.prototype.init = function(sId)
{
  ;
  FakeWorkItem.superclass.init.call(this, sId);
  this.singleEvaluation = null;
  var question = new Question(this.sId + '_singleQuestion', 1);
  var answer = new Answer(this.sId + '_singleAnswer', 1);
  this.singleEvaluation = new Evaluation(this.sId + '_singleEvaluation', question, answer, true, true);
  this.registerQuestion(question);
  this.registerAnswer(answer);
  this.registerEvaluation(this.singleEvaluation);
}
FakeWorkItem.prototype.getSingleEvaluation = function()
{
  return this.singleEvaluation;
}

/* common/widgets/TextPushButton.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>DomletCtr</DEPENDS>
    <DEPENDS>Domlet</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
TextPushButton.prototype = new DomletCtr();
TextPushButton.prototype.constructor = TextPushButton;
TextPushButton.superclass = DomletCtr.prototype;
function TextPushButton( domId, enabled, name, parent )
{
  ;
  if( domId ) 
  {
    this.init(domId, enabled, name, parent);
  }
}
TextPushButton.prototype.init = function( domId, enabled, name, parent )
{
  TextPushButton.superclass.init.call(this, domId, name, parent);
  ;
  this.addDomlet(new Domlet(
          this.domId, 
          this.name, 
          this.parent, 
          new Array(
          'mouseover',
          'mouseout',
          'mousedown',
          'mouseup',
          'click'
          )
          ));
  this.state = {
    down : false,          
    highlighted : false   
  };
  this.pressedFgndColor = null;
  this.pressedBgndColor = null;
  this.unpressedFgndColor = null;
  this.unpressedBgndColor = null;
  this.disabledFgndColor = null;
  this.disabledBgndColor = null;
  this.mouseoverFgndColor = null;
  this.mouseoverBgndColor = null;
  this.activeCursor = this.domlets[this.name].getStyle('cursor');
  this.inactiveCursor = this.activeCursor;
  this.state.enabled = ((enabled == undefined) ? true : enabled);
  ;
} 
TextPushButton.prototype.getEnableAction = function()
{
  return new ObjectMethodAction(this, "setEnabled", true);
}
TextPushButton.prototype.getDisableAction = function()
{
  return new ObjectMethodAction(this, "setEnabled", false);
}
TextPushButton.prototype.setPressedAction = function( action )
{
  ;
  this.pressedAction = action;
  ;
}
TextPushButton.prototype.setDownAction = function( action )
{
  ;
  this.downAction = action;
}
TextPushButton.prototype.setUpAction = function( action )
{
  ;
  this.upAction = action;
}
TextPushButton.prototype.setEnabled = function( enabled )
{
  this.state.enabled = enabled;
  if( !enabled )
  {
    this.state.highlighted = false;
    this.state.down = false;
  }
  this.displayState();
}
TextPushButton.prototype.setPressedColor = function( color )
{
  this.pressedFgndColor = color;
}
TextPushButton.prototype.setPressedBgndColor = function( color )
{
  this.pressedBgndColor = color;
}
TextPushButton.prototype.setUnpressedColor = function( color )
{
  this.unpressedFgndColor = color;
}
TextPushButton.prototype.setUnpressedBgndColor = function( color )
{
  this.unpressedBgndColor = color;
}
TextPushButton.prototype.setDisabledColor = function( color )
{
  this.disabledFgndColor = color;
}
TextPushButton.prototype.setDisabledBgndColor = function( color )
{
  this.disabledBgndColor = color;
}
TextPushButton.prototype.setMouseoverColor = function( color )
{
  this.mouseoverFgndColor = color;
}
TextPushButton.prototype.setMouseoverBgndColor = function( color )
{
  this.mouseoverBgndColor = color;
}
TextPushButton.prototype.eventFromChild = function( childName, eventType )
{
  ;
  if( this.state.enabled )
  {
    switch( eventType )
            {
      case "click" :
      {
        if( this.buttonGroupManager )
          this.buttonGroupManager.buttonPushed(this);
        if( this.pressedAction )
        {
          ;
          this.displayState();
          this.pressedAction.invoke();
          break;
        }
      }
      case 'mouseover' :
      {
        this.state.highlighted = true;
        this.displayState();
        break;
      }
      case 'mouseout' :
      {
        this.state.highlighted = false;
        if( this.state.down == true )
        {
          this.state.down = false;
          if( this.upAction )
          {
            ;
            this.upAction.invoke();
          }
        }
        this.displayState();
        break;
      }
      case 'mousedown' :
      {
        this.state.down = true;
        this.displayState();
        if( this.downAction )
        {
          ;
          this.downAction.invoke();
        }
        break;
      }
      case 'mouseup' :
      {
        this.state.down = false;
        this.displayState();
        if( this.upAction )
        {
          ;
          this.upAction.invoke();
        }
        break;
      }
    }
  }
} 
TextPushButton.prototype.displayState = function()
{
  if( this.state.enabled )
  {
    this.domlets[this.name].setStyle('cursor', this.activeCursor);
    if( this.state.highlighted && !this.state.down )
    {
      this.setFgColor(this.mouseoverFgndColor);
      this.setBgColor(this.mouseoverBgndColor);
    }
    else
    {
      if( this.state.down )
      {
        this.setFgColor(this.pressedFgndColor);
        this.setBgColor(this.pressedBgndColor);
      }
      else
      {
        this.setFgColor(this.unpressedFgndColor);
        this.setBgColor(this.unpressedBgndColor);
      }
    }
  }
  else
  {
    this.domlets[this.name].setStyle('cursor', this.inactiveCursor);
    this.setFgColor(this.disabledFgndColor);
    this.setBgColor(this.disabledBgndColor);
  }
}
TextPushButton.prototype.setActiveCursor = function( activeCursor )
{
  this.activeCursor = activeCursor;
}
TextPushButton.prototype.setInactiveCursor = function( inactiveCursor )
{
  this.inactiveCursor = inactiveCursor;
}
TextPushButton.prototype.setFgColor = function( color )
{
  if( color )
    Browser.changeStyleById(this.domId, "color", color);
}
TextPushButton.prototype.setBgColor = function( color )
{
  if( color )
    Browser.changeStyleById(this.domId, "backgroundColor", color);
}
TextPushButton.prototype.start = function()
{
  this.displayState();
}
TextPushButton.prototype.setButtonGroupManager = function( buttonGroupManager )
{
  this.buttonGroupManager = buttonGroupManager;
}

/* toolkit/exercise/ComprehensionManager.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
function ComprehensionManager(sId)
{
    this.init(sId)
}
ComprehensionManager.prototype.init = function(sId)
{
  ;
  this.sId = sId;
  this.aContents = new Array();
  this.oComprehensionBroker = null;
  this.bCompletelyConsumed = false;
  this.reset();
}
ComprehensionManager.prototype.setComprehensionBroker = function(comprehensionBroker)
{
  this.oComprehensionBroker = comprehensionBroker;
}
ComprehensionManager.prototype.registerContent = function(contentId)
{
  this.aContents[contentId] = false;
  ;
  return new ObjectMethodAction(this, 'contentConsumed', contentId);
}
ComprehensionManager.prototype.contentConsumed = function(contentId)
{
  this.aContents[contentId] = true;
  ;
  this.checkCompletelyConsumed();
}
ComprehensionManager.prototype.checkCompletelyConsumed = function()
{
  if(!this.bCompletelyConsumed)
  {
    for(var id in this.aContents)
    {
      if(this.aContents[id])
      {
        this.bCompletelyConsumed = true;
      }
    }
    if(this.bCompletelyConsumed && this.oComprehensionBroker)
    {
      this.oComprehensionBroker.doContentConsumed();
    }
  }
}
ComprehensionManager.prototype.reset = function()
{
  this.bCompletelyConsumed = false;
  for(var id in this.aContents)
  {
    this.aContents[id] = false;
  }
}
ComprehensionManager.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, 'reset');
}

/* toolkit/pager/VisibilityBroker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
</DEPENDENCIES>
*/
VisibilityBroker.prototype             = new Broker();
VisibilityBroker.prototype.constructor = VisibilityBroker;
VisibilityBroker.superclass            = Broker.prototype;
function VisibilityBroker(sId)
{
  if (sId) { this.init(sId); }
}
VisibilityBroker.prototype.init = function(sId)
{
  ;
  this.sId = sId;
  this.visibilityController = null;
  VisibilityBroker.superclass.init.call(this, sId);
}
VisibilityBroker.prototype.setPagingController = function(visibilityController)
{
  this.visibilityController = visibilityController;
}
VisibilityBroker.prototype.doActivity = function()
{
  this.getMessage('activity');
}
VisibilityBroker.prototype.addActivityHandle = function(action)
{
  this.registerAction('activity', action);
}
VisibilityBroker.prototype.getActivityAction = function(action)
{
  return new ObjectMethodAction(this, 'activity');
}
VisibilityBroker.prototype.activity = function()
{
  ;
  if (this.visibilityController)
    this.visibilityController.shouldActivity();
  else
    this.doActivity();
}
VisibilityBroker.prototype.doShow = function()
{
  this.getMessage('show');
}
VisibilityBroker.prototype.addShowHandle = function(action)
{
  this.registerAction('show', action);
}
VisibilityBroker.prototype.getShowAction = function(action)
{
  return new ObjectMethodAction(this, 'show');
}
VisibilityBroker.prototype.show = function()
{
  ;
  if (this.visibilityController)
    this.visibilityController.shouldShow();
  else
    this.doShow();
}
VisibilityBroker.prototype.doHide = function()
{
  this.getMessage('hide');
}
VisibilityBroker.prototype.addHideHandle = function(action)
{
  this.registerAction('hide', action);
}
VisibilityBroker.prototype.getHideAction = function(action)
{
  return new ObjectMethodAction(this, 'hide');
}
VisibilityBroker.prototype.hide = function()
{
  ;
  if (this.visibilityController)
    this.visibilityController.shouldHide();
  else
    this.doHide();
}
VisibilityBroker.prototype.doEvaluatesWrong = function()
{
  this.getMessage('evaluatesWrong');
}
VisibilityBroker.prototype.addEvaluatesWrongHandle = function(action)
{
  this.registerAction('evaluatesWrong', action);
}
VisibilityBroker.prototype.getEvaluatesWrongAction = function(action)
{
  return new ObjectMethodAction(this, 'evaluatesWrong');
}
VisibilityBroker.prototype.evaluatesWrong = function()
{
  ;
  if (this.visibilityController)
    this.visibilityController.shouldEvaluatesWrong();
  else
    this.doEvaluatesWrong();
}

/* toolkit/info/InformationVisibilityManager.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>VisibilityBroker</DEPENDS>
</DEPENDENCIES>
*/
function InformationVisibilityManager(sId)
{
  if (sId) { this.init(sId); }
}
InformationVisibilityManager.prototype.init = function(sId)
{
  this.sId = sId;
  this.nBrokers = 0;
  this.brokers = new Array();
  this.registeredInformation = new Array();
}
InformationVisibilityManager.prototype.getShowAction = function(usage)
{
  return this.findBroker(usage).getShowAction();
}
InformationVisibilityManager.prototype.registerShowHandle = function(usage, handle)
{
  this.findBroker(usage).addShowHandle(handle);
}
InformationVisibilityManager.prototype.registerInformationVisibility = function(usage, visibilityBroker)
{
  this.findBroker(usage).addShowHandle(visibilityBroker.getShowAction());
  this.findBroker(usage).addHideHandle(visibilityBroker.getHideAction());
  this.registeredInformation[usage] = true;
}
InformationVisibilityManager.prototype.getHideAction = function(usage)
{
  return this.findBroker(usage).getHideAction();
}
InformationVisibilityManager.prototype.registerHideHandle = function(usage, handle)
{
  this.findBroker(usage).addHideHandle(handle);
}
InformationVisibilityManager.prototype.getActivateAction = function(usage)
{
  return this.findBroker(usage).getActivityAction();
}
InformationVisibilityManager.prototype.registerActivateHandle = function(usage, handle)
{
  this.findBroker(usage).addActivityHandle(handle);
}
InformationVisibilityManager.prototype.findBroker = function(usage)
{
  var broker;
  if(this.brokers[usage])
  {
    broker = this.brokers[usage];
  }
  else
  {
    broker = new VisibilityBroker(this.createBrokerName());
    this.brokers[usage] = broker;
  }
  return broker;
}
InformationVisibilityManager.prototype.activateAll = function()
{
  for(var usage in this.registeredInformation)
  {
    if(this.registeredInformation[usage])
    {
      this.findBroker(usage).activity();
    }
  }
}
InformationVisibilityManager.prototype.createBrokerName = function()
{
  this.nBrokers++;
  return 'infoVisibilityBroker_' + this.nBrokers;
}
InformationVisibilityManager.prototype.start = function()
{
  this.activateAll();
}

/* toolkit/toggle/ToggleManagerStatic.js */
 /*
 <DEPENDENCIES>
   <DEPENDS>Logger</DEPENDS>
   <DEPENDS>ScreenBroker</DEPENDS>
   <DEPENDS>ServiceRegistryAccessor</DEPENDS>
 </DEPENDENCIES>
 */
 function ToggleManagerStatic(sId, propertyKey)
 {
 if (sId) { this.init(sId, propertyKey); }
 }
 ToggleManagerStatic.prototype.init = function(sId, propertyKey)
 {
	this.sId = sId;
	this.propertyKey = propertyKey;
	this.initService();
	if (this.propertyservice && this.propertyservice.getPropertyValue(propertyKey))
	{
		;
		this.toogleOn = this.propertyservice.getPropertyValue(propertyKey);
	}
	else
	{
		this.toogleOn = false
	}
 }
 ToggleManagerStatic.prototype.setScreenBroker = function(broker)
 {
	;
	this.broker = broker;
 }
 ToggleManagerStatic.prototype.registerToggleBroker = function(toggleBroker)
 {
	;
	this.toggleBroker = toggleBroker;
 }
 ToggleManagerStatic.prototype.start = function()
 {
	 ;
	 if (this.toggleBroker)
	 {
		if (this.toogleOn)
		{
			this.toggleBroker.doIsToggleOn();
		}
		else
		{
			this.toggleBroker.doIsToggleOff();
		}
	 }
 }
 ToggleManagerStatic.prototype.getToggleOnAction = function()
 {
	return new ObjectMethodAction(this, 'setToggleOn');
 }
 ToggleManagerStatic.prototype.getToggleOffAction = function()
 {
	return new ObjectMethodAction(this, 'setToggleOff');
 }
 ToggleManagerStatic.prototype.setToggleOn = function()
 {
	if (this.propertyservice)
		this.propertyservice.setPropertyValue(this.propertyKey, true);
	this.broker.getReloadAction().invoke();
 }
 ToggleManagerStatic.prototype.setToggleOff = function()
 {
	if (this.propertyservice)
		this.propertyservice.setPropertyValue(this.propertyKey, false);
	this.broker.getReloadAction().invoke();
 }
 ToggleManagerStatic.prototype.initService = function()
 {
 	var serviceregistry = ServiceRegistryAccess.getServiceRegistry();
 	if (serviceregistry)
   		this.propertyservice = serviceregistry.getService("propertyservice");
 }

/* toolkit/screen/ScreenController.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>FinalObjectMethodAction</DEPENDS>
</DEPENDENCIES>
*/
function ScreenController(sId, contentId, popupAllowed, popup)
{
    this.init(sId, contentId, popupAllowed, popup)
}
ScreenController.prototype.init = function(sId, contentId, popupAllowed, popup)
{
  ;
  this.sId = sId;
  this.contentId = contentId;
  this.screenBroker = null;
  this.popupAllowed = popupAllowed;
  this.popup = popup;
  this.openerFrame = false;
}
ScreenController.prototype.start = function()
{
  ;
  if(this.popup)
    this.startPopupAware();
  if(this.screenBroker)
  {
    Browser.addUnloadHandle(new ObjectMethodAction(this.screenBroker, 'doUnload'));
    this.screenBroker.addReplaceHandle(new FinalObjectMethodAction(this, "doTheReplace"));
    this.screenBroker.addReloadHandle(new FinalObjectMethodAction(this, "doTheReload"));
    this.screenBroker.addLogoutHandle(new FinalObjectMethodAction(this, "doTheLogout"));
  }
}
ScreenController.prototype.startPopupAware = function()
{
  if (opener)
  {
    if (opener.top.frames.length > 1)
    {
      this.openerFrame = true;
    }
  }
  if (opener && !this.openerFrame)
  {
      if (opener.screenBroker)
      {
        opener.screenBroker.notifyPopupAware(this.screenBroker);
      }
  }
  else
  {
    if (opener)
    {
      if (this.openerFrame && opener.top.frames.length < 2)
      {
	    if (Browser.getId() == 'moz')
	    {
	      top.close();
	    }
	    else
	    {
	      self.close();
	    }
      }
      else if (opener.screenBroker)
      {
        opener.screenBroker.notifyPopupAware(this.screenBroker);
      }
    }
    else if (this.openerFrame)
    {
      if (Browser.getId() == 'moz')
      {
        top.close();
      }
      else
      {
        self.close();
      }
    }
  }
  var thisScreenController = this;
  var callback = function()
  {
     thisScreenController.startPopupAware();
  };
  window.setTimeout(callback, 1000);
}
if (openedWindows == undefined) {
    ;
    var openedWindows = new Object();
}
ScreenController.prototype.register = function(screenBroker)
{
    this.screenBroker = screenBroker;
    this.screenBroker.setScreenController(this);
}
ScreenController.prototype.shouldReplace = function(extURL)
{
  this.screenBroker.doLeave();
  this.screenBroker.doReplace(extURL);
}
ScreenController.prototype.shouldReload = function()
{
  this.screenBroker.doLeave();
  this.screenBroker.doReload();
}
ScreenController.prototype.shouldPopup = function(extURL, windowName, width, height, scrollbars, resizable)
{
    this.doThePopup(extURL, windowName, width, height, scrollbars, resizable);
}
ScreenController.prototype.shouldIFrame = function(extURL, windowName)
{
    this.doTheIFrame(extURL, windowName);
}
ScreenController.prototype.shouldInvokeServletMethod = function(extURL, successAction, errorAction)
{
    this.doTheInvokeServletMethod(extURL, successAction, errorAction);
}
ScreenController.prototype.shouldCallServer = function(extURL, successAction, errorAction)
{
    this.doTheCallServer(extURL, successAction, errorAction);
}
ScreenController.prototype.shouldClosePopup = function(windowName)
{
    this.doTheClosePopup(windowName);
}
ScreenController.prototype.shouldReloadPopup = function(extURL, windowName)
{
    this.doTheReloadPopup(extURL, windowName);
}
ScreenController.prototype.doTheReplace = function(extURL)
{
  ;
  try
  {
    window.location.href = extURL;
  }
  catch(exeption)
  {
  }
}
ScreenController.prototype.doTheReload = function()
{
  ;
  this.screenBroker.doLeave();
  window.location.reload();
}
ScreenController.prototype.doTheReloadPopup = function(extURL, windowName)
{
    var wind = window.open('', windowName);
    if(wind)
        wind.location.href = extURL;
    openedWindows[windowName] = wind;
}
ScreenController.prototype.doThePopup = function(extURL, windowName, width, height, scrollbars, resizable)
{
  ;
  if (extURL == null) extURL = "/error";
  if (width == null) width = 850;
  if (height == null) height = 650;
  if (windowName == null) windowName = 'scola-af' + new Date().getTime();
  windowName = windowName.replace (/\W/g, "");
  var wind = window.open('', windowName, 'width=' + width + ', height=' + height + ', menubar=no, scrollbars=' + scrollbars + ', toolbar=no, status=no, resizable=' + resizable);
  if (wind)
  {
    wind.location.href = extURL;
  	wind.focus();
  }
  ;
  openedWindows[windowName] = wind;
}
ScreenController.prototype.doTheIFrame = function(extURL, windowName)
{
  ;
  if (extURL == null) extURL = "/error";
  var width = 350;
  var height = 200;
  if (windowName == null) windowName = 'scola-af' + new Date().getTime();
  var wind = window.open('', windowName, 'width=' + width + ', height=' + height + ', menubar=no, scrollbars=no, toolbar=no, status=no, resizable=no');
  if(wind)
    wind.location.href = extURL;
}
ScreenController.prototype.doTheInvokeServletMethod = function(extURL, successAction, errorAction)
{
  ;
  Browser.callURL(extURL, successAction, errorAction);
}
ScreenController.prototype.doTheCallServer = function(extURL, successAction, errorAction)
{
  ;
  Browser.callServer(extURL, successAction, errorAction);
}
ScreenController.prototype.doTheClosePopup = function(windowName)
{
  ;
  ;
  if(openedWindows[windowName])
  {
    ;
    openedWindows[windowName].close();
  }
  else 
  {
    ;
    top.close();
  }
}
ScreenController.prototype.doNotifyPopupAware = function(screenBroker)
{
  ;
  if (!this.popupAllowed)
    screenBroker.closePopup(null);
}
ScreenController.prototype.shouldNotifyPopupAware = function(screenBroker)
{
    this.doNotifyPopupAware(screenBroker);
}
ScreenController.prototype.shouldLoaded = function()
{
    this.screenBroker.doLoaded();
}
ScreenController.prototype.shouldUnload = function()
{
    this.screenBroker.doUnload();
}
ScreenController.prototype.shouldOpenWindow = function(url, target)
{
    this.doTheOpenWindow(url, target);
}
ScreenController.prototype.doTheOpenWindow = function(url, target)
{
  if (target == null) target = '_blank';
  var wind = window.open('', target);
  if (wind)
  {
    wind.location.href = url;
  	wind.focus();
  }
}
ScreenController.prototype.shouldOpenerReplace = function(internalURL)
{
    this.doTheOpenerReplace(internalURL);
}
ScreenController.prototype.doTheOpenerReplace = function(internalURL)
{
    opener.location.href = internalURL;
}
ScreenController.prototype.shouldAddUserMarker = function(markerId)
{
    if (this.screenBroker.bookmarkable)
        this.doAddUserMarker(markerId);
}
ScreenController.prototype.shouldLogout = function()
{
  this.screenBroker.doLeave();
  this.screenBroker.doLogout();
}
ScreenController.prototype.doTheLogout = function()
{
  Browser.replaceRelative("../credits");
}
ScreenController.prototype.doAddUserMarker = function(markerId)
{
    this.doTheInvokeServletMethod("../multimarker?ACTION=add&MARKER_TYPE=user&MARKER_ID=" + encodeURIComponent(markerId) +
                        "&CONTENT=" + encodeURIComponent(this.contentId));
    this.screenBroker.doAddUserMarker(markerId);
}

/* toolkit/staticexport/ScoreStatistics.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
</DEPENDENCIES>
*/
function ScoreStatistics(fractionCorrect, fractionCorrectWithHelp, fractionWrong, fractionDone, fractionUndone) {
  	;
  	this.fractionCorrect = fractionCorrect;
  	this.fractionCorrectWithHelp = fractionCorrectWithHelp;
  	this.fractionWrong = fractionWrong;
  	this.fractionDone = fractionDone;
  	this.fractionUndone = fractionUndone;
}
ScoreStatistics.prototype.getFractionCorrect= function() {
	;
	return this.fractionCorrect;
}
ScoreStatistics.prototype.getFractionCorrectWithHelp= function() {
	;
	return this.fractionCorrectWithHelp;
}
ScoreStatistics.prototype.getFractionWrong= function() {
	;
	return this.fractionWrong;
}
ScoreStatistics.prototype.getFractionDone= function() {
	;
	return this.fractionDone;
}
ScoreStatistics.prototype.getFractionUndone= function() {
	;
	return this.fractionUndone;
}

/* toolkit/achievement/AchievementStatistic.js */
/*
 <DEPENDENCIES>
	 <DEPENDS>Browser</DEPENDS>
	 <DEPENDS>Logger</DEPENDS>
	 <DEPENDS>ServiceRegistryAccessor</DEPENDS>
 </DEPENDENCIES>
 */
function AchievementStatistic(structureName, contentidentifier, trace, scorestatistics)
{
	if (scorestatistics != undefined)
	{
   		this.scorestatistics = scorestatistics;
	}
	else
	{
		this.scorestatistics = null;
		this.trace = trace;
		this.init(structureName, contentidentifier);
	}
}
AchievementStatistic.prototype.init = function(structureName, contentidentifier)
{
	;
	var achievementservice = null;
	var serviceregistry = ServiceRegistryAccess.getServiceRegistry();
	if (serviceregistry)
		achievementservice = serviceregistry.getService("achievementservice");
	if (achievementservice)
	{
		if (this.trace)
			this.scorestatistics = achievementservice.getScoreStatisticsTrace(structureName, contentidentifier);
		else
			this.scorestatistics = achievementservice.getScoreStatisticsTrack(structureName, contentidentifier);
	}
} 
AchievementStatistic.prototype.getScoreStatistics = function()
{
	return this.scorestatistics;
}
AchievementStatistic.prototype.parseFormula = function(formula)
{
	if (!this.scorestatisticsvalues)
	{
		
		;
		this.scorestatisticsvalues = new Array();
		this.scorestatisticsvalues["\\[scorestatistics.fractionCorrect\\]"] = this.scorestatistics.getFractionCorrect();
		this.scorestatisticsvalues["\\[scorestatistics.fractionCorrectWithHelp\\]"] = this.scorestatistics.getFractionCorrectWithHelp();
		this.scorestatisticsvalues["\\[scorestatistics.fractionWrong\\]"] = this.scorestatistics.getFractionWrong();
		this.scorestatisticsvalues["\\[scorestatistics.fractionDone\\]"] = this.scorestatistics.getFractionDone();
		this.scorestatisticsvalues["\\[scorestatistics.fractionUndone\\]"] = this.scorestatistics.getFractionUndone();
	}
	for (var param in this.scorestatisticsvalues)
	{
		while (formula.search(param) > -1)
		{
			formula = formula.replace(new RegExp(param), this.scorestatisticsvalues[param]);
		}
	}
	return formula;
}

/* toolkit/achievement/AchievementManager.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>ScoreStatistics</DEPENDS>
    <DEPENDS>AchievementStatistic</DEPENDS>
</DEPENDENCIES>
*/
function ServerResultAction(scoreStatisticsAction)
{
	this.scoreStatisticsAction = scoreStatisticsAction;
}
ServerResultAction.prototype.invoke = function(resultMap)
{
	if (resultMap && this.scoreStatisticsAction)
		this.scoreStatisticsAction.invoke(
				new ScoreStatistics(
				resultMap['fractionCorrect'],
				resultMap['fractionCorrectWithHelp'],
				resultMap['fractionWrong'],
				resultMap['fractionDone'],
				resultMap['fractionUndone']));
}
function CommitedAction(broker, otherAction)
{
	;
	this.broker = broker;
	this.otherAction = otherAction;
}
CommitedAction.prototype.invoke = function()
{
	;
	if (this.otherAction)
	{
		this.otherAction.invoke();
	}
	if (this.broker)
	{
		this.broker.commited();
	}
}
AchievementManager.prototype = new Broker();
AchievementManager.prototype.constructor = AchievementManager;
AchievementManager.superclass = Broker.prototype;
function AchievementManager()
{
	this.init('AchievementManager');
}
AchievementManager.prototype.init = function(name)
{
	this.conditionHandles = new ConditionHandleMap();
	AchievementManager.superclass.init.call(this, name);
}
AchievementManager.prototype.start = function()
{
	var initialHandles = this.conditionHandles.getInitialHandleCallbacks();
	for (var i in initialHandles)
	{
		var callback = initialHandles[i];
		this.getScoreStatisticsTrace(callback, 'lrn', callback.getContentIdentifier());
	}
}
AchievementManager.prototype.getStartTestAction = function(cid)
{
	return new ObjectMethodAction(this, 'startTest', cid);
}
AchievementManager.prototype.startTest = function(cid)
{
	
}
AchievementManager.prototype.getFinishTestAction = function(cid, callback)
{
	return new ObjectMethodAction(this, 'finishTest', cid, callback);
}
AchievementManager.prototype.finishTest = function(cid, callback)
{
	
}
AchievementManager.prototype.addCommitedHandle = function(action)
{
	this.registerAction("commited", action);
}
AchievementManager.prototype.commited = function()
{
	this.getMessage("commited");
}
AchievementManager.prototype.addTestStartedHandle = function(action)
{
	this.registerAction("teststarted", action);
}
AchievementManager.prototype.addTestFinishedHandle = function(action)
{
	this.registerAction("testfinished", action);
}
AchievementManager.prototype.addContionHandle = function(action, cid, condition, initial, permanent)
{
	this.conditionHandles.addConditionHandle(new ConditionHandle(action, cid, condition, initial, permanent));
}
AchievementManagerDynamicEnvironment.prototype = new AchievementManager();
AchievementManagerDynamicEnvironment.prototype.constructor = AchievementManagerDynamicEnvironment;
AchievementManagerDynamicEnvironment.superclass = AchievementManager.prototype;
function AchievementManagerDynamicEnvironment()
{
	this.init();
}
AchievementManagerDynamicEnvironment.prototype.init = function()
{
	AchievementManagerDynamicEnvironment.superclass.init.call(this, 'AchievementManagerDynamicEnvironment');
}
AchievementManagerDynamicEnvironment.prototype.sendTrackResultData = function(callback, identifier,
																			  nTouched, nTotal, nCorrect, nWrong, nCorrectStatus,
																			  nWrongStatus, nCorrectSystem, nWrongSystem)
{
	;
	var URL = '../addachievement?IDENTIFIER=' + encodeURIComponent(identifier);
	URL += '&TOUCHED=' + nTouched;
	URL += '&TOTAL=' + nTotal;
	URL += '&CORRECT=' + nCorrect;
	URL += '&WRONG=' + nWrong;
	URL += '&CORRECTSTATUS=' + nCorrectStatus;
	URL += '&WRONGSTATUS=' + nWrongStatus;
	URL += '&CORRECTSYSTEM=' + nCorrectSystem;
	URL += '&WRONGSYSTEM=' + nWrongSystem;
	URL += '&EVALMODE=TRACK';
	var commitedAction = new CommitedAction(this, callback);
	Browser.loadDocument(URL, commitedAction, commitedAction);
}
AchievementManagerDynamicEnvironment.prototype.sendTraceResultData = function(callback, identifier,
																			  nTouched, nTotal, nCorrect, nWrong, nCorrectStatus,
																			  nWrongStatus, nCorrectSystem, nWrongSystem)
{
	;
	var URL = '../addachievement?IDENTIFIER=' + encodeURIComponent(identifier);
	URL += '&TOUCHED=' + nTouched;
	URL += '&TOTAL=' + nTotal;
	URL += '&CORRECT=' + nCorrect;
	URL += '&WRONG=' + nWrong;
	URL += '&CORRECTSTATUS=' + nCorrectStatus;
	URL += '&WRONGSTATUS=' + nWrongStatus;
	URL += '&CORRECTSYSTEM=' + nCorrectSystem;
	URL += '&WRONGSYSTEM=' + nWrongSystem;
	URL += '&EVALMODE=TRACE';
	var commitedAction = new CommitedAction(this, callback);
	Browser.loadDocument(URL, commitedAction, commitedAction);
}
AchievementManagerDynamicEnvironment.prototype.getScoreStatisticsTrace
		= function(callback, structureName, contentidentifier)
{
	;
	var URL = '../getachievement?IDENTIFIER=' + encodeURIComponent(contentidentifier);
	URL += '&STRUCTURE=lrn';
	URL += '&EVALMODE=TRACE';
	var serverResultAction = new ServerResultAction(callback);
	Browser.callServer(URL, serverResultAction, serverResultAction);
}
AchievementManagerDynamicEnvironment.prototype.getScoreStatisticsTrack
		= function(callback, structureName, contentidentifier)
{
	;
	var URL = '../getachievement?IDENTIFIER=' + encodeURIComponent(contentidentifier);
	URL += '&STRUCTURE=lrn';
	URL += '&EVALMODE=TRACK';
	var serverResultAction = new ServerResultAction(callback);
	Browser.callServer(URL, serverResultAction, serverResultAction);
}
AchievementManagerDynamicEnvironment.prototype.startTest = function(cid)
{
	this.getMessage("teststarted");
}
AchievementManagerDynamicEnvironment.prototype.finishTest = function(callback)
{
	this.getMessage("testfinished");
}
AchievementManagerStaticEnvironment.prototype = new AchievementManager();
AchievementManagerStaticEnvironment.prototype.constructor = AchievementManagerStaticEnvironment;
AchievementManagerStaticEnvironment.superclass = AchievementManager.prototype;
function AchievementManagerStaticEnvironment()
{
	this.init();
}
AchievementManagerStaticEnvironment.prototype.init = function()
{
	this.achievementservice = null;
	AchievementManagerStaticEnvironment.superclass.init.call(this, 'AchievementManagerStaticEnvironment');
}
AchievementManagerStaticEnvironment.prototype.start = function()
{
	if (this.asname)
	{
		var serviceregistry = ServiceRegistryAccess.getServiceRegistry();
		if (serviceregistry)
			this.achievementservice = serviceregistry.getService("achievementservice");
	}
	AchievementManagerStaticEnvironment.superclass.start.call(this);
}
AchievementManagerStaticEnvironment.prototype.setAchievementServiceName = function(asname)
{
	this.asname = asname;
}
AchievementManagerStaticEnvironment.prototype.sendTrackResultData = function(callback, identifier,
																			 nTouched, nTotal, nCorrect, nWrong, nCorrectStatus,
																			 nWrongStatus, nCorrectSystem, nWrongSystem)
{
	if (this.achievementservice)
		this.achievementservice.setTrackData(identifier, nTouched, nTotal, nCorrect,
				nWrong, nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem);
	var commitedAction = new CommitedAction(this, callback);
	commitedAction.invoke();
}
AchievementManagerStaticEnvironment.prototype.sendTraceResultData = function(callback, identifier,
																			 nTouched, nTotal, nCorrect, nWrong, nCorrectStatus,
																			 nWrongStatus, nCorrectSystem, nWrongSystem)
{
	if (this.achievementservice)
		this.achievementservice.setTraceData(identifier, nTouched, nTotal, nCorrect,
				nWrong, nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem);
	var commitedAction = new CommitedAction(this, callback);
	commitedAction.invoke();
}
AchievementManagerStaticEnvironment.prototype.getScoreStatisticsTrace
		= function(callback, structureName, contentidentifier)
{
	var scorestatistics = null;
	if (this.achievementservice)
	{
		scorestatistics = this.achievementservice.getScoreStatisticsTrace(structureName, contentidentifier);
	}
	if (callback)
		callback.invoke(scorestatistics);
}
AchievementManagerStaticEnvironment.prototype.getScoreStatisticsTrack
		= function(callback, structureName, contentidentifier)
{
	;
	var scorestatistics = null;
	if (this.achievementservice)
	{
		scorestatistics = this.achievementservice.getScoreStatisticsTrack(structureName, contentidentifier);
	}
	if (callback)
		callback.invoke(scorestatistics);
}
AchievementManagerStaticEnvironment.prototype.startTest = function(cid)
{
	if (this.achievementservice)
		this.achievementservice.startTest(cid);
	this.getMessage('teststarted');
}
AchievementManagerStaticEnvironment.prototype.finishTest = function(callback)
{
	if (this.achievementservice)
		this.achievementservice.finishTest();
	this.getMessage('testfinished');
	if (callback)
		callback.invoke();
}
function ConditionHandleMap()
{
	this.initialCallbacks = new Array();
	this.permanentCallbacks = new Array();
}
ConditionHandleMap.prototype.addConditionHandle = function(handle)
{
	if (handle.isInitial())
	{
		var initialCallback = this.initialCallbacks[handle.getContentIdentifier()];
		;
		if (initialCallback == null || initialCallback == undefined)
		{
			initialCallback = new ConditionHandleMapCallback(handle.getContentIdentifier());
			this.initialCallbacks[handle.getContentIdentifier()] = initialCallback;
		}
		initialCallback.addHandle(handle);
	}
	else if (handle.isPermanent())
	{
		var permanentCallback = this.permanentCallbacks[handle.getContentIdentifier()];
		if (permanentCallback == null || permanentCallback == undefined)
		{
			permanentCallback = new ConditionHandleMapCallback(handle.getContentIdentifier());
			this.permanentCallbacks[handle.getContentIdentifier()] = permanentCallback;
		}
		permanentCallback.addHandle(handle);
	}
}
ConditionHandleMap.prototype.getInitialHandleCallbacks = function()
{
	return this.initialCallbacks;
}
ConditionHandleMap.prototype.getPermanentHandleCallbacks = function()
{
	return this.permanentCallbacks;
}
function ConditionHandleMapCallback(cid)
{
	;
	this.cid = cid;
	this.handles = new Array();
}
ConditionHandleMapCallback.prototype.addHandle = function(handle)
{
	;
	this.handles.push(handle);
}
ConditionHandleMapCallback.prototype.getContentIdentifier = function()
{
	return this.cid;
}
ConditionHandleMapCallback.prototype.invoke = function(scorestatistics)
{
	;
	for (var i = 0; i < this.handles.length; i++)
	{
		var statistic = new AchievementStatistic(null, null, null, scorestatistics);
		var handle = this.handles[i];
		;
		if (eval(statistic.parseFormula(handle.getCondition())))
		{
			;
			handle.getAction().invoke();
		}
	}
}
function ConditionHandle(action, cid, condition, initial, permanent)
{
	this.action = action;
	this.cid = cid;
	this.condition = condition;
	this.initial = initial;
	this.permanent = permanent;
}
ConditionHandle.prototype.getAction = function()
{
	return this.action;
}
ConditionHandle.prototype.getContentIdentifier = function()
{
	return this.cid;
}
ConditionHandle.prototype.getCondition = function()
{
	return this.condition;
}
ConditionHandle.prototype.isInitial = function()
{
	return this.initial;
}
ConditionHandle.prototype.isPermanent = function()
{
	return this.permanent;
}

/* toolkit/lms/LMSDispatcherStatic.js */
/*
<DEPENDENCIES>
	<DEPENDS>Logger</DEPENDS>
	<DEPENDS>ServiceRegistryAccessor</DEPENDS>
</DEPENDENCIES>
*/
function LMSDispatcherStatic(sId)
{
  if (sId) { this.init(sId); }
}
LMSDispatcherStatic.prototype.init = function(sId)
{
	this.sId = sId;
	this.timeout = 0;   
	this.sendachievement = false;
	this.entryId = null;
}
LMSDispatcherStatic.prototype.getId = function()
{
  return this.sId;
}
LMSDispatcherStatic.prototype.start = function()
{
    var lmsconnector = null;
	var serviceregistry = ServiceRegistryAccess.getServiceRegistry();
	if (serviceregistry)
   		lmsconnector = serviceregistry.getService("lmsconnector");
	
	if (lmsconnector)
	{
		if (this.entryId || this.finished || this.sendachievement)
		{
			if (this.entryId)
				lmsconnector.setLastVisitedLocation(this.folderId, this.entryId);
			if (this.sendachievement)
				lmsconnector.retrieveAchievementData();
			if (this.timeout && this.timeout > 0)
				lmsconnector.notifyFinished(this.timeout);
			lmsconnector.sendData();
		}
		else
		{
			;
		}
	}
	else
	{
		;
	}
}
LMSDispatcherStatic.prototype.notifyAchievement = function()
{
	;
	this.sendachievement = true;
}
LMSDispatcherStatic.prototype.notifyFinish = function(timeout)
{
	;
	this.timeout = timeout;
}
LMSDispatcherStatic.prototype.notifyLocation = function(folderId, entryId)
{
	;
	this.entryId = entryId;
	this.folderId = folderId;
}

/* toolkit/exercise/ComprehensionBroker.js */
/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Broker</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
</DEPENDENCIES>
*/
ComprehensionBroker.prototype             = new Broker();
ComprehensionBroker.prototype.constructor = ComprehensionBroker;
ComprehensionBroker.superclass            = Broker.prototype;
function ComprehensionBroker()
{
    ;
    this.comprehensionController = null;
     ComprehensionBroker.superclass.init.call(this, "ComprehensionBroker");
}
ComprehensionBroker.prototype.setComprehensionController = function(comprehensionController)
{
    this.comprehensionController = comprehensionController;
}
ComprehensionBroker.prototype.doContentConsumed = function()
{
    ;
    this.getMessage("contentConsumed");
}
ComprehensionBroker.prototype.addContentConsumedHandle = function(action)
{
    this.registerAction("contentConsumed", action);  
}
ComprehensionBroker.prototype.getContentConsumedAction = function(action)
{
    return new ObjectMethodAction(this, "contentConsumed");
}
ComprehensionBroker.prototype.contentConsumed = function()
{
    if (this.comprehensionController)
        this.comprehensionController.shouldContentConsumed();
    else
        this.doContentConsumed();
}
